var gsBasePath = "";

function Geosetter(map)
{
  this.map = map;
  this.signals = this.map.getSignals();

  // create and enable marker layers
  this.markerLayer = new SMap.Layer.Marker();
  this.map.addLayer(this.markerLayer);
  this.markerLayer.enable();
  
  this.markerBlueLayer = new SMap.Layer.Marker();
  this.map.addLayer(this.markerBlueLayer);
  this.markerBlueLayer.enable();

  // track layer
  this.trackLayer = new SMap.Layer.Geometry("trackLayer");
  this.map.addLayer(this.trackLayer);
  this.trackLayer.enable();

  this.init();
  this.createDragMarker();
  this.markers = {};
  this.markersCoords = {};
  this.tracks = {};
}

Geosetter.prototype.init = function()
{
  var that = this;
  
  this.signals.addListener(window, "marker-drag-stop", function(e, elm)
  {
    var marker = e.target;
    
    if (marker.getId() == "dragMarker")
    {
      that.moveDragMarker(e.target.getCoords());
    }

    if (marker.getId().match(/blue2_/))
    {
      document.getElementById("current_marker").innerHTML = marker.AID;
    }
    
    if (marker.getId().match(/blue3_/) || marker.getId() == "focused")
    {
      var coords = marker.getCoords();
      document.getElementById("current_marker").innerHTML = marker.AID;
      document.getElementById("current_marker_lng").innerHTML = coords.lon();
      document.getElementById("current_marker_lat").innerHTML = coords.lat();
      
      // reposition blue marker on Google Maps
      if (marker.getId().match(/blue3_/))
      {
        for (var i = 0; i < Markers.length; i++)
        {
          if (Markers[i].ID == marker.AID) 
          {
            Markers[i].Marker.setPoint(new GLatLng(coords.lat(), coords.lon()));
          }
        }       
      }
    }
    
    

  });
  
  this.signals.addListener(window, "map-click", function(e, elm)
  {
    // move drag marker on map click (but not on CTRL, which is for clicking the map within the track mode)
    if (!e.data.event.ctrlKey)
    {
      that.moveDragMarker(new SMap.Coords.fromEvent(e.data.event, that.map));
      that.hideShowDragMarker(true);
    }
  });
  
  this.signals.addListener(window, "marker-click", function(e)
  {
    var marker = e.target;
    
    if (marker.getId().match(/blue3_/))
    {
      document.getElementById("current_marker").innerHTML = marker.AID;
    }
  });
  
  this.signals.addListener(window, "map-redraw", function(e, elm)
  {
  });    
}

Geosetter.prototype.createDragMarker = function()
{
  var options = {url: JAK.mel("img", {className: "gsDragMarker", src: gsBasePath + "img/marker_red.png"})};
  this.dragMarker = new SMap.Marker(new SMap.Pixel(0,0).toCoords(this.map), "dragMarker", options);
  this.dragMarker.decorate(SMap.Marker.Feature.Draggable);
  this.markerLayer.addMarker(this.dragMarker);
}

Geosetter.prototype.moveDragMarker = function(coords, AScrollInView)
{
  this.dragMarker.setCoords(coords);
  document.getElementById("drag_mrk_lng").innerHTML = coords.lon();
  document.getElementById("drag_mrk_lat").innerHTML = coords.lat();
  
  if (AScrollInView)
    this.map.setCenter(coords);
    
  setDragMarkerToPositionOrig(coords.lat(), coords.lon(), false);
}

Geosetter.prototype.hideShowDragMarker = function(AVisible)
{
  this.dragMarker.getContainer()[SMap.LAYER_MARKER].style.display = AVisible ? "" : "none";
  var lon = document.getElementById("drag_mrk_lng").innerHTML;
  var lat = document.getElementById("drag_mrk_lat").innerHTML;
  this.dragMarker.setCoords(SMap.Coords.fromWGS(lat, lon));
}

Geosetter.prototype.createMarkerBlue = function(ALatitude, ALongitude, ADraggable, AID)
{//log("create blue - " + AID);
  var coords = new SMap.Coords.fromWGS(ALatitude, ALongitude);
  var options = {url: gsBasePath + "img/marker_blue.png"};
  var marker = new SMap.Marker(coords, (ADraggable ? "blue3_" : "blue2_") + AID, options);

  if (ADraggable)
    marker.decorate(SMap.Marker.Feature.Draggable);

  marker.AID = AID;
  this.markers[AID] = marker;
  this.markersCoords[AID] = {coords: coords, draggable: ADraggable};
  //log("added: " + AID);
  
  this.markerBlueLayer.addMarker(marker);
}

Geosetter.prototype.setMarkerFocused = function(ALatitude, ALongitude, AID, ACenter)
{
  var coords = new SMap.Coords.fromWGS(ALatitude, ALongitude);
  
  if (this.markerFocused)
  {
    this.markerLayer.removeMarker(this.markerFocused);    
  }
  
  var options = {url: JAK.mel("img", {className: "gsFocusedMarker", src: gsBasePath + "img/marker_focused.png"})};
  this.markerFocused = new SMap.Marker(coords, "focused", options);
  this.markerFocused.decorate(SMap.Marker.Feature.Draggable);
  
  this.markerLayer.addMarker(this.markerFocused);
  
  // set AID
  this.markerFocused.AID = AID;

  // center
  if(ACenter)
    this.map.setCenter(coords);
    
  // reposition blue marker
  if (AID && this.markers[AID])
    this.markers[AID].setCoords(coords);
}

// Geosetter.prototype.deleteMarkerByAID = function(AID)
// {
//   if (!this.markers[AID])
//     return;
// 
//   this.markerBlueLayer.removeMarker(this.markers[AID]);
//   //var node = this.markers[AID].getContainer()[SMap.LAYER_MARKER];
//   //node.parentNode.removeChild(node);
//   //delete this.markers[AID];
//   log("removed: " + AID);
// }

var gs = new Geosetter(m);

//****************************

/* deal with red (drag) marker */
encapsulate(window, "setDragMarkerToPosition", function(ALatitude, ALongitude, AScrollInView)
{
  if (ALatitude > 999 || ALongitude > 999)
  {
    gs.hideShowDragMarker(false);
    return;
  }

  var coords = new SMap.Coords.fromWGS(ALatitude, ALongitude);
  gs.moveDragMarker(coords, AScrollInView);
  setDragMarkerToPositionOrig.apply(this, arguments);
  
  gs.hideShowDragMarker(true);
});

encapsulate(window, "setDragMarkerVisible", function(AVisible)
{
  log(AVisible);
  setDragMarkerVisibleOrig.apply(this, arguments);
  gs.hideShowDragMarker(AVisible);
});

encapsulate(window, "gotoDragMarker", function(AVisible)
{
  gotoDragMarkerOrig.apply(this, arguments);
  gs.hideShowDragMarker(true);
  
  var center = gDragMarker.getPoint();
  m.setCenter(SMap.Coords.fromGoogle(center));
});

/* zooming */
encapsulate(window, "zoomBounds", function(ABounds, AMaxZoomLevel)
{
  zoomBoundsOrig.apply(this, arguments);

  var ne = SMap.Coords.fromGoogle(ABounds.getNorthEast());
  var sw = SMap.Coords.fromGoogle(ABounds.getSouthWest());
  //sw = SMap.Coords.fromWGS(sw.lat() + 0.000001, sw.lon());
  m.setCenterZoom.apply(m, m.computeCenterZoom([ne, sw]));
  
  //m.setCenter(SMap.Coords.fromGoogle(ABounds.getCenter())); 
});

/* blue / violet markers */
encapsulate(window, "addMarker2", function(ALatitude, ALongitude, AID, ACaption, AThumbnailFile, ADate, AFocalLength, AAperture, AISO, AExposureTime, ACenter)
{                    
  gs.createMarkerBlue(ALatitude, ALongitude, false, AID);
  addMarker2Orig.apply(this, arguments);
});

encapsulate(window, "addMarker3", function(ALatitude, ALongitude, AID, ADirLat1, ADirLon1, ADirLat2, ADirLon2)
{
  // marker already exists? then delete it (need to implement actual deletion)
  if (gs.markers[AID])
  {
    var coords = new SMap.Coords.fromWGS(ALatitude, ALongitude);
    gs.markersCoords[AID] = coords;
    gs.markers[AID].setCoords(coords);
    addMarker3Orig.apply(this, arguments);
    return;
  }

  gs.createMarkerBlue(ALatitude, ALongitude, true, AID);
  addMarker3Orig.apply(this, arguments);
});

encapsulate(window, "showFocusMarker", function(ALatitude, ALongitude, AID, ACenter, APOVLat, APOVLon, APOVLat1, APOVLon1, APOVLat2, APOVLon2, AIsNewPOV)
{
  gs.setMarkerFocused(ALatitude, ALongitude, AID, ACenter);
  showFocusMarkerOrig.apply(this, arguments);
});

// switch to original map on clicking ";"
EventUtil.addHandler(document.documentElement, "keydown", function (e, target)
{
  if (contains_class(document.body, "lightbox_on"))
    return;

  var key = String.fromCharCode(e.keyCode).toUpperCase();
  
  if (e.keyCode == 192 && !e.shiftKey)
  {
    toggle_class(document.body, "geosetter_original_map");

    // position Google Maps
    if (contains_class(document.body, "geosetter_original_map"))
    {
       map.setCenter(new google.maps.LatLng(m.getCenter().lat(), m.getCenter().lon()));
       var nearestZoom = m.getNearestZoom({layerTo: "GSatellite_ID"});
       map.setZoom(nearestZoom);
       google.maps.event.trigger(map, 'resize');
    }
    
    // position Mapy.cz
    else
    {
      m.setCenter(new SMap.Coords.fromGoogle(map.getCenter()));
      var nearestZoom = m.getNearestZoom({layerFrom: "GSatellite_ID", zoomFrom: map.getZoom()});
      m.setZoom(nearestZoom);
      m.syncPort();
    }
  }
});


Geosetter.prototype.createTrack = function(AID, AColor, AHint, AWidth, AOpacity)
{ 
  AWidth = (typeof AWidth !== 'undefined') ? AWidth : 10;
  AOpacity = (typeof AOpacity !== 'undefined') ? AOpacity : 1;

  // iterate over tracks
  for (var i = 0; i < Tracks.length; i++)
  {
    if (Tracks[i].ID == AID) 
    { 
      if (Tracks[i].Track == null)
      {
        // convert points
        var points = [];
        
        for (var j = 0; j < Tracks[i].Points.length; j++)
        {
          points.push(new SMap.Coords.fromGoogle(Tracks[i].Points[j]));
        }

        // create track
        var options = {color: AColor, width: AWidth, outlineWidth: 0, outlineColor: AColor, opacity: AOpacity};
        var track = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, null, points, options);
  
        
        this.trackLayer.addGeometry(track);
        this.tracks[AID] = track;
        track.trackOpacity = AOpacity;
      }
      
      return;
    }
  }
}

Geosetter.prototype.removeTrack = function(AID)
{
  var track = this.tracks[AID];
  
  if (track)
    this.trackLayer.removeGeometry(track);
}

Geosetter.prototype.showTrack = function(AID, AShow)
{
  var track = this.tracks[AID];

  if (track)
  {
    var opacity = (typeof track.trackOpacity !== 'undefined') ? track.trackOpacity : 1;
    track.setOptions({opacity: AShow ? opacity : 0});
  }
}

encapsulate(window, "addTrack", function(AID, AColor, AHint)
{
  gs.createTrack.apply(gs, arguments);
  return (addTrackOrig.apply(this, arguments));
});

encapsulate(window, "addTrack2", function(AID, AColor, AHint, AWidth)
{
  gs.createTrack.apply(gs, arguments);
  return (addTrack2Orig.apply(this, arguments));
});

encapsulate(window, "addTrackWithOpacity", function(AID, AColor, AHint, AWidth, AOpacity)
{
  gs.createTrack.apply(gs, arguments);
  return (addTrackWithOpacityOrig.apply(this, arguments));
});

encapsulate(window, "removeTrack", function(AID)
{
  gs.removeTrack.apply(gs, arguments);
  return (removeTrackOrig.apply(this, arguments));
});

encapsulate(window, "showTrack", function(AID, AShow)
{
  gs.showTrack.apply(gs, arguments);
  return (showTrackOrig.apply(this, arguments));
});

// log_object(gs);
log("load_end");

