use strict;
use warnings;
use utf8;
use LWP::Simple;

my $hybridRegex = qr|lyrs=h@(\d+)|;
my $satRegexJS = qr|googleapis\.com/kh\?v=(\d+)|;
my $satRegexMy = qr|google\.com/kh/v=(\d+)|;

my $filename = "other_maps.js";

# get version from server
my $js = get('http://maps.googleapis.com/maps/api/js?sensor=false') || die "Couldn't download JS file!";

(my $hybridVerJS) = $js =~ $hybridRegex;
(my $satVerJS) = $js =~ $satRegexJS;

# get our version
my $myjs = read_file($filename);
my $myjs_mod = $myjs;

(my $hybridVerMy) = $myjs =~ $hybridRegex;
(my $satVerMy) = $myjs =~ $satRegexMy;

# compare and possibly replace
if ($hybridVerJS != $hybridVerMy)
{
  print "Changed hybrid: $hybridVerMy --> $hybridVerJS\n";
  $myjs_mod =~ s|$hybridRegex|lyrs=h\@$hybridVerJS|;
}

if ($satVerJS != $satVerMy)
{
  print "Changed satellite: $satVerMy --> $satVerJS\n";
  $myjs_mod =~ s|$satRegexMy|google.com/kh/v=$satVerJS|;
}

if ($myjs ne $myjs_mod)
{
  unlink "$filename.bak";
  rename $filename, "$filename.bak";
  save_file($filename, $myjs_mod);
}
else
{
  print "Nothing changed!\n";
}

################################################################################

# read_file (filename)
sub read_file
{
  open (FILE, "<:utf8", shift) or die $!;
  local $/;
  my $str = <FILE>;
  close FILE;
  $str;
}

# save_file (filename, content)
sub save_file
{
  open (MYFILE, ">:utf8", shift) or die $!;
  print MYFILE shift;
  close (MYFILE);
}