use strict;
use warnings;
use LWP;
use JSON;
use Encode;
use Win32API::File;
use utf8;

# download category lists
my @params =
(
  '__EVENTTARGET' => '',
  '__EVENTARGUMENT' => '',
  '__VIEWSTATE' => '',  
  'ctl00$ContentBody$ddlListTypes' => '1',
  'ctl00$ContentBody$btnSubmit' => 'go',
);

my $alphabetical = download("http://www.waymarking.com/cat/categorydirectory.aspx?f=1", "", \@params);
# my $hierarchical = download("http://www.waymarking.com/cat/categorydirectory.aspx?f=1", "categories_hierarchical.htm");

# load saved categories
my $cat_filename = "Categories.txt";
my %categories = %{from_json(read_file($cat_filename), {})} if -e $cat_filename;

# extract category names (and their links)
my %cat_names = reverse ($alphabetical =~ m|(\Qhttp://www.waymarking.com/cat/details.aspx?f=1&guid=\E.*?).>(.*?)\s?<|g);

# get icon names for missing categories
for (keys %cat_names)
{
  unless (exists $categories{$_})
  {
    my $htm = download($cat_names{$_});    
    my $cat_icon = ($htm =~ m|ctl00_ContentBody_CategoryControl1_imgIcon.*?cat_icons/(.*?)\.gif\"|)[0];
  
    print $_ . ": " . $cat_icon . "\n";
    $categories{$_} = $cat_icon;
  }
}

# save categories back to file
save_file($cat_filename, to_json(\%categories, {pretty => 1}));

# download missing files
for (keys %categories)
{
  my $filename = $categories{$_};
  
  unless (-e "Icons/$filename.gif" && (!(-e "Markers") || -e "Markers/$filename.png"))
  {
    print "Downloading: $filename\n";
    mkdir "Icons";
    download ("http://www.waymarking.com/dyn_img/bubble.aspx?path=$filename", "Markers/$filename.png") if (-e "Markers" && !(-e "Markers/$filename.png"));
    download ("http://www.waymarking.com/images/cat_icons/$filename.gif", "Icons/$filename.gif") unless (-e "Icons/$filename.gif");
  }

  # create named icon
  my $newname = $_;

  $newname =~ s|<|[less than]|g;
  $newname =~ s|>|[greater than]|g;
  $newname =~ s|:|[colon]|g;
  $newname =~ s|\"|[double quote]|g;  
  $newname =~ s|/|[forward slash]|g;
  $newname =~ s|\\|[backslash]|g;  
  $newname =~ s/\|/[vertical bar]/g;    
  $newname =~ s|\?|[question mark]|g;
  $newname =~ s|\*|[asterisk]|g;
  
  $newname = decode('UTF-8', $newname);

  mkdir "Icons2";
  Win32API::File::CopyFileW(encode('UTF16-LE', "Icons/$filename.gif\0"), encode('UTF16-LE', "Icons2/$newname.gif\0"), 1);
}

######################################################################
######################################################################
######################################################################

sub download
{
  my $url = shift;
  my $filename = shift;
  my @params = @{shift @_} if $_[0];
  
  return read_file($filename) if ($filename && -e $filename);
  
  my @headers = 
  (
    'User-Agent' =>	'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; BOIE9;ENUS)',
    'Accept-Language' =>	'cs-CZ',
    'Connection' =>	'keep-alive'  
  );
  
  #print 'Downloading: ' . $url . "\n";
  my $data;

  if ($#params >= 1)
  {
    $data = LWP::UserAgent->new->post($url, [@params], @headers)->content();
  }
  else
  {
    $data = LWP::UserAgent->new->get($url, @headers)->content();  
  }

  if ($filename)
  {
    open (MYFILE, ">", $filename) or die $!;
    binmode MYFILE;
    print MYFILE $data;
    close (MYFILE);
  }
  
  return $data;
}

sub read_file
{
  my $filename = shift;
  open (FILE, "<", $filename) or die $!;
  local $/;
  my $str = <FILE>;
  close FILE;
  $str;
}

sub save_file
{
  my $filename = shift;
  my $data = shift;
  open (MYFILE, ">", $filename) or die $!;
  print MYFILE $data;
  close (MYFILE);
}