// zjistíme, zda bod leží na úsečce dané dvěma body (podmínkou je, že leží na přímce dané těmi 2 body)
// -- to uděláme tak, že ověříme, zda azimut z prvního bodu do bodu p je blízký celkovému azimutu projekce mezi dvěma body
// a azimut z konce projekce do bodu je blízký celkovému azimutu + 180°

function isOnSegment(pointStart, pointEnd, p)
{
  if (!p)
    return false;

  var azDiff1 = SMap.Coords.angDiff(pointStart.azimuth(p), pointStart.azimuth(pointEnd));
  var azDiff2 = SMap.Coords.angDiff(pointEnd.azimuth(p), pointEnd.azimuth(pointStart));      

  if (azDiff1 < 90 && azDiff2 < 90)
     return true;     

  return false;
}

// tato funkce slouží pro následující 3 funkce
// -- po nalezení průsečíku musíme iteračně zpřesnit výsledek
// p - průsečík (který budeme zpřesňovat
// f_p0, f_p1 - funkce, které vrátí koncový bod (tedy například u kružnice to bude projekce, kde délka = r a azimut = azimut k poslední iteraci průsečíku)

function intersectionIterate(p, f_p0, f_p1)
{
  if (!p)
    return null;
    
  var last_dist = 1000000000;

  for (i = 0; i < 3000 /* prevent possible infinite loop */; i++)
  {
    var p0 = f_p0(p);
    var p1 = f_p1(p);

    // po každých 10 iteracích zkontrolujeme, zda se vzdálenost snižuje, pokud ne, ukončíme
    var dist = p0.distance(p1);

    if (dist >= last_dist && (i % 10) == 0 && i >= 30) 
      break;

    if ((i % 10) == 0)
      last_dist = dist;

    // zjistíme střed mezi dvěma projekcemi bodu, to bude náš nový průsečík
    p = p0.midpoint(p1);
  }

  // zkontrolujeme, zda náš nový průsečík je na správném místě a nedivergoval
  if (p.distance(f_p0(p)) > 0.1 || p.distance(f_p1(p)) > 0.1)
    return null;

  return p; 
}

// from: http://jsfiddle.net/justin_c_rounds/Gd2S2/light/
function getLineIntersection(p1, p2, p3, p4, options /* allow not iterating */)
{
  options = options || {noIteration: false};

  var line1StartX = p1.lon();
  var line1StartY = p1.lat(); 
  var line1EndX = p2.lon();
  var line1EndY = p2.lat(); 
  var line2StartX = p3.lon();
  var line2StartY = p3.lat(); 
  var line2EndX = p4.lon();
  var line2EndY = p4.lat();  
  
  //log(line1StartX + ", " + line1StartY);
  //alert(SMap.Coords.angDiff(p1.azimuth(p2), p3.azimuth(p4))); 

  // if the lines intersect, the result contains the x and y of the intersection (treating the lines as infinite) and booleans for whether line segment 1 or line segment 2 contain the point
  var denominator, a, b, numerator1, numerator2
  var result = {x: null, y: null, coords: null, onLine1: false, onLine2: false};
  
  denominator = ((line2EndY - line2StartY) * (line1EndX - line1StartX)) - ((line2EndX - line2StartX) * (line1EndY - line1StartY));
  
  if (denominator == 0) 
    return result;
    
  // if angle difference is too small, we have parallel lines
  // - but is acceptable for huge distances though (parallel lines may cross each other)
  if (SMap.Coords.angDiff(p1.azimuth(p2), p3.azimuth(p4)) < 0.01
    && p1.distance(p2) / 1000 < 100 && p3.distance(p4) / 1000 < 100)
    return result;  

  a = line1StartY - line2StartY;
  b = line1StartX - line2StartX;
  numerator1 = ((line2EndX - line2StartX) * a) - ((line2EndY - line2StartY) * b);
  numerator2 = ((line1EndX - line1StartX) * a) - ((line1EndY - line1StartY) * b);
  a = numerator1 / denominator;
  b = numerator2 / denominator;

  // if we cast these lines infinitely in both directions, they intersect here:
  result.x = line1StartX + (a * (line1EndX - line1StartX));
  result.y = line1StartY + (a * (line1EndY - line1StartY));

//     // it is worth noting that this should be the same as:
//     x = line2StartX + (b * (line2EndX - line2StartX));
//     y = line2StartX + (b * (line2EndY - line2StartY));

//   // if line1 is a segment and line2 is infinite, they intersect if:
//   if (a > 0 && a < 1)
//       result.onLine1 = true;
// 
//   // if line2 is a segment and line1 is infinite, they intersect if:
//   if (b > 0 && b < 1)
//       result.onLine2 = true;
 
  // if line1 and line2 are segments, they intersect if both of the above are true
  result.coords = SMap.Coords.fromWGS(result.y, result.x);
  
  if (!options.noIteration)
  {  
    // interpolace
    var endPoint = function (a, b)
    {
      return function(p)
      {
        var azimuth = a.azimuth(b);
        return a.projectPoint(azimuth + (SMap.Coords.angDiff(azimuth, a.azimuth(p)) >= 90 ? 180 : 0), a.distance(p));    
      };
    }
    
    result.coords = intersectionIterate(result.coords, endPoint(p1, p2), endPoint(p3, p4));
  }

  result.onLine1 = isOnSegment(p1, p2, result.coords);
  result.onLine2 = isOnSegment(p3, p4, result.coords);
 
  return result;
};

// funkce pro zjištění průsečíky dvou kružnic
// c1, r1 - střed a poloměr první kružnice
// c2, r2 - střed a poloměr druhé kružnice
function getTwoCircleIntersection(c1, r1, c2, r2)
{
  var result = {first: null, second: null};
  
  var middle = c1.midpoint(c2);                                    /* střed úsečky spojující středy */
  var intersections1 = getLineCircleIntersection(c1, c2, c1, r1);  /* průsečíky první kružnice s přímkou mezi středy */
  var intersections2 = getLineCircleIntersection(c1, c2, c2, r2);  /* průsečíky druhé kružnice s přímkou mezi středy */
  
  // test zda byl nějaký průsečík vůbec nalezen (měl by být)
  if (!intersections1.first || !intersections2.first)
    return result;
  
  // každá kružnice má dva průsečíky, z každých dvou vezmeme ten bližší středu druhé kružnice
  var p3 = c1.distance(intersections2.first) < c1.distance(intersections2.second) ? intersections2.first : intersections2.second;
  var p4 = c2.distance(intersections1.first) < c2.distance(intersections1.second) ? intersections1.first : intersections1.second;
  
  // střed úsečky spojující oba průsečíky
  // -- pokud střed jedné z kružnic leží uvnitř druhé, tak toto není příliš přesné, ale mělo by fungovat
  var p5 = p3.midpoint(p4);

  var d = c1.distance(p5);               /* vzdálenost mezi středem kružnice a střed úsečky spojující průsečíky */
  var azi = c1.finalBearing(p5);         /* azimut mezi těmito dvěma body (finalBearing) */
  var x = Math.sqrt(r1 * r1 - d * d);    /* vzdálenost mezi středem úsečky spojující průsečíky a průnikem kružnic */
  
  // zkontrolujeme, zda průsečík vůbec existuje
  if (!isFinite(x))
    return result;
  
  // sestrojíme oba průsečíky (pokud jedna kružnice leží uvnitř druhé, tak jen velmi hrubý odhad)
  var result = {first: p5.projectPoint(azi + 90, x), second: p5.projectPoint(azi - 90, x)};

  // ** nakonec upravíme, aby průsečík co nejvíce odpovídal reálu **

  // iterujeme:
  // z obou středů vedeme projekci do daného průsečíku se vzdáleností rovnou poloměru,
  // z obou těchto projekcí pak spočteme střed a to bude náš nový průsečík pro další iteraci

  var endPoint = function (c, r)
  {
    return function(p) {return c.projectPoint(c.azimuth(p), r);};
  }  

  result.first = intersectionIterate(result.first, endPoint(c1, r1), endPoint(c2, r2));
  result.second = intersectionIterate(result.second, endPoint(c1, r1), endPoint(c2, r2));
 
  return result;
}

// p1, p2 - body na přímce
// c - střed kružnice, r - poloměr kružnice
function getLineCircleIntersection(p1, p2, center, r)
{
  var result = {first: null, second: null, onLine1: false, onLine2: false};

  var p3 = center.nearestPoint(p1, p2); /* bod na přímce, který je nejbližší středu kružnice */
  var azi12 = p1.azimuth(p2);           /* azimut mezi p1 a p2 */
  var d13 = p1.distance(p3);            /* vzdálenost p1 a p3 */
  var dC3 = center.distance(p3);        /* vzdálenost středu a p3 */

  // test zda přímka leží vně kružnice
  if (dC3 > r)
    return result;
  
  var x = Math.sqrt(r * r - dC3 * dC3);      /* vzdálenost od p3 k průsečíku přímky a kružnice */

  if (SMap.Coords.angDiff(azi12, p1.azimuth(p3)) >= 90)
    azi12 += 180;

  var intersection1 = p1.projectPoint(azi12, d13 - x);   /* průsečík 1 */
  var intersection2 = p1.projectPoint(azi12, d13 + x);   /* průsečík 2 */

  // iterace - najdeme průsečík přesně:
  // ze středu kruhu vedeme do průsečíku projekci o délce poloměru
  // ze začátku projekce vedeme projekci, kde azimut = azimut ze začátku projekce do průsečíku, distance = vzd. začátku projekce a průsečíku
  function endPoint1 (c, r) {return function(p) {return c.projectPoint(c.azimuth(p), r);};}  
  function endPoint2 (a, b) {return function(p) 
  {
    var azimuth = a.azimuth(b);
    return a.projectPoint(azimuth + (SMap.Coords.angDiff(azimuth, a.azimuth(p)) >= 90 ? 180 : 0), a.distance(p));
  };}  

  result.first = intersectionIterate(intersection1, endPoint1(center,r), endPoint2(p1, p2));
  result.second = intersectionIterate(intersection2, endPoint1(center,r), endPoint2(p1, p2));

  // zjistíme, zda průsečíky leží na úsečce
  result.onLine1 = isOnSegment(p1, p2, result.first);
  result.onLine2 = isOnSegment(p1, p2, result.second);   

  return result;
}

function pataKolmice(p1, p2, p3 /* bod, ze kterého vedeme patu kolmice */)
{
  // začneme s nejlepším odhadem
  var intersection = p3.nearestPoint(p1, p2);

  // nastavíme nějaké rozumné výchozí hodnoty, ze kterých budeme postupně hledat nejlepší vzdálenost
  var bestDist = p3.distance(intersection);    /* nejlepší známá vzdálenost mezi patou kolmice a bodem, ze kterého ji vedeme */
  var finalDist1 = p1.distance(intersection);  /* umístění paty kolmice na projekci ve směru p1 -> p2 */
  var finalDist2 = p2.distance(intersection);  /* umístění paty kolmice na projekci ve směru p2 -> p1 */

  function findBest(finalDist, a, b, mod)
  {
    var step = 100 * 1000;           /* počáteční krok - 100 km */
    var azimuth = a.azimuth(b);      /* azimut mezi body projekce */
    var startDist = finalDist;       /* počáteční místo na projekci, od kterého budeme zkoušet */

    for (var dist = startDist + step * mod;; dist += step * mod)
    {
      var newDist = p3.distance(a.projectPoint(azimuth, dist));
      
      // našli jsme takový bod na projekci, kde je vzdálenost k bodu, ze kterého vedeme kolmici, nižší?
      // -- pak updatujeme nejlepší vzdálenost (na projekci)
      if (newDist < bestDist)
      {
        bestDist = newDist;
        finalDist = dist;
        
        // pokud by další krok nebyl úspěšný, tak preemptivně snížíme krok
        if (newDist = p3.distance(a.projectPoint(azimuth, dist + step * mod)) >= bestDist)
          step /= 10;
      }
      
      // jinak vrátíme předchozí krok a snížíme krok
      else if (step > 0.0000000001)
      {
        dist -= step * mod;
        step /= 10;
      }
      
      // pokud jsme dosáhli dostatečné přesnosti, ukončíme
      else
        break;
    }

    return finalDist;
  }
  
  // iterujeme dostatečněkrát, abychom určitě dosáhli minima
  // -- a to jak pro obě natočení projekce (z p1 do p2, a naopak), tak i na obě strany
  for (var i = 0; i < 5; i++)
  {
    finalDist1 = findBest(finalDist1, p1, p2, 1);
    finalDist1 = findBest(finalDist1, p1, p2, -1);
    
    finalDist2 = findBest(finalDist2, p2, p1, 1);
    finalDist2 = findBest(finalDist2, p2, p1, -1);          
  }
  
  // log("bestDist: " + bestDist);

  // nejlepší body pro obě natočení projekce
  var finalPoint1 = p1.projectPoint(p1.azimuth(p2), finalDist1);
  var finalPoint2 = p2.projectPoint(p2.azimuth(p1), finalDist2);
  
  // vybereme ten bližší bodu, ze kterého vedeme kolmici
  var foot = null;
  var step = 0.001; /* m */
  
  // rozhodnome, který směr projekce nám přinesl lepší výsledek a ten použijeme
  if (p3.distance(finalPoint1) < p3.distance(finalPoint2))
  {
    foot = finalPoint1;
    
    // kontrola, zda jsou dva body v bezprostředním okolí dále od bodu, ze kterého vedeme kolmici
    // -- pokud ne, tak nemáme minimum a tedy někde nastala chyba a bod nevytvoříme
    if (bestDist > p3.distance(p1.projectPoint(p1.azimuth(p2), finalDist1 + step))
      || bestDist > p3.distance(p1.projectPoint(p1.azimuth(p2), finalDist1 - step)))
      return;
  
  }
  else
  {
    foot = finalPoint2;
    
    if (bestDist > p3.distance(p2.projectPoint(p2.azimuth(p1), finalDist2 + step))
      || bestDist > p3.distance(p2.projectPoint(p2.azimuth(p1), finalDist2 - step)))
      return;        
  }
  
  return foot;      
}

// http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript
function guid()
{
    function _p8(s) {
        var p = (Math.random().toString(16)+"000000000").substr(2,8);
        return s ? "-" + p.substr(0,4) + "-" + p.substr(4,4) : p ;
    }
    return _p8() + _p8(true) + _p8(true) + _p8();
}

// retrieves value or coordinates
function evalFormula(text)
{
  var result = {value: null, isValue: false, coords: null, isCoords: false};

  if (!text)
    return result;

  // zkusíme, zda nejsou na konci jednotky
  var units = /(km|hm|m|dm|cm|mm|in|ft|yd|mi|nmi|deg|rad|grad|gon|mgon)\s*$/i;
  var regex = units.exec(text);
  //alert(regex + regex[0].equals("km"));
  if (regex)
  {
    var append = "";
    
    switch(regex[0])
    {
      /* délky */
      case "km":  append = "*1000"; break;        
      case "hm":  append = "*100"; break;        /* hektometr */
      case "m":   append = "*1"; break;
      case "dm":  append = "*0.1"; break;
      case "cm":  append = "*0.01"; break;
      case "mm":  append = "*0.001"; break;
      case "in":  append = "*0.0254"; break;     /* inch */
      case "ft":  append = "*0.3048"; break;     /* foot */
      case "yd":  append = "*0.9144"; break;     /* yard */
      case "mi":  append = "*1609.344"; break;   /* míle */
      case "nmi": append = "*1852"; break;       /* nautická míle */
    
      /* úhly */  
      case "deg":  append = "*1"; break;               /* stupeň */
      case "rad":  append = "*(180/Math.PI)"; break;   /* radián */
      case "gon":                                      /* gradián */
      case "grad": append = "*0.9"; break;
      case "mgon": append = "*0.9*0.001"; break;
    }

    if (append !== "")
      text = text.replace(units, "") + append;
  }    
  //
  var value = Math.NaN;
  
  // nejdříve zkusíme prohodit čárku za tečku (ale tím můžeme zabít nějaký skript, kde je čárka potřeba)
  try {value = eval(text.replace(",","."));} catch (e){};
  
  // ještě zkusíme bez změny čárky na tečku
  if (!isFinite(value))
  {
    try {value = eval(text);} catch (e){};
  }

  // pokud máme hodnotu, ukončíme
  if (isFinite(value))
  {
    result.value = value;
    result.isValue = true;
    return result;  
  }
  
  // ještě zkusíme souřadnice
  // -- souřadnice typu "50 -15" byly převedly výše na výraz, je třeba použít jiný formát!
  var coords = SMap.Coords.fromString(text);

  if (coords)
  {
    result.coords = coords;
    result.isCoords = true;
    return result;  
  }       

  return result;
}

//****************************************
//****************************************
//****************************************
translations["_project_point_distance_coords"] = {"English": "distance (m) or end coords", "Czech": "Vdálenost (m) nebo Souřadnice"};
translations["_project_point_distance"] = {"English": "distance (m)", "Czech": "Vdálenost (m)"};
translations["_project_point_azimuth"] = {"English": "Enter azimuth (in degrees)", "Czech": "Zadej azimut (ve stupních)"};
translations["_project_point_delete"] = {"English": "Delete", "Czech": "Smazat"};
translations["_delete_all_title"] = {"English": "SHIFT: delete all", "Czech": "SHIFT: smaž vše"};
translations["_project_point_set_distance"] = {"English": "Set distance...", "Czech": "Nastav vzdálenost..."};
translations["_project_point_set_azimuth"] = {"English": "Set azimuth...", "Czech": "Nastav azimut..."};
translations["_project_point_add_to_azimuth"] = {"English": "Change azimuth by...", "Czech": "Změň azimut o..."};
translations["_project_point_set_start"] = {"English": "Set starting coordinates...", "Czech": "Nastav souřadnice začátku..."};
translations["_project_point_set_end"] = {"English": "Set ending coordinates...", "Czech": "Nastav souřadnice konce..."};
translations["_project_point_swap_start_end"] = {"English": "Swap start and end", "Czech": "Prohoď začátek a konec"};
translations["_project_point_duplicate"] = {"English": "Duplicate", "Czech": "Vytvoř kopii"};
translations["_project_point_intersections"] = {"English": "Mark intersections", "Czech": "Nalezni průsečíky"};
translations["_project_point_intersections_title"] = {"English": "SHIFT: treat segments as lines", "Czech": "SHIFT: chová se k úsečkám jako k přímkám"};
translations["_project_point_parallel_lines"] = {"English": "Create parallel line(s)...", "Czech": "Vytvoř rovnoběžky..."}

translations["_project_point_info_start"] = {"English": "Start", "Czech": "Začátek"};
translations["_project_point_info_end"] = {"English": "End", "Czech": "Konec"};
translations["_project_point_info_distance"] = {"English": "Distance", "Czech": "Vzdálenost"};
translations["_project_point_info_azimuth"] = {"English": "Initial Bearing", "Czech": "Azimut"};
translations["_project_point_info_finalBearing"] = {"English": "Final Bearing", "Czech": "Azimut na konci"};
translations["_project_point_info_tunnel_distance"] = {"English": "Tunnel Distance", "Czech": "Vzdálenost skrz Zemi"};

window.projectPointObjects = {};

// create layers for project point - if not already created  
window.projectPointLayer = new SMap.Layer.Marker();
m.addLayer(window.projectPointLayer).enable();

window.projectPointGeomLayer = new SMap.Layer.Geometry();
m.addLayer(window.projectPointGeomLayer).enable();

function projectPoint(coords, distance /* altenatively: point2 */, azimuth)
{
  // můžeme funkci i zadat pomocí počátečního a koncového bodu
  if (typeof azimuth === 'undefined')
  {
    var p1 = coords;
    var p2 = distance;
    
    this.distance = p1.distance(p2);
    this.azimuth = p1.azimuth(p2);
    this.pointStart = p1;
    this.pointEnd = p2;
  }
  
  else
  {
    this.distance = distance;
    this.azimuth = azimuth % 360;

    if (this.azimuth < 0)
      this.azimuth = 360 + this.azimuth;    
    
    this.pointStart = coords;
    this.pointEnd = this.pointStart.projectPoint(azimuth, distance);
  }
    
  var that = this;
  this.guid = guid();
  window.projectPointObjects[this.guid] = this;  

  // create line between points
  function drawLine(pointStart, pointEnd)
  {
    var points = [];
    var totalDistance = that.distance;//pointStart.distance(pointEnd);
    var totalPoints;
    var azimuth = pointStart.azimuth(pointEnd);
    
    // nastavíme počet bodů dle délky
    if (totalDistance < 10 /* km */ * 1000)
      totalPoints = 10;
    else if (totalDistance < 100 /* km */ * 1000)
      totalPoints = 100;
    else if (totalDistance < 20000 /* km */ * 1000)
      totalPoints = 300;      
    else
      totalPoints = 5000;
                
    points[0] = pointStart;
    
    for (var i = 1; i < totalPoints; i++)
      points[i] = pointStart.projectPoint(azimuth, i * totalDistance / totalPoints);
      
    points[totalPoints] = pointEnd;
    
    var options = {color: "red", width: 3, outlineWidth: 0, opacity: 0.85}
    var polyline = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, null, points, options);
    window.projectPointGeomLayer.addGeometry(polyline);
    
    drawArrow();
    return polyline;  
  
//     var options = {color: "red", width: 3, outlineWidth: 0, opacity: 0.85};
//     var line = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, null, [pointStart, pointEnd], options);
//     window.projectPointGeomLayer.addGeometry(line);
//     
//     drawArrow();
//     return line;  
  }
  
  // create direction arrow
  function drawArrow()
  {
    if (that.arrowGeom)
      window.projectPointGeomLayer.removeGeometry(that.arrowGeom);
    
//     // stará verze (projekce je přímka a ne polyline)  
//     var arrowAzimuth = that.pointStart.pixelAzimuth(that.pointEnd);
//     var arrowPoint = that.pointStart.pixelProject(arrowAzimuth, that.pointStart.pixelDistance(that.pointEnd) / 2);
  
    var p1 = that.pointStart;
    var p2 = that.pointEnd;
    
    // spočteme vrchol šipky
    var arrowPoint = p1.midpoint(p2);
    var arrowAzimuth = arrowPoint.azimuth(p2);

    var arrowAngle = 20;
    var arrowLength = 18;

    // spočteme délky stran šipek
    var arrowPt2Distance = arrowPoint.pixelProject(arrowAzimuth + 180 + arrowAngle, arrowLength).distance(arrowPoint);
    var arrowPt3Distance = arrowPoint.pixelProject(arrowAzimuth + 180 - arrowAngle, arrowLength).distance(arrowPoint);
    
    // a koncové body
    arrowPt2 = arrowPoint.projectPoint(arrowAzimuth + 180 + arrowAngle, arrowPt2Distance);
    arrowPt3 = arrowPoint.projectPoint(arrowAzimuth + 180 - arrowAngle, arrowPt3Distance);

    var options = {color: "red", width: 6, outlineWidth: 0, outlineColor: "red", opacity: 1}; 
    that.arrowGeom = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, [arrowPoint, arrowPt2, arrowPt3], options);
    window.projectPointGeomLayer.addGeometry(that.arrowGeom);      
  }
  
  this.onMapRedraw = m.getSignals().addListener(window, "map-redraw", function(e)
  {
    drawArrow();
  });  

  // create project point start
  this.markerStartDiv = JAK.mel("div", {className: "projectPointStart"});
  this.markerStart = new SMap.Marker(this.pointStart, null, {url: this.markerStartDiv, anchor: {left: 4, top: 4}});
  window.projectPointLayer.addMarker(this.markerStart);
  this.markerStart.decorate(SMap.Marker.Feature.Draggable);
  
  // create projected point
  this.markerEndDiv = JAK.mel("div", {className: "projectPointEnd"});
  this.markerEnd = new SMap.Marker(this.pointEnd, null, {url: this.markerEndDiv, anchor: {left: 4, top: 4}});
  window.projectPointLayer.addMarker(this.markerEnd);
  this.markerEnd.decorate(SMap.Marker.Feature.Draggable);
  
  // create bubble
  this.markerStart.decorate(SMap.Marker.Feature.Bubble, new SMap.Bubble({minWidth: 200}));
  this.markerEnd.decorate(SMap.Marker.Feature.Bubble, new SMap.Bubble({minWidth: 200}));
  
  // create card
  this.card = new SMap.Card(220);
  this.markerStart.decorate(SMap.Marker.Feature.Card, this.card);
  this.markerEnd.decorate(SMap.Marker.Feature.Card, this.card);
  
  // set bubble and card
  function setProjectPointInfo(/*pointStart, pointEnd, distance, azimuth*/)
  {                                
    var html = tr("_project_point_info_start") + ": " + that.pointStart.format("WGS84", "min")
      + "<br>" + tr("_project_point_info_end") + ": " + that.pointEnd.format("WGS84", "min") 
      + "<br>" + tr("_project_point_info_distance") + ": " + (that.distance > 10000 ? (parseFloat(that.distance / 1000).toFixed(4) + " km") : (parseFloat(that.distance).toFixed(1) + " m"))
      + "<br>" + tr("_project_point_info_azimuth") + ": " + parseFloat(that.azimuth).toFixed(5) + "°";

    that.markerEnd.bubble.setHTML(html);
    that.markerStart.bubble.setHTML(html);
    
    // add more info for the card
    var finalBearing = that.pointStart.finalBearing(that.pointEnd);
    html += "<br>" + tr("_project_point_info_finalBearing") + ": " + parseFloat(finalBearing).toFixed(5) + "°";
    var tunnelDistance = that.pointStart.tunnelDistance(that.pointEnd);
    html += "<br>" + tr("_project_point_info_tunnel_distance") + ": " + (tunnelDistance > 10000 ? (parseFloat(tunnelDistance / 1000).toFixed(4) + " km") : (parseFloat(tunnelDistance).toFixed(3) + " m"))
    
    that.card.getBody().innerHTML = html;
  }      
  
  setProjectPointInfo(this.pointStart, this.pointEnd, this.distance, this.azimuth);
  
  // add markers to layer
  window.projectPointLayer.addMarker(this.markerStart);
  window.projectPointLayer.addMarker(this.markerEnd);
  
  // create line
  this.markerStart.projectPointLine = drawLine(this.pointStart, this.pointEnd);
  
  // drag start or end points
  function redraw()
  {
    window.projectPointGeomLayer.removeGeometry(that.markerStart.projectPointLine);
    that.markerStart.projectPointLine = drawLine(that.pointStart, that.pointEnd);
    setProjectPointInfo(that.pointStart, that.pointEnd);  
  }
  
  this.markerDragHandler = m.getSignals().addListener(window, "marker-drag-move", function(e, target)
  {
    var marker = e.target;
  
    if (marker !== that.markerStart && marker !== that.markerEnd)
      return;

    // CTRL - move both points and leave distance/azimuth intact
    if (contains_class(m.getContainer(), "key_down_ctrl"))
    {
      if (marker === that.markerStart)
      {
        that.pointStart = marker.getCoords();
        that.pointEnd = marker.getCoords().projectPoint(that.azimuth, that.distance)
        that.markerEnd.setCoords(that.pointEnd);
      }
      else
      {
        var azimuth = that.pointEnd.azimuth(that.pointStart);
        that.pointEnd = marker.getCoords();
        that.pointStart = marker.getCoords().projectPoint(azimuth, that.distance)
        that.markerStart.setCoords(that.pointStart);
      }    
    }
    
    // SHIFT - move the point and leave azimuth intact
    if (contains_class(m.getContainer(), "key_down_shift"))
    {
      var p1 = that.pointStart;
      var p2 = that.pointEnd;

      var intersection = getLineIntersection(p1, p2, marker.getCoords(), marker.getCoords().projectPoint(p1.azimuth(p2) + 90, 1000), {noIteration: true});
      var distance = intersection.coords ? ((marker === that.markerStart ? p2 : p1).distance(intersection.coords)) : 0; 
      
      if (intersection.coords && distance > 5)
      {
        that.distance = distance;
        
        if (marker === that.markerStart)
        {
          that.pointStart = p2.projectPoint(p2.azimuth(p1), that.distance);
          that.markerStart.setCoords(that.pointStart);
        }
        else
        {
          that.pointEnd = p1.projectPoint(that.azimuth, that.distance);
          that.markerEnd.setCoords(that.pointEnd);          
        }
      }
      
      // pokud projekce zkolapsuje do nulové délky, tak jen updatujeme markery, jinak budou někde viset mimo!
      else
      {
        that.markerStart.setCoords(that.pointStart);
        that.markerEnd.setCoords(that.pointEnd);
      }

      that.distance = that.pointStart.distance(that.pointEnd);
      that.azimuth = that.pointStart.azimuth(that.pointEnd);        
    }    

    // else - just move the point and recalculate distance and azimuth
    else
    {
      if (marker === that.markerStart)
        that.pointStart = marker.getCoords();
      else
        that.pointEnd = marker.getCoords();    
      
      that.distance = that.pointStart.distance(that.pointEnd);
      that.azimuth = that.pointStart.azimuth(that.pointEnd);
    }

    redraw();
    marker.bubble.show();
  });
  
  // skryjeme bubliny, protože u SHIFT funkce může zůstat viset na místě
  this.markerDragHandler = m.getSignals().addListener(window, "marker-drag-stop", function(e, target)
  {
    var marker = e.target;
  
    if (marker !== that.markerStart && marker !== that.markerEnd)
      return;  
  
    redraw();
    marker.bubble.hide();
  });  
  
  // destructor
  this.destroy = function()
  {
    window.projectPointGeomLayer.removeGeometry(this.arrowGeom);
    window.projectPointGeomLayer.removeGeometry(this.markerStart.projectPointLine);
    m.getSignals().removeListener(this.onMapRedraw);
    m.getSignals().removeListener(this.markerDragHandler);
    contextMenu.removeContextMenu(this.cMenuStartIndex);
    contextMenu.removeContextMenu(this.cMenuEndIndex);
    window.projectPointLayer.removeMarker(this.markerStart);
    window.projectPointLayer.removeMarker(this.markerEnd);
    delete window.projectPointObjects[this.guid];
  }  
  
  // *** context menu ***
  var cMenu = "" +  
    "<ul class=\"context_menu\" style=\"display: none\">\n" +
    "  <li class=\"project_cmenu_delete\" title=\"" + tr("_delete_all_title") + "\">" + tr("_project_point_delete") + "</li>\n" +
    "  <li class=\"project_cmenu_set_distance\">" + tr("_project_point_set_distance") + "</li>\n" +
    "  <li class=\"project_cmenu_set_azimuth\">" + tr("_project_point_set_azimuth") + "</li>\n" +
    "  <li class=\"project_cmenu_add_to_azimuth\">" + tr("_project_point_add_to_azimuth") + "</li>\n" +
    "  <li class=\"project_cmenu_set_start\" title=\"" + tr("_cm_precise_coordinates_title") + "\">" + tr("_project_point_set_start") + "</li>\n" +
    "  <li class=\"project_cmenu_set_end\" title=\"" + tr("_cm_precise_coordinates_title") + "\">" + tr("_project_point_set_end") + "</li>\n" +
    "  <li class=\"project_cmenu_swap_start_end\">" + tr("_project_point_swap_start_end") + "</li>\n" +
    "  <li class=\"project_cmenu_duplicate\">" + tr("_project_point_duplicate") + "</li>\n" +
    "  <li class=\"project_cmenu_intersections\" title=\"" + tr("_project_point_intersections_title") + "\">" + tr("_project_point_intersections") + "</li>\n" +
    "  <li class=\"project_cmenu_parallel_lines\">" + tr("_project_point_parallel_lines") + "</li>\n" +        
        
    "  <li class=\"project_cmenu_project_point\">" + tr("Project point...") + "</li>\n" +
    "  <li class=\"project_cmenu_draw_circle\">" + tr("Draw a circle...") + "</li>\n" +
    "  <li class=\"cm_create_marker_point\">" + tr("Create marker") + "</li>\n" + 
    "</ul>";

  var cMenuDiv = document.createElement("div");
  cMenuDiv.innerHTML = cMenu;
  cMenu = cMenuDiv.firstChild;
  document.body.appendChild(cMenu);
  
  function contextMenuHandler(e, target, menu, options) 
  {
    var coords = options.point.getCoords();

    // delete (SHIFT: delete all projections)
    if (contains_class(target, "project_cmenu_delete"))
    { 
      for (var i in window.projectPointObjects)
      {
        if (e.shiftKey || window.projectPointObjects[i] === that)
          window.projectPointObjects[i].destroy();
      }
    }
    
    // set distance
    else if (contains_class(target, "project_cmenu_set_distance"))
    {
      ui("copybox", {text: that.distance + "" /* convert to string, 0 wouldn't otherwise appear */,
        width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_distance"),
        getText: function(text)
        {
          var result = evalFormula(text);
          
          if (!result.isValue || result.value == 0)
            return;

          that.distance = result.value;
          that.pointEnd = that.pointStart.projectPoint(that.azimuth, that.distance);
          that.markerEnd.setCoords(that.pointEnd);  
          redraw();
        }
      });
    }
    
    // set azimuth
    else if (contains_class(target, "project_cmenu_set_azimuth"))
    {
      ui("copybox", {text: that.azimuth + "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_azimuth"),
        getText: function(text)
        {
          var result = evalFormula(text);
          
          if (!result.isValue)
            return;        
            
          that.azimuth = result.value % 360;
          
          if (that.azimuth < 0)
            that.azimuth = 360 + that.azimuth;
          
          that.pointEnd = that.pointStart.projectPoint(that.azimuth, that.distance);
          that.markerEnd.setCoords(that.pointEnd);
          redraw();
        }
      });
    }
    
    // change azimuth by (add to azimuth)
    else if (contains_class(target, "project_cmenu_add_to_azimuth"))
    {
      ui("copybox", {text: "+0", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_add_to_azimuth"),
        getText: function(text)
        {
          var result = evalFormula(text);
        
          if (!result.isValue)
            return;        

          that.azimuth = (that.azimuth + result.value + 360) % 360;
          that.pointEnd = that.pointStart.projectPoint(that.azimuth, that.distance);
          that.markerEnd.setCoords(that.pointEnd);
          redraw();
        }
      });
    }    
    
    // set coordinates of starting point
    else if (contains_class(target, "project_cmenu_set_start"))
    {
      ui("copybox", {text: that.markerStart.getCoords().format("WGS84", "min", e.shiftKey ? -1 : 3), width: e.shiftKey ? 400 : 300, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_set_start"),
        getText: function(text)
        {
          var result = evalFormula(text);
          
          if (!result.isCoords)
            return;
          
          that.pointStart = result.coords;
          that.markerStart.setCoords(that.pointStart);
          that.distance = that.pointStart.distance(that.pointEnd);
          
          if (that.distance) /* if distance becomes 0, azimuth is meaningless, so retain the old one */ 
            that.azimuth = that.pointStart.azimuth(that.pointEnd);
                               
          redraw();
        }
      });    
    }
    
    // set coordinates of ending point
    else if (contains_class(target, "project_cmenu_set_end"))
    {
      ui("copybox", {text: that.markerEnd.getCoords().format("WGS84", "min", e.shiftKey ? -1 : 3), width: e.shiftKey ? 400 : 300, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_set_end"),
        getText: function(text)
        {
          var result = evalFormula(text);
          
          if (!result.isCoords)
            return;
          
          that.pointEnd = result.coords;
          that.markerEnd.setCoords(that.pointEnd);
          that.distance = that.pointStart.distance(that.pointEnd);
          
          if (that.distance) /* if distance becomes 0, azimuth is meaningless, so retain the old one */ 
            that.azimuth = that.pointStart.azimuth(that.pointEnd);
                   
          redraw();
        }
      });    
    }
    
    // swap starting and ending point
    else if (contains_class(target, "project_cmenu_swap_start_end"))
    {
        var temp = that.pointStart;
        that.pointStart = that.pointEnd;
        that.pointEnd = temp;
        
        that.azimuth = that.pointStart.azimuth(that.pointEnd);
 
        that.markerStart.setCoords(that.pointStart);
        that.markerEnd.setCoords(that.pointEnd);
         
        redraw();
    }
    // find intersections with other projections and circle
    else if (contains_class(target, "project_cmenu_intersections"))
    {
        for (var i in window.projectPointObjects)
        {
          // skip comparison of this projection with itself
          if (window.projectPointObjects[i] === that)
            continue;
        
          var p1 = window.projectPointObjects[i].pointStart;
          var p2 = window.projectPointObjects[i].pointEnd;
          
          var intersection = getLineIntersection(that.pointStart, that.pointEnd, p1, p2);
          
          // SHIFT --> draw imaginary intersections (intersections that are not on segments, but are on lines!)
          if (intersection.coords && (e.shiftKey || (intersection.onLine1 && intersection.onLine2))
            && that.pointStart.distance(intersection.coords) > 0.001 && that.pointEnd.distance(intersection.coords) > 0.001)
            new createPointMarker(intersection.coords);
        }
        
        for (var i in window.createCircleObjects)
        {
          var center = window.createCircleObjects[i].center;
          var radius = window.createCircleObjects[i].radius;
          
          var intersections = getLineCircleIntersection(that.pointStart, that.pointEnd, center, radius);

          // SHIFT --> draw imaginary intersections (intersections that are not on segments, but are on lines!)
          if (intersections.first && (e.shiftKey || intersections.onLine1))
            new createPointMarker(intersections.first);
            
          if (intersections.second && (e.shiftKey || intersections.onLine2))
            new createPointMarker(intersections.second);            
        }        
    }

    // duplicate
    else if (contains_class(target, "project_cmenu_duplicate"))
      new projectPoint(that.pointStart, that.pointEnd);
      
    // create parallel lines 
    else if (contains_class(target, "project_cmenu_parallel_lines"))
    {
      ui("copybox", {text: "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_distance_coords"),
        getText: function(text)
      {
        var result = evalFormula(text);
      
        if (result.isValue && result.value > 0)
        {
          var distance = result.value;
          new projectPoint(that.pointStart.projectPoint(that.azimuth + 90, distance), that.pointEnd.projectPoint(that.azimuth + 90, distance));
          new projectPoint(that.pointStart.projectPoint(that.azimuth - 90, distance), that.pointEnd.projectPoint(that.azimuth - 90, distance));
        }
        else if (result.isCoords)
        {
          new projectPoint(result.coords, that.distance, that.azimuth);
        }
      }
      });
    }

    // project point (recursive)
    else if (contains_class(target, "project_cmenu_project_point"))
      projectPointAsk(coords);
      
    // draw a circle
    else if (contains_class(target, "project_cmenu_draw_circle"))
      createCircleAsk(coords);
      
    // create point
    else if (contains_class(target, "cm_create_marker_point"))
      createPointMarkerAsk(coords);
  }

  this.cMenuStartIndex = contextMenu.addContextMenu(this.markerStartDiv, cMenu, function (e, target, menu, options) {}, contextMenuHandler, {point: this.markerStart});
  this.cMenuEndIndex = contextMenu.addContextMenu(this.markerEndDiv, cMenu, function (e, target, menu, options) {}, contextMenuHandler, {point: this.markerEnd});
}

function projectPointAsk(coords)
{
  ui("copybox", {text: "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_distance_coords"),
    getText: function(text)
    {
      var result = evalFormula(text);

      // pokud máme vzdálenost, tak se zeptáme na azimut
      if (result.isValue && result.value != 0)
      {
        var distance = result.value;
        
        ui("copybox", {text: "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_azimuth"),
          getText: function(text)
          {
            var result = evalFormula(text);
            var azimuth = result.value;
            
            if (result.isValue)
              new projectPoint(coords, distance, azimuth);
          }
        });
      }
      
      // jinak uděláme projekci mezi dvěma body
      else if (result.isCoords)
      {
        new projectPoint(coords, result.coords);
      }
    }
  });
}

//****************************************
//****************************************
//****************************************

translations["_draw_circle_radius_ask"] = {"English": "Radius (m) or Coordinates", "Czech": "Poloměr (m) nebo Souřadnice"};
translations["_draw_circle_radius"] = {"English": "Radius", "Czech": "Poloměr"};
//translations["_draw_circle_center_ask"] = {"English": "Enter circle center coordinates", "Czech": "Zadej souřadnice středu"};
translations["_draw_circle_center"] = {"English": "Center", "Czech": "Střed"};
translations["_draw_circle_delete"] = {"English": "Delete", "Czech": "Smazat"};
//translations["_draw_circle_set_center"] = {"English": "Set center...", "Czech": "Nastav střed..."};
translations["_draw_circle_set_radius"] = {"English": "Set radius...", "Czech": "Nastav poloměr..."};
translations["_draw_circle_set_center"] = {"English": "Set coordinates of center...", "Czech": "Nastav souřadnice středu..."};
translations["_draw_circle_intersections"] = {"English": "Mark intersections", "Czech": "Nalezni průsečíky"};
translations["_draw_circle_intersections_title"] = {"English": "SHIFT: treat segments as lines", "Czech": "SHIFT: chová se k úsečkám jako k přímkám"};

//translations["_draw_circle_delete_ask"] = {"English": "Do you really want to delete the circle?", "Czech": "Opravdu chcete kruh smazat?"};

window.createCircleObjects = {};

// create layer (if not already exists)
window.drawCircleCenterLayer = new SMap.Layer.Marker();
m.addLayer(window.drawCircleCenterLayer).enable();  

window.drawCircleLayer = new SMap.Layer.Geometry("drawCircleLayer");
m.addLayer(window.drawCircleLayer).enable();

function createCircle(coords, radius)
{
  this.radius = radius;
  this.center = coords;
  var that = this;
  this.guid = guid();
  window.createCircleObjects[this.guid] = this;    
  
  // create circle
  function drawCircle(center, radius)
  {
    // pro malé vzdálenosti nakreslíme kružnici
    if (radius < 10 /* km */ * 1000)
    {
      var point_on_circle = center.projectPoint(45, radius);
      var circle = new SMap.Geometry(SMap.GEOMETRY_CIRCLE, null, [center, point_on_circle], {color: "#00f", opacity: 0.3, outlineWidth: 0});
      window.drawCircleLayer.addGeometry(circle);
      return circle;    
    }

    // pro velké vzdálenosti kružnici aproximujeme polygonem
    else
    {
      var pointsOnCircle = [];
      var totalPoints = (radius < 100 /* km */ * 1000) ? 100 : 300;
      
      for (var i = 0; i < totalPoints; i++)
      {
        pointsOnCircle[i] = center.projectPoint(i * 360 / totalPoints, radius);
      }
      
      var options = {color: "#00f", outlineWidth: 0, /*outlineColor: "red",*/ opacity: 0.3}; 
      var polygon = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, pointsOnCircle, options);
      window.drawCircleLayer.addGeometry(polygon);
      return polygon;
    }
  }

  // create circle center
  this.markerDiv = JAK.mel("div", {className: "circleCenter"});
  this.marker = new SMap.Marker(this.center, null, {url: this.markerDiv, anchor: {left: 4, top: 4}});
  this.marker.decorate(SMap.Marker.Feature.Draggable);
  
  // create bubble
  var bubble = new SMap.Bubble({minWidth: 200});
  this.marker.decorate(SMap.Marker.Feature.Bubble, bubble);
  
  // create card
  this.card = new SMap.Card(200);
  this.marker.decorate(SMap.Marker.Feature.Card, this.card);
  
  // set bubble and card
  function setCircleInfo(center, radius)
  {
    var html = tr("_draw_circle_center") + ": " + center.format("WGS84", "min") 
      + "<br>" + tr("_draw_circle_radius") + ": " + (radius > 10000 ? (parseFloat(radius / 1000).toFixed(4) + " km") : (parseFloat(radius).toFixed(1) + " m"));
    that.marker.bubble.setHTML(html);
    that.card.getBody().innerHTML = html;  
  }      
  
  setCircleInfo(this.center, this.radius);
  
  // add marker to layer
  window.drawCircleCenterLayer.addMarker(this.marker);
  
  // create circle
  this.marker.drawCircle = drawCircle(this.center, this.radius);
  
  // move circle on drag
  this.markerDragHandler = m.getSignals().addListener(window, "marker-drag-move", function(e, target)
  {
    var marker = e.target;
  
    if (marker !== that.marker)
      return;
  
    var dragCoords = marker.getCoords();
    window.drawCircleLayer.removeGeometry(marker.drawCircle);            
    
    // CTRL -> resize
    if (contains_class(m.getContainer(), "key_down_ctrl"))
      that.radius = that.center.distance(dragCoords);
  
    // else -> move
    else
      that.center = dragCoords;
    
    marker.drawCircle = drawCircle(that.center, that.radius);
    setCircleInfo(that.center, that.radius);
    marker.bubble.show();
  });
  
  this.markerDragStopHandler = m.getSignals().addListener(window, "marker-drag-stop", function(e, target)
  {
    var marker = e.target;
  
    if (marker !== that.marker)
      return;
  
    // set center marker
    marker.setCoords(that.center);
    
    // update center / radius
    setCircleInfo(that.center, that.radius);
  });
  
  // destructor
  this.destroy = function()
  {
    window.drawCircleLayer.removeGeometry(this.marker.drawCircle);
    m.getSignals().removeListener(this.markerDragHandler);
    m.getSignals().removeListener(this.markerDragStopHandler);
    contextMenu.removeContextMenu(this.cMenuIndex);
    window.drawCircleCenterLayer.removeMarker(this.marker);
    delete window.createCircleObjects[this.guid];    
  }  
  
  // *** context menu ***
  var cMenu = "" +  
    "<ul class=\"context_menu\" style=\"display: none\">\n" +
    "  <li class=\"circle_cmenu_delete\" title=\"" + tr("_delete_all_title") + "\">" + tr("_draw_circle_delete") + "</li>\n" +
    //"  <li class=\"circle_cmenu_set_center\">" + tr("_draw_circle_set_center") + "</li>\n" +
    "  <li class=\"circle_cmenu_set_radius\">" + tr("_draw_circle_set_radius") + "</li>\n" +
    "  <li class=\"circle_cmenu_set_center\" title=\"" + tr("_cm_precise_coordinates_title") + "\">" + tr("_draw_circle_set_center") + "</li>\n" +
    "  <li class=\"circle_cmenu_intersections\" title=\"" + tr("_project_point_intersections_title") + "\">" + tr("_draw_circle_intersections") + "</li>\n" +
    "  <li class=\"circle_cmenu_project_point\">" + tr("Project point...") + "</li>\n" +
    "  <li class=\"circle_cmenu_draw_circle\">" + tr("Draw a circle...") + "</li>\n" +
    "  <li class=\"circle_create_marker_point\">" + tr("Create marker") + "</li>\n" +     
    "</ul>";

  var cMenuDiv = document.createElement("div");
  cMenuDiv.innerHTML = cMenu;
  cMenu = cMenuDiv.firstChild;
  document.body.appendChild(cMenu);

  this.cMenuIndex = contextMenu.addContextMenu(this.markerDiv, cMenu, 
  function (e, target, menu, options) {},
  function (e, target, menu, options) 
  {
    // delete (SHIFT: delete all circles)
    if (contains_class(target, "circle_cmenu_delete"))
    { 
      for (var i in window.createCircleObjects)
      {
        if (e.shiftKey || window.createCircleObjects[i] === that)
          window.createCircleObjects[i].destroy();
      }    
    }      

    else if (contains_class(target, "circle_cmenu_set_radius"))
    {
      ui("copybox", {text: that.radius + "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_draw_circle_radius_ask"),
        getText: function(text)
        {
          var result = evalFormula(text);
    
          // jestli máme hodnotu, tak vytvoříme kruh o daném poloměru
          if (result.isValue && result.value > 0)
            that.radius = result.value;
          
          // jestli máme souřadnice, tak je bereme jako bod na kružnici
          else if (result.isCoords)
            that.radius = that.center.distance(result.coords);

          else
            return;

          window.drawCircleLayer.removeGeometry(that.marker.drawCircle);            
          that.marker.drawCircle = drawCircle(that.center, that.radius);
          setCircleInfo(that.center, that.radius);
        }
      });
    }
    
    // set coordinates of center
    else if (contains_class(target, "circle_cmenu_set_center"))
    {
      ui("copybox", {text: that.marker.getCoords().format("WGS84", "min", e.shiftKey ? -1 : 3), width: e.shiftKey ? 400 : 300, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_draw_circle_set_center"),
        getText: function(text)
        {
          var result = evalFormula(text);
          
          if (!result.isCoords)
            return;
          
          that.center = result.coords;  
          window.drawCircleLayer.removeGeometry(that.marker.drawCircle);            
          that.marker.drawCircle = drawCircle(that.center, that.radius);
          that.marker.setCoords(that.center);
          setCircleInfo(that.center, that.radius);
        }
      });    
    }
    
    
    else if (contains_class(target, "circle_cmenu_intersections"))
    {
        for (var i in window.createCircleObjects)
        {
          // skip comparison of this circle with itself
          if (window.createCircleObjects[i] === that)
            continue;

          var center1 = that.center;
          var radius1 = that.radius;
          var center2 = window.createCircleObjects[i].center;
          var radius2 = window.createCircleObjects[i].radius;
          
          var intersections = getTwoCircleIntersection(center1, radius1, center2, radius2);
          
          if (intersections.first)
            new createPointMarker(intersections.first);            
          
          if (intersections.second)
            new createPointMarker(intersections.second);
        }

        for (var i in window.projectPointObjects)
        {
          var p1 = window.projectPointObjects[i].pointStart;
          var p2 = window.projectPointObjects[i].pointEnd;
          
          var intersections = getLineCircleIntersection(p1, p2, that.center, that.radius);

          // SHIFT --> draw imaginary intersections (intersections that are not on segments, but are on lines!)
          if (intersections.first && (e.shiftKey || intersections.onLine1))
            new createPointMarker(intersections.first);
            
          if (intersections.second && (e.shiftKey || intersections.onLine2))
            new createPointMarker(intersections.second);            
        }        
    }    
    
    // project point (recursive)
    else if (contains_class(target, "circle_cmenu_project_point"))
      projectPointAsk(that.center);
      
    // draw a circle
    else if (contains_class(target, "circle_cmenu_draw_circle"))
      createCircleAsk(that.center);
      
    // create a point
    else if (contains_class(target, "circle_create_marker_point"))
      createPointMarkerAsk(that.center);    
  }); 
}

function createCircleAsk(coords)
{
  ui("copybox", {text: "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_draw_circle_radius_ask"),
    getText: function(text)
    {
      var result = evalFormula(text);

      // jestli máme hodnotu, tak vytvoříme kruh o daném poloměru
      if (result.isValue && result.value > 0)
        new createCircle(coords, result.value);
      
      // jestli máme souřadnice, tak je bereme jako bod na kružnici
      else if (result.isCoords)
        new createCircle(coords, coords.distance(result.coords));
    }
  });
}

//****************************************
//****************************************
//****************************************

translations["_point_marker_delete"] = {"English": "Delete", "Czech": "Smazat"};
translations["_point_marker_set_coordinates"] = {"English": "Set coordinates...", "Czech": "Nastav souřadnice..."};
translations["_point_marker_change_color"] = {"English": "Change color", "Czech": "Změň barvu"};
translations["_point_marker_perpendicular_feet"] = {"English": "Mark perpendicular feet", "Czech": "Najdi paty kolmic"};

window.createPointMarkerObjects = {};

// create layer
window.markerPointLayer = new SMap.Layer.Marker();
m.addLayer(window.markerPointLayer).enable();

function createPointMarker(coords)
{
  this.coords = coords;
  var that = this;
  this.guid = guid();
  window.createPointMarkerObjects[this.guid] = this;  

  // create point
  this.markerDiv = JAK.mel("div", {className: "markerPoint"});
  this.markerDiv.style.background = "rgb(" + Math.floor(Math.random() * 256) + "," + Math.floor(Math.random() * 256) + "," + Math.floor(Math.random() * 256) + ")";
  this.marker = new SMap.Marker(this.coords, null, {url: this.markerDiv, anchor: {left: 4, top: 4}});
  this.marker.decorate(SMap.Marker.Feature.Draggable);
  
  // create bubble
  var bubble = new SMap.Bubble({minWidth: 160/*, alwaysShow: true*/});
  this.marker.decorate(SMap.Marker.Feature.Bubble, bubble);
  /*bubble.show();*/
  
  // create card
  this.card = new SMap.Card(300);
  this.marker.decorate(SMap.Marker.Feature.Card, this.card);
  
  // set bubble and card
  function setMarkerInfo(coords)
  {
    // marker
    var html = "";
    html += tr("Degrees") + ":  " + coords.format("WGS84", "deg", 11) + "<br>";
    html += tr("Minutes") + ":  " + coords.format("WGS84", "min", 7) + "<br>";
    html += tr("Seconds") + ":  " + coords.format("WGS84", "sec", 4) + "<br>";
    html += tr("UTM") + ":  " + coords.format("UTM", "", 5) + "<br>";
    html += tr("S-JTSK") + ":  " + coords.format("SJTSK", "", 6) + "<br>";
    that.card.getBody().innerHTML = html;  
    
    // bubble
    html = coords.format("WGS84", "min");
    that.marker.bubble.setHTML(html);
  }      
  
  setMarkerInfo(this.coords);
  
  // add marker to layer
  window.markerPointLayer.addMarker(this.marker);

  // move coords on drag
  function moveMarker(e)
  {
    var marker = e.target;
  
    if (marker !== that.marker)
      return;
  
    that.coords = marker.getCoords();
    setMarkerInfo(that.coords);
  }
  
  this.markerDragHandler = m.getSignals().addListener(window, "marker-drag-move", moveMarker);
  this.markerDragStopHandler = m.getSignals().addListener(window, "marker-drag-stop", moveMarker);
  
  // destructor
  this.destroy = function()
  {
    m.getSignals().removeListener(this.markerDragHandler);
    m.getSignals().removeListener(this.markerDragStopHandler);
    contextMenu.removeContextMenu(this.cMenuIndex);
    window.markerPointLayer.removeMarker(this.marker);
    delete window.createPointMarkerObjects[this.guid];    
  }
  
  // *** context menu ***
  var cMenu = "" +  
    "<ul class=\"context_menu\" id=\"abcdef\" style=\"display: none\">\n" +
    "  <li class=\"marker_point_cmenu_delete\" title=\"" + tr("_delete_all_title") + "\">" + tr("_point_marker_delete") + "</li>\n" +
    "  <li class=\"marker_point_cmenu_set_coordinates\" title=\"" + tr("_cm_precise_coordinates_title") + "\">" + tr("_point_marker_set_coordinates") + "</li>\n" +
    "  <li class=\"marker_point_add_child_here\" class=GSAK_Only>" + tr("Add Child Here") + "</li>\n" +
    "  <li class=\"marker_point_add_waypoint_here\" class=GSAK_Only>" + tr("Add Waypoint Here") + "</li>\n" +
    "  <li class=\"marker_point_change_color\">" + tr("_point_marker_change_color") + "</li>\n" +
    "  <li class=\"marker_point_perpendicular_feet\" title=\"" + tr("_project_point_intersections_title") + "\">" + tr("_point_marker_perpendicular_feet") + "</li>\n" +
    "  <li class=\"marker_point_cmenu_project_point\">" + tr("Project point...") + "</li>\n" +
    "  <li class=\"marker_point_cmenu_draw_circle\">" + tr("Draw a circle...") + "</li>\n" + 
       
    "</ul>";

  var cMenuDiv = document.createElement("div");
  cMenuDiv.innerHTML = cMenu;
  cMenu = cMenuDiv.firstChild;
  document.body.appendChild(cMenu);    
  
  this.cMenuIndex = contextMenu.addContextMenu(this.markerDiv, cMenu, 
  function (e, target, menu, options) {},
  function (e, target, menu, options) 
  { 
    // delete (SHIFT: delete all marker points)
    if (contains_class(target, "marker_point_cmenu_delete"))
    { 
      for (var i in window.createPointMarkerObjects)
      {
        if (e.shiftKey || window.createPointMarkerObjects[i] === that)
          window.createPointMarkerObjects[i].destroy();
      }    
    }       
      
    // set coordinates
    else if (contains_class(target, "marker_point_cmenu_set_coordinates"))
    {
      ui("copybox", {text: that.marker.getCoords().format("WGS84", "min", e.shiftKey ? -1 : 3), width: e.shiftKey ? 400 : 300, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_point_marker_set_coordinates"),
        getText: function(text)
        {
          var result = evalFormula(text);
          
          if (!result.isCoords)
            return;

          that.coords = result.coords;
          that.marker.setCoords(that.coords);
          setMarkerInfo(that.coords);
          
          if (!that.coords.inMap(m, true))
            m.setCenter(that.coords);            
        }
      });
    }
    
    else if (contains_class(target, "marker_point_add_child_here"))
      window.location.href = "gsak://%FF/Addchild/" + that.coords.lat() + "/" + that.coords.lon() + "/" + cachecode;          

    else if (contains_class(target, "marker_point_add_waypoint_here"))
      window.location.href = "gsak://%FF/AddWpt/" + that.coords.lat() + "/" + that.coords.lon();
      
    else if (contains_class(target, "marker_point_change_color"))
    that.markerDiv.style.background = "rgb(" + Math.floor(Math.random() * 256) + "," + Math.floor(Math.random() * 256) + "," + Math.floor(Math.random() * 256) + ")";

    else if (contains_class(target, "marker_point_perpendicular_feet"))
    {
      for (var i in window.projectPointObjects)
      {
        var p1 = window.projectPointObjects[i].pointStart;
        var p2 = window.projectPointObjects[i].pointEnd;
        var p3 = that.coords;
        
        var foot = pataKolmice(p1, p2, p3);

        // nevytvoříme bod, pokud leží na krajních bodech projekce (úsečky),
        // pokud je zaplý SHIFT tak vytvoříme i body, co neleží na úsečkách
        if (foot && p3.distance(foot) > 0.001 && (e.shiftKey || isOnSegment(p1, p2, foot)))
          new createPointMarker(foot);
      }
    }
    
    // project point (recursive)
    else if (contains_class(target, "marker_point_cmenu_project_point"))
      projectPointAsk(that.coords);
      
    // draw a circle
    else if (contains_class(target, "marker_point_cmenu_draw_circle"))
      createCircleAsk(that.coords);
  }); 
}

function createPointMarkerAsk(coords)
{
  new createPointMarker(coords);
}

