translations["_couldnt_parse_format"] = {"English": "Unknown format!", "Czech": "Neznámý formát!"};

translations["_merge_track_entities"] = 
{
  "English": "File contains multiple entities. Merge them? (they will be otherwised displayd as individual tracks)?", 
  "Czech": "Soubor obsahuje více entit (tracky, waypointy atd.). Chcete je sloučit (pokud ne, tak bude každá zobrazena jako samostatný track)?"
};

function Formats(data, options)
{
  this.options = options || {};
  this.extra = {};
  this.data = data;
  this.type = this.getFormat(this.data);
  this.filename = this.options.filename ? this.getFilename(this.options.filename) : "";
  this.displayName = null;

  this.serializer = new XMLSerializer();
  this.track = [];
  
  // individual tracks or other distinct sets of points as separate Formats instances
  this.Formats = [this];

  if (!data.length)
    return;
   
  if (this.type == "PLT")
    this.parsePLT(data);
    
  if (this.type == "WPT")
    this.parseWPT(data);    

  else if (this.type == "GSAK")
    this.parseGSAK(data);

  else if (this.type == "GPX")
    this.parseGPX(data);

  else if (this.type == "KML")
    this.parseKML(data);

  //if (this.type)
  //  this.sendToProcessFurther();
  //else
  //if (!this.type)
  //  ui("copybox", {text: tr("_couldnt_parse_format"), width: 200, height: 25});
}

Formats.prototype.getFilename = function(filename)
{
  return(filename.replace(/^.*?\//, ""));
}

Formats.prototype.getFormat = function()
{
  if (/OziExplorer Track Point File/.test(this.data))
    return "PLT";

  if (/OziExplorer Waypoint File/.test(this.data))
    return "WPT";

  if (/<gpx/.test(this.data))
    return "GPX";
    
  if (/<kml/.test(this.data))
    return "KML";    
    
  if (/[\d\.]+\D+[\d\.]+/.test(this.data))
    return "GSAK";
}

// comma's not allowed in text fields, character 209 can be used instead and a comma will be substituted.
Formats.prototype.oziEncodeString = function(str)
{
  var isArray = Array.isArray(str);
  str = isArray ? str : [str];
  
  for (var i = 0; i < str.length; i++)
    str[i] = (typeof str[i] === 'string' && str[i].length) ? str[i].replace(/,/g, "Ń") : str[i];
    
  return (isArray ? str : str[0]);      
}

Formats.prototype.oziDecodeString = function(str)
{
  var isArray = Array.isArray(str);
  str = isArray ? str : [str];
  
  for (var i = 0; i < str.length; i++)
    str[i] = (typeof str[i] === 'string' && str[i].length) ? str[i].replace(/Ń/g, ",") : str[i];

  return (isArray ? str : str[0]);  
}

Formats.prototype.metersToFeet = function(val)
{
  return (val / 0.3048);
}

Formats.prototype.feetToMeters = function(val)
{
  return (val * 0.3048);
}

Formats.prototype.isSameCoord = function(coord1, coord2)
{
  return (coord1.lat() == coord2.lat() && coord1.lon() == coord2.lon());
}


// left pads number with zeroes (to have 2 digits) - use for formatting date and time
Formats.prototype.pad = function(val)
{
  return ((val < 10) ? ("0" + val) : val);
}

Formats.prototype.round = function(val, places)
{
  val = parseFloat(val).toFixed(places);
  val = /\./.test(val) ? (val.replace(/\.?[0]+$/, "")) : val;
  return val;
}

Formats.prototype.parseDelphiTime = function(timeString)
{
//   url: http://www.rus-roads.ru/gps/help_ozi/fileformats.html
//
//   Delphi stores date and time values in the TDateTime type.
//   The integral part of a TDateTime value is the number of days that have passed since 12/30/1899.
//   The fractional part of a TDateTime value is the time of day. 
//   
//   Following are some examples of TDateTime values and their corresponding dates and times: 
//   
//   0 - 12/30/1899 12:00 am
//   2.75 - 1/1/1900 6:00 pm
//   -1.25 - 12/29/1899 6:00 am
//   35065 - 1/1/1996 12:00 am

  var delphiTimestamp = parseFloat(timeString);
  var date = parseInt(delphiTimestamp);
  var time = delphiTimestamp - date;
  this.delphiTimeStart = this.delphiTimeStart || new Date(Date.parse("1899-12-30T00:00:00Z")).getTime();
  var msInDay = 24 * 60 * 60 * 1000;
  var ms = this.delphiTimeStart + /* date */ (date * msInDay) + /* time */ + (time * msInDay);

  return (new Date(ms));
}

Formats.prototype.exportDelphiTime = function(timestamp)
{
  timestamp = (timestamp instanceof Date) ? timestamp.getTime() : timestamp;
  this.delphiTimeStart = this.delphiTimeStart || new Date(Date.parse("1899-12-30T00:00:00Z")).getTime();
  timestamp -= this.delphiTimeStart;
  var msInDay = 24 * 60 * 60 * 1000;
  var time = timestamp % msInDay;
  var date = (timestamp - time) / msInDay;
  time /= msInDay;

  return (date + time).toFixed(7);
}

/* extracts non-empty lines and all params on them (separated by comma) */
Formats.prototype.oziExtractAll = function(data)
{
  var lines = data.split("\n");
  
  // get lines
  for (var i = 0; i < lines.length; i++)
  {
    // get params from lines
    var line = lines[i].split(",");

    // trim everything
    for (var j = 0; j < line.length; j++)
      line[j] = line[j].trim();

    // remove empty items from the end
    while (line.length && line[line.length - 1] == "")
      line.pop();
    
    lines[i] = line;
    
    // remove line altogether if empty  
    if (!line.length)
      lines.remove(i--);
  }
  
  return lines;
}

// http://www.rus-roads.ru/gps/help_ozi/fileformats.html
Formats.prototype.parseWPT = function(data)
{
  var track = this.track;
  var lines = this.oziExtractAll(data);
  
  for (var i = 0; i < lines.length; i++)
  {
    var point = lines[i];
    
    // **** Header ****
    // Line 1 : File type and version information
    // Line 2 : Geodetic Datum used for the Lat/Lon positions for each waypoint
    // Line 3 : Reserved for future use
    // Line 4 : GPS Symbol set - not used yet 
    //     
    // Example:
    // OziExplorer Waypoint File Version 1.1
    // WGS 84
    // Reserved 2
    // magellan    

    if (i < 4 )
      continue;

    // **** Points ****
    // Field 1 : Number - this is the location in the array (max 1000), must be unique, usually start at 1 and increment. Can be set to -1 (minus 1) and the number will be auto generated.
    // Field 2 : Name - the waypoint name, use the correct length name to suit the GPS type.
    // Field 3 : Latitude - decimal degrees.
    // Field 4 : Longitude - decimal degrees.
    // Field 5 : Date - see Date Format below, if blank a preset date will be used
    // Field 6 : Symbol - 0 to number of symbols in GPS
    // Field 7 : Status - always set to 1
    // Field 8 : Map Display Format
    // Field 9 : Foreground Color (RGB value)
    // Field 10 : Background Color (RGB value)
    // Field 11 : Description (max 40), no commas
    // Field 12 : Pointer Direction
    // Field 13 : Garmin Display Format
    // Field 14 : Proximity Distance - 0 is off any other number is valid 
    // Field 15 : Altitude - in feet (-777 if not valid)
    // Field 16 : Font Size - in points
    // Field 17 : Font Style - 0 is normal, 1 is bold.
    // Field 18 : Symbol Size - 17 is normal size
    // Field 19 : Proximity Symbol Position
    // Field 20 : Proximity Time 
    // Field 21 : Proximity or Route or Both 
    // Field 22 : File Attachment Name 
    // Field 23 : Proximity File Attachment Name 
    // Field 24 : Proximity Symbol Name 
    // 
    // Examples:
    // 1,Krokodýlí farma - Agrodružstvo JevišoviceŃ Velký KarlovŃ Velký KarlovŃ Unikátní projekt chovu krokodýlů nilských. +420515275236,  48.803390,  16.310080,38929.6341390,  0, 1, 3,         0,     65535,, 0, 0,    0,    678,10, 0,17,0,10.0,2,,,
    // 2,ZOO BrnoŃ U Zoologické zahrady 46Ń 9-16+420546432311,  49.229170,  16.533430,38929.6341390,  0, 1, 3,         0,     65535,, 0, 0,    0,    700,10, 0,17,0,10.0,2,,,

    // skip points that don't at least have both coordinates
    if (point.length < 4)
      continue;
    
    // latitude / longitude
    var p = new SMap.Coords.fromWGS(point[2], point[3]);
    p.custom = p.custom || {};    
    p.custom.isPoint = true;

    // name
    p.custom.name = this.oziDecodeString(point[1]);

    // timestamp
    if (point.length >= 5 && point[4] != 0)
      p.custom.timestamp = this.parseDelphiTime(point[4]).getTime();

    if (point.length >= 6 && point[5]) p.custom.oziSymbol = point[5];
    if (point.length >= 7 && point[6]) p.custom.oziStatus = point[6];
    if (point.length >= 8 && point[7]) p.custom.oziMapDisplayFormat = point[7];
    if (point.length >= 9 && point[8]) p.custom.oziForegroundColor = point[8];
    if (point.length >= 10 && point[9]) p.custom.oziBackgroundColor = point[9];
    
    // description
    if (point.length >= 11)
      p.custom.desc = this.oziDecodeString(point[10]);

    if (point.length >= 12 && point[12]) p.custom.oziPointerDirection = point[11];
    if (point.length >= 13 && point[13]) p.custom.oziGarminDisplayFormat = point[12];
    if (point.length >= 14 && point[14]) p.custom.proximityDistance = point[13];
     
    // elevation
    if (point.length >= 15 && point[14] != -777 /* invalid value */)
      p.custom.elevation = this.feetToMeters(point[14]);
    else
      p.custom.evelation = NaN;

    if (point.length >= 16 && point[16]) p.custom.oziFontSize = point[15];
    if (point.length >= 17 && point[17]) p.custom.oziFontStyle = point[16];
    if (point.length >= 18 && point[18]) p.custom.oziSymbolSize = point[17];
    if (point.length >= 19 && point[19]) p.custom.oziProximitySymbolPosition = point[18];
    if (point.length >= 20 && point[20]) p.custom.oziProximityTime = point[19];
    if (point.length >= 21 && point[21]) p.custom.oziProximityOrRouteOrBoth = point[20];
    if (point.length >= 22 && point[22]) p.custom.oziFileAttachmentName = point[21];
    if (point.length >= 23 && point[23]) p.custom.oziProximityFileAttachmentName = point[22]; 
    if (point.length >= 24 && point[24]) p.custom.oziProximitySymbolName = point[23]; 

    track.push([p]);  
  }
}

// http://www.rus-roads.ru/gps/help_ozi/fileformats.html
Formats.prototype.exportWPT = function(segments)
{
  var time = new Date();
  var separator = ",";

  // header
  var text = "";
  text += "OziExplorer Waypoint File Version 2.1\n";
  text += "WGS 84\n";
  text += "Reserved 2\n";
  text += "magellan\n";
  
  var points = this.flattenPoints(segments);

  for (var i = 0; i < points.length; i++)
  {
    var p = points[i];
    
    var exp = [];
    
    // id
    exp.push(i + 1)
    
    // name
    exp.push(this.oziEncodeString(p.custom.name || ""));
    
    // Latitude, Longitude
    exp.push(p.lat().toFixed(6));
    exp.push(p.lon().toFixed(6));
     
    // timestamp
    if (!isNaN(p.custom.timestamp))
    {
      time.setTime(p.custom.timestamp);
      exp.push(this.exportDelphiTime(time));
    }
    else
      exp.push("");
    
    exp.push(p.custom.oziSymbol || 0);
    exp.push(p.custom.oziStatus || 1);
    exp.push(p.custom.oziMapDisplayFormat || 3);
    exp.push(p.custom.oziForegroundColor || "");
    exp.push(p.custom.oziBackgroundColor || "");
    exp.push(this.oziEncodeString(p.custom.desc || ""));
    exp.push(p.custom.oziPointerDirection || "");
    exp.push(p.custom.oziGarminDisplayFormat || "");
    exp.push(p.custom.proximityDistance || "");
    
    // altitude
    exp.push(isNaN(p.custom.elevation) ? "" : this.metersToFeet(p.custom.elevation).toFixed(1));
        
    exp.push(p.custom.oziFontSize || "");
    exp.push(p.custom.oziFontStyle || "");
    exp.push(p.custom.oziSymbolSize || "");
    exp.push(p.custom.oziProximitySymbolPosition || "");
    exp.push(p.custom.oziProximityTime || "");
    exp.push(p.custom.oziProximityOrRouteOrBoth || "");
    exp.push(this.oziEncodeString(p.custom.oziFileAttachmentName || ""));
    exp.push(this.oziEncodeString(p.custom.oziProximityFileAttachmentName || ""));
    exp.push(this.oziEncodeString(p.custom.oziProximitySymbolName || ""));

    // remove empty
    while (exp.length && exp[exp.length - 1] == "")
      exp.pop();

    text += exp.join(separator) + "\n";
  }

  return text;
}

// http://www.rus-roads.ru/gps/help_ozi/fileformats.html
Formats.prototype.parsePLT = function(data)
{
  var track = this.track;
  var lines = this.oziExtractAll(data);
  
  // last point (to help in segment break detection)
  var lastSegmentBreak;
  
  // track segments (each starts with parameter 3 being 1; first one may start with 0)
  for (var i = 0; i < lines.length; i++)
  {
    var trkpt = lines[i];

    // Line 1 : File type and version information
    // Line 2 : Geodetic Datum used for the Lat/Lon positions for each trackpoint
    // Line 3 : "Altitude is in feet" - just a reminder that the altitude is always stored in feet
    // Line 4 : Reserved for future use    

    // track params (color etc.)
    // Line 5 : multiple fields as below
    // Field 1 : always zero (0)
    // Field 2 : width of track plot line on screen - 1 or 2 are usually the best
    // Field 3 : track color (RGB)
    // Field 4 : track description (no commas allowed)
    // Field 5 : track skip value - reduces number of track points plotted, usually set to 1 
    // Field 6 : track type - 0 = normal, 10 = closed polygon, 20 = Alarm Zone
    // Field 7 : track fill style - 0 = bsSolid; 1 = bsClear; 2 = bsBdiagonal; 3 = bsFdiagonal; 4 = bsCross; 5 = bsDiagCross; 6 = bsHorizontal; 7 = bsVertical;
    // Field 8 : track fill color (RGB)

    if (i == 4)
    {
      this.extra.PLTParams = trkpt;
        
      // get params
      this.extra.width = trkpt[1];
      this.extra.color = trkpt[2];
      this.extra.description = this.oziDecodeString(trkpt[3]);
      this.displayName = this.extra.description;
      this.extra.type = trkpt[5];
      this.isPolygon = (this.extra.type == 10);
      this.extra.fillStyle = trkpt[6];
      this.extra.fillColor = trkpt[7];
    }
      
    // skip header and empty lines
    if (i <= 5 || trkpt.length <= 2)
      continue;

    // *** extract data from point ***
    // Line 6 : Number of track points in the track, not used, the number of points is determined when reading the points file     
    // Field 1 : Latitude - decimal degrees.
    // Field 2 : Longitude - decimal degrees.
    // Field 3 : Code - 0 if normal, 1 if break in track line
    // Field 4 : Altitude in feet (-777 if not valid)
    // Field 5 : Date - see Date Format below, if blank a preset date will be used
    // Field 6 : Date as a string 
    // Field 7 : Time as a string
    
    // latitude / longitude
    var p = new SMap.Coords.fromWGS(trkpt[0], trkpt[1]);
    p.custom = p.custom || {};
    
    // start of a track segment? then create a new track segment
    // note: OziExplorer has 1 in first point of a segment,
    // SmartComGPS alternates 0 and -1
    if (!track.length 
      || /* OziExplorer */ trkpt[2] == 1 
      || /* SmartComGPS */ (lastSegmentBreak != trkpt[2] && (lastSegmentBreak == -1 || trkpt[2] == -1)))
    {
      track.push([]);
      p.custom.segmentStart = true;
    }
    
    lastSegmentBreak = trkpt[2];
                    
    // elevation
    if (trkpt.length >= 4 && trkpt[3] != -777 /* invalid value */)
      p.custom.elevation = this.feetToMeters(trkpt[3]);
    else
      p.custom.evelation = NaN;
      
    // timestamp
    if (trkpt.length >= 5)
    {
      p.custom.timestamp = this.parseDelphiTime(trkpt[4]).getTime();
    }

    // append the point to the last track segment
    track[track.length - 1].push(p);
  }
}

// http://www.rus-roads.ru/gps/help_ozi/fileformats.html
Formats.prototype.exportPLT = function(segments)
{
  var time = new Date();
  var separator = ", ";
  
  // header
  var text = "";
  text += "OziExplorer Track Point File Version 2.1\n";
  text += "WGS 84\n";
  text += "Altitude is in Feet\n";
  text += "Reserved 3\n";

  // track params
  var params = [];
  params.push("0");
  params.push(this.extra.width || "2");
  params.push(this.extra.color || "255");
  params.push(this.displayName || "MapyCZ Macro PLT File");
  params.push("0");
  params.push(this.isPolygon ? "10" : (this.extra.type || "0"));
  params.push(this.extra.fillStyle || "2");
  params.push(this.extra.fillColor || "8421376");
  params.push("-1");  // unknown parameter
  params.push("0");   // unknown parameter
  
  this.oziEncodeString(params);
  text += params.join(separator) + "\n";
  
  // number of points
  text += this.flattenPoints(segments).length + "\n";

  for (var i = 0; i < segments.length; i++)
  {
    var points = segments[i];
    
    for (var j = 0; j < points.length; j++)
    {
      var p = points[j];
      var exp = [];
      
      // Latitude, Longitude
      exp.push(p.lat().toFixed(6));
      exp.push(p.lon().toFixed(6));
       
      // segment indicator
      exp.push(j ? "0" : "1");
      
      // altitude
      exp.push(isNaN(p.custom.elevation) ? "" : this.metersToFeet(p.custom.elevation).toFixed(1));
      
      // timestamp
      if (!isNaN(p.custom.timestamp))
      {
        time.setTime(p.custom.timestamp);
        exp.push(this.exportDelphiTime(time));
        exp.push(time.getDate() + "-" + (time.getMonth() + 1) + "-" + time.getFullYear());
        exp.push(time.getHours() + ":" + this.pad(time.getMinutes()) + ":" + this.pad(time.getSeconds()));
        //exp.push(time.toLocaleDateString());
        //exp.push(time.toLocaleTimeString());
      }
      
      // remove empty
      while (exp.length && exp[exp.length - 1] == "")
        exp.pop();

      this.oziEncodeString(exp);
      text += exp.join(separator) + "\n";
    }  
  }

  return text;
}

// Latitude, Longitude 
// Both the following formats are supported
// 
// 32.7123, -101.2045
// S32 25.921, E 115 46.050  
// 
// To include a waypoint from the database
// use the format W,xxxxxx (xxxxxx = code)
// or just flag them and click on the button
// below this box.

Formats.prototype.parseGSAK = function(data)
{
  var lines = data.split("\n");
  this.extra.comments = [];
  
  var tr = [];
  
  for (var i = 0; i < lines.length; i++)
  {
    var line = lines[i];
    
    // is line a comment (starts with #)? if not, then parse coords
    if (!line.match(/^\s*#/))
    {
      var point = SMap.Coords.fromString(line);
      
      if (point)
      {
        point.custom = point.custom || {};
        tr.push(point);
      }
    }
    
    // comment
    else
    {
      // not coords? then save as comment
      var comment = line.trim();
  
      if (comment.length && comment != "#")
        this.extra.comments.push(comment);
    }
  }

  // polygon? (first and last points identical) --> may contain segments (islands), so parse them
  if (tr.length > 1 && this.isSameCoord(tr[0], tr[tr.length - 1]))
  {
    var segments = [[]];
    var seg = segments[0];
    
    for (var i = 0, len = tr.length; i < len; i++)
    {
      seg.push(tr[i]);
      
      // segment end (same coordinates as start)
      if (i && this.isSameCoord(tr[0], tr[i]))
      {
        // remove track end
        seg.pop();

        // remove segment end (if not first segment)
        if (segments.length > 1)
          seg.pop();

        // if not last point, add another segment
        if (i != tr.length - 1)
        {
          seg = [];
          segments.push(seg);
        }
      }    
    }

    tr = this.flattenPoints(segments);
    this.isPolygon = true; 
  }

  this.track = [tr];
}

Formats.prototype.exportCoords = function(point, separator, fixed)
{
  separator = separator || ", ";
  fixed = fixed || 7;

  return(point.lat().toFixed(fixed) + separator + point.lon().toFixed(fixed))
}

Formats.prototype.exportGSAK = function(segments)
{
  var text = "";
  var separator = ", ";

  // put comments to start (if any)
  if (this.extra.comments && this.extra.comments.length)
    text += this.extra.comments.join("\n") + "\n";

  for (var i = 0; i < segments.length; i++)
  {
    var seg = segments[i];
  
    for (var j = 0, len = seg.length; j < len; j++)
      text += this.exportCoords(seg[j], separator) + "\n";
    
    // handle polygon segments
    if (this.isPolygon)
    {
      // not first and only segment? then append starting point of the segment
      if (i >= 1)
        text += this.exportCoords(seg[0], separator) + "\n";
  
      // append overall starting point
      text += this.exportCoords(segments[0][0], separator) + "\n";
        
      // segment separator if there are more segments
      if (i != segments.length - 1)
        text += "#\n";
    }
  }

  return text;
}

Formats.prototype.parseKML = function(data)
{
  // get Placemark elements and convert each one to a different segment, then it contains one of this:
  // - Point
  // - Line
  // - Polygon

  var kml = JAK.XML.createDocument(data);

  // set Polygon mode if any polygon found
  if (kml.getElementsByTagName("Polygon").length)
    this.isPolygon = true;  

  // go through Placemarks
  kml = kml.getElementsByTagName("Placemark");
  
  for (var i = 0, len = kml.length; i < len; i++)
  {
    var placemark = kml[i];
    var seg = [];
    var c = {};

    // get coordinates
    var coords = placemark.getElementsByTagName("coordinates");
    
    if (!coords.length)
      continue;
    
    if (coords[0].parentNode.tagName == "Point")
      c.isPoint = true;
    
    /* following is unreliable, coords are sometimes separated by spaces */  
    coords = JAK.XML.textContent(coords[0]).split(/[\n]//*/[^+\-\.\d,]+/*/);
    
    for (j = 0; j < coords.length; j++)
    {
     	var arr = coords[j].split(",");
      
      if (arr.length >= 2)
      {
       	var coord = new SMap.Coords.fromWGS(parseFloat(arr[1]), parseFloat(arr[0]));
        coord.custom = coord.custom || {};
        seg.push(coord);
      }
      
      // elevation
      if (arr.length >= 3)
        coord.custom.elevation = parseFloat(arr[2]);
    }

    // get params
    var params = {"name": "name", "description": "desc"};

    for (var j = 0, pmlen = placemark.childNodes.length; j < pmlen; j++)
    {
      var node = placemark.childNodes[j];

      if (params[node.tagName])
        c[params[node.tagName]] = JAK.XML.textContent(node);
    }
    
    // get time
    var TimeStamp = placemark.getElementsByTagName('TimeStamp');
    
    if (TimeStamp.length)
    {
      c.timestamp= new Date(JAK.XML.textContent(TimeStamp[0].firstChild)).getTime();
    }

    // set params for all points within
    for (var j = 0; j < seg.length; j++)
    {
      for (prop in c)
        seg[j].custom[prop] = c[prop];
    }

    this.track.push(seg);
  }

//           <Placemark>
//             <name>Ve$kou a Malou Studenou dolinou-118</name>
//             <Snippet/>
//             <description><![CDATA[
//               <table>
//                 <tr><td>Longitude: 20.179710 </td></tr>
//                 <tr><td>Latitude: 49.177208 </td></tr>
//                 <tr><td>Speed: 187.9 mph </td></tr>
//                 <tr><td>Heading: 32.6 </td></tr>
//                 <tr><td>Time: 2011-04-08T12:08:30.900Z </td></tr>
//               </table>
//             ]]></description>
//             <LookAt>
//               <longitude>20.179710</longitude>
//               <latitude>49.177208</latitude>
//               <tilt>66</tilt>
//             </LookAt>
//             <TimeStamp><when>2011-04-08T12:08:30.900Z</when></TimeStamp>
//             <styleUrl>#track</styleUrl>
//             <Point>
//               <coordinates>20.179710,49.177208</coordinates>
//             </Point>
//           </Placemark>
}

Formats.prototype.serializeCollection = function(collection)
{
  var str = "";
  
  for (var i = 0, len = collection.length; i < len; i++)
    str += this.serializer.serializeToString(collection[i]) + "\n";
    
  return str;  
}

// serializes child nodes of an XML element (excluding those within "supported" array)
Formats.prototype.serializeUnsupported = function(parentNode, supported)
{
  supported = supported || [];
  var str = "";

  for (var i = 0, len = parentNode.childNodes.length; i < len; i++)
  {
    var node = parentNode.childNodes[i];
    var tagName = new String(node.tagName).toLowerCase();
    
    // skip text nodes
    if (tagName == "undefined")
      continue;
    
    // try to find
    var found = false;
    
    // based on provided parameter, either run callback function
    if (typeof supported === 'function')
    {
      found = !supported(node, tagName, i);
    }
    
    // or iterate through parameters
    else
    {
      for (var j = 0; j < supported.length; j++)
      {
        if (tagName == supported[j])
        {
          found = true;
          break;
        }    
      }
    }
    
    // not found? then serialize
    if (!found)
    {
      str += this.serializer.serializeToString(node) + "\n";
    }  
  }
  
  return str;
}

/* gets all serialized text before certain element and also element's opening tag (including all attributes) */
Formats.prototype.getXMLHeader = function(data, tagName)
{
  var xmlObject = JAK.XML.createDocument(data).getElementsByTagName(tagName)[0];

  var re = new RegExp("^[\\d\\D]*?<" + tagName, "i");
  var header = data.match(re)[0];
  
  for (var i = 0; i < xmlObject.attributes.length; i++)
  {
    var attr = xmlObject.attributes[i];
    header += ' ' + attr.nodeName + '="' + attr.nodeValue + '"'; 
  }
  
  header += ">\n";
  return header;
}

/* this first function cuts GPX to separate entities (if there are mixed tracks, routes, wpts etc.) */
Formats.prototype.parseGPX = function(data)
{
  if (this.options.allInOne)
    this.parseGPX_Single(data);  

  var gpx = JAK.XML.createDocument(data);
  gpx = gpx.getElementsByTagName("gpx")[0];

  // *** get GPX header (everything before and including <gpx> tag) + unrecognized children ***
  this.gpxHeader = this.getXMLHeader(data, "gpx");
  this.gpxMetadata = this.serializeUnsupported(gpx, ["wpt", "trk", "rte"]);

  // extracts all distincs parts (all WPTs, all RTEs and single TRKs)
  var wptNodes = gpx.getElementsByTagName("wpt");
  var rteNodes = gpx.getElementsByTagName("rte");
  var trkNodes = gpx.getElementsByTagName("trk");
  
  // only one distinct type? then just parse it
  if (((wptNodes.length ? 1 : 0) + (rteNodes.length ? 1 : 0) + trkNodes.length) <= 1
    || (!this.options.dontAskDivideTrack && confirm(tr("_merge_track_entities"))))
  {
    this.parseGPX_Single(data);
  }

  // more types - generate new Formats class for each one  
  else
  {
    this.Formats = [];
    var collections = [];
    
    if (wptNodes.length)
    {
      var gpxData = this.gpxHeader + this.gpxMetadata + this.serializeCollection(wptNodes) + "</gpx>";
      var wptOptions = {filename: "Waypoints", allInOne: true};
      var wptFormats = new Formats(gpxData, wptOptions);
      this.Formats.push(wptFormats);    
    }

    if (rteNodes.length)
    {
      var gpxData = this.gpxHeader + this.gpxMetadata + this.serializeCollection(rteNodes) + "</gpx>";
      var rteOptions = {filename: "Route", allInOne: true};
      var rteFormats = new Formats(gpxData, rteOptions);
      this.Formats.push(rteFormats);  
    }
    
    if (trkNodes.length)
    {
      for (var i = 0; i < trkNodes.length; i++)
      {
        var gpxData = this.gpxHeader + this.gpxMetadata + this.serializer.serializeToString(trkNodes[i]) + "</gpx>";
        var trkOptions = {filename: ("Track " + i), allInOne: true};
        var trkFormats = new Formats(gpxData, trkOptions);
        this.Formats.push(trkFormats);
      }        
    }
  }
}

Formats.prototype.parseGPX_Single = function(data)
{
  var serializer = new XMLSerializer();

  var gpx = JAK.XML.createDocument(data);
  gpx = gpx.getElementsByTagName("gpx")[0];
  
  // get WPT, RTE, TRKSEG elements and convert each of them to separate segment
  // - WPT is a point (one of possibly many), so make it to segment to not have any lines on it
  // - RTE is a parent element and contains connected track points
  // - TRKSEG is a child of TRK (which can have more TRKSEG elements) - so extract all segments irregardless of the number of tracks (for now)
  
  // *** get GPX header (everything before and including <gpx> tag) + unrecognized children ***
  this.gpxHeader = this.getXMLHeader(data, "gpx");
  this.gpxMetadata = this.serializeUnsupported(gpx, ["wpt", "trk", "rte"]);

  // *** get segments ***
  var xmlSegments = [];

  // WPT
  var wpt = gpx.getElementsByTagName("wpt");
  
  for (var i = 0, len = wpt.length; i < len; i++)
    xmlSegments.push(wpt[i]);
  
  // RTE  
  var rte = gpx.getElementsByTagName("rte");
  
  for (var i = 0, len = rte.length; i < len; i++)
  {
    // get all extra elements (save for RTEPT)
    // - will be then appended to every route (irrespective of different settings for each route)
    if (!this.gpxRteExtra)
      this.gpxRteExtra = this.serializeUnsupported(rte[i], ["rtept"]);  
  
    xmlSegments.push(rte[i]);
  }
    
  // TRKSEG  
  var trkseg = gpx.getElementsByTagName("trkseg");
  
  for (var i = 0, len = trkseg.length; i < len; i++)
  {
    // get all extra elements (save for TRKSEG) - like above with RTE
    if (!this.gpxTrkExtra)
      this.gpxTrkExtra = this.serializeUnsupported(trkseg[i].parentNode, ["trkseg"]);
    
    xmlSegments.push(trkseg[i]);
    
  }
  
  // *** get points ***
  var segments = [];
  
  for (var i = 0; i < xmlSegments.length; i++)
  {
    var seg = [];
    segments.push(seg);
    
    var elm = xmlSegments[i];
    
    // WPT - no point type children, just add as a lone point in a segment
    if (elm.tagName == "wpt")
    {
      seg.push(elm)
    }
    
    // RTE - retrieve all RTEPT elements (route points)
    else if (elm.tagName == "rte")
    {
      var rtept = elm.getElementsByTagName("rtept");
      
      for (var j = 0, len = rtept.length; j < len; j++)
        seg.push(rtept[j]);
    }
    
    // TRKSEG - retrieve all TRKPT elements (track points)
    else if (elm.tagName == "trkseg")
    {
      var trkpt = elm.getElementsByTagName("trkpt");
      
      for (var j = 0, len = trkpt.length; j < len; j++)
        seg.push(trkpt[j]);
    }
  }
  
  // *** parse points ***
  for (var i = 0; i < segments.length; i++)
  {
    var seg = segments[i];

    for (var j = 0; j < seg.length; j++)
    {
      var elm = seg[j];

      var p = new SMap.Coords.fromWGS(elm.getAttribute("lat") || 0, elm.getAttribute("lon") || 0);
      p.custom = p.custom || {};      
      
      // special handling of points
      if (elm.tagName == "wpt")
        p.custom.isPoint = true;
      
      // parse through child nodes  
      p.custom.gpxMisc = this.serializeUnsupported(elm, function(node, tagName)
      {
        // elevation
        if (tagName == "ele")
          p.custom.elevation = parseFloat(JAK.XML.textContent(node));
      
        // timestamp
        else if (tagName == "time")
          p.custom.timestamp = new Date(JAK.XML.textContent(node)).getTime();
        
        // other values  
        else if (tagName == "name" || tagName == "desc" || tagName == "cmt" || tagName == "sym" || tagName == "type")
         p.custom[tagName] = JAK.XML.textContent(node);
         
        else
          return true;      
      });      
      
      // append to segment
      seg[j] = p;
    }
  }

  this.track = segments;  
}

Formats.prototype.escapeXML = function(str)
{
  // info: http://stackoverflow.com/questions/7918868/how-to-escape-xml-entities-in-javascript
  //str = "<![CDATA[" + str + "]]>";
  str = str.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;');
  return str;
}

Formats.prototype.exportGPX = function(segments, type)
{
  var time = new Date();
  var that = this;
  
  // Track:
  // <trkpt lat="50.424786" lon="15.352440">
  // <ele>281.829</ele>
  // <time>2012-05-01T22:01:12Z</time>
  // </trkpt>
  
  var mainTag = (type == "Track") ? "trk" : ((type == "Route") ? "rte" : "");  
  var subTag = (type == "Track") ? "trkseg" : ""; 
  var ptTag = (type == "Track") ? "trkpt" : ((type == "Route") ? "rtept" : "wpt"); 
  
  // GPX header (use original if we loaded this from GPX)
  var text = "";
  
  if (this.gpxHeader)
    text += this.gpxHeader;  
  else
  {
    text += '<?xml version="1.0" encoding="UTF-8"?>\n';
    text += '<gpx xmlns="http://www.topografix.com/GPX/1/1" creator="MapyCZ Macro" version="1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd">\n'
  }
  text += this.gpxMetadata || "";

  text += mainTag ? ("<" + mainTag + ">\n") : "";
  
  if (mainTag == "trk")
    text += this.gpxTrkExtra || "";

  for (var i = 0; i < segments.length; i++)
  {
    var seg = segments[i];
    text += subTag ? ("<" + subTag + ">\n") : "";
    
    for (var j = 0, len = seg.length; j < len; j++)
    {
      var p = seg[j];
      
      // lat/lon
      text += ' <' + ptTag  + ' lat="' + this.exportCoords(p, '" lon="', 7) + '">\n';

      // elevation
      if (!isNaN(p.custom.elevation))
         text += '    <ele>' + (p.custom.elevation).toFixed(3) + '</ele>\n';
       
      // time
      if (!isNaN(p.custom.timestamp))
      {
        time.setTime(p.custom.timestamp);
        text += "    <time>";
        text += time.getUTCFullYear() + "-" + this.pad(time.getUTCMonth() + 1) + "-" + this.pad(time.getUTCDate())
          + "T" + this.pad(time.getUTCHours()) + ":" + this.pad(time.getUTCMinutes()) + ":" + this.pad(time.getUTCSeconds()) + "Z";
        text += "</time>\n"; 
      }
      
      // other params
      function setParam(param)
      {
        if (typeof p.custom[param] !== 'undefined')
          text += '    <' + param + '>' + that.escapeXML(p.custom[param]) + '</' + param + '>\n';  
      }
      
      setParam("name");
      setParam("desc");
      setParam("cmt");
      setParam("sym");
      setParam("type");
      
      // misc stuff
      text += p.custom.gpxMisc || "";

      text += " </" + ptTag  + ">\n";
    }
    
    text += subTag ? ("</" + subTag + ">\n") : ""; 
  }

  text += mainTag ? ("</" + mainTag + ">\n") : "";
  text += "</gpx>\n";

  return text;
}

Formats.prototype.exportFormat = function(format, segments, options)
{
  options = options || {};
  this.isPolygon = !!options.isPolygon;

  if (format == "PLT")
    return(this.exportPLT(segments));

  if (format == "WPT")
    return(this.exportWPT(segments));

  else if (format == "GSAK")
    return(this.exportGSAK(segments));

  else if (format == "GPX_Track")
    return(this.exportGPX(segments, "Track"));

  else if (format == "GPX_Route")
    return(this.exportGPX(segments, "Route"));

  else if (format == "GPX_Waypoints")
    return(this.exportGPX(segments, "Waypoints"));
     
  return "export error";
}

Formats.prototype.flattenPoints = function(track)
{
  var points = [];
  
  for (var i = 0; i < track.length; i++)
  {
    var segment = track[i];
    
    // add segment start flag
    if (segment.length)
    {
      segment[0].custom = segment[0].custom || {};
      segment[0].custom.segmentStart = true;
    }
    
    for (var j = 0; j < segment.length; j++)
      points.push(segment[j]);
  }
  
  return points;  
}

Formats.prototype.sendToPolygon = function()
{
  if (!window.polygon)
    return;

  // flatten track and send to track editor
  var points = this.flattenPoints(this.track);
  var options = {};
  options.name = this.displayName || this.filename;
  options.points = points;
  options.closed = false;
  options.isPolygon = !!this.isPolygon;
  options.Formats = this;
  polygon.addPolyline(options);     
}



// function parseFile(data, filename)
// {
//   var maxPoints = 10000;
// 
//   // PLT (OziExplorer)
//   // http://www.rus-roads.ru/gps/help_ozi/fileformats.html
//   if (/\.plt$/.test(filename))
//   {
// 
//     parsePLT(data, filename);
//     
//     if (window.polygon)
//       polygon.addPolyline({name: "file", points: window.track, closed: true});
// 
// //     // **** convert tracks to gpx ****
// //     var gpx = '<?xml version="1.0" encoding="UTF-8" standalone="no" ?>\n';
// //     gpx += '<gpx xmlns="http://www.topografix.com/GPX/1/1" creator="" version="1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd">\n'
// //     
// //     gpx += "<trk>\n"
// //     gpx += "<name>PLT_Track</name>\n";
// //     
// //     for (var i = 0; i < tracks.length; i++)
// //     {
// //       gpx += "  <trkseg>\n"  
// // 
// //       for (var j = 0; j < tracks[i].length; j++)
// //       {
// //         var p = tracks[i][j];
// //         
// //         // lat/lon
// //         gpx += '    <trkpt lat="' + p[0] + '" lon="' + p[1] + '">\n';
// // 
// //         // elevation
// //         if (p[2] != -777 && p.length > 3)
// //            gpx += '    <ele>' + (p[3] * 0.3048).toFixed(4) + '</ele>\n';
// //            
// //         // date
// //         
// //         
// //         gpx += "    </trkpt>\n"
// //       }
// // 
// //       
// //       gpx += "  </trkseg>\n" 
// //       
// //       
// //     
// // 
// // //    <trkpt lat="50.424786" lon="15.352440">
// // //     <ele>281.829</ele>
// // //     <time>2012-05-01T22:01:12Z</time>
// // 
// //    
// //     
// //     
// //     }
// //     
// //     gpx += "</trk>\n"
// //     
// //     gpx += "</gpx>\n";
//     //log(track);
//     
//     //ui("copybox", {text: gpx, width: 700, height: 300});
//     //log(gpx.replace(/<>/,"@"));
// 
// //     log(lines.length);
// //     log(data.length);
//     //log(lines);
//     //log(data);
//   
// 
// //     var xmlDoc = JAK.XML.createDocument(gpx);
// //     gpx = new SMap.Layer.GPX(xmlDoc, "", {maxPoints: maxPoints});
// //     m.addLayer(gpx);
// //     gpx.enable();
// //     gpx.fit();  
//   
//   
//   }
// 
// 
//   if (/\.gpx$/.test(filename))
//   {
//     var xmlDoc = JAK.XML.createDocument(data);
//     var gpx = new SMap.Layer.GPX(xmlDoc, "", {maxPoints: maxPoints});
//     m.addLayer(gpx);
//     gpx.enable();
//     gpx.fit();
//   }
//   
//   else if (/\.kml$/.test(filename))
//   {
//     var xmlDoc = JAK.XML.createDocument(data);
//     var gpx = new SMap.Layer.KML(xmlDoc, "", {maxPoints: maxPoints});
//     m.addLayer(gpx);
//     gpx.enable();
//     gpx.fit();
//   } 
//   else
//   {
//     ui("copybox", {text: tr("_filetype_not_supported") + ":\n" + filename});
//   }
//   
// 
// // log(data);
// // log(filename);
// 
// 
// }

// if(!Array.prototype.last)
// {
//   Array.prototype.last = function()
//   {
//     return this[this.length - 1];
//   }                                
// }

// trim polyfill (IE7-8)
// info: http://blog.stevenlevithan.com/archives/faster-trim-javascript
if(!String.prototype.trim)
{  
  String.prototype.trim = function ()
  {  
    return this.replace(/^\s*((?:[\S\s]*\S)?)\s*$/, '$1');  
  };  
}

// IE7-8 polyfill for XMLSerializer
if (typeof window.XMLSerializer === 'undefined' && JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
{
  window.XMLSerializer = function()
  {
    this.serializeToString = function(node)
    {
      return (node.xml || "");
    }
  }
}

