//==============================================
//============== parse results =================
//==============================================

function parseLocationSearchResult(jData)
{
  if (jData == null || !jData.totalResultsCount)
  {
    ui("copybox", {text: tr("Nothing found!"), width: 200, height: 20});
    //alert(tr("Nothing found!"));
    return;
  }

  // go through all received coords and retrieve closest ones
  var min_distance;
  var new_coords;

  for (var i = 0; i < jData.geonames.length; i++)
  {
    var g = jData.geonames[i];
    /* 
    // skip administrative divisions and country name != Czechia/Slovakia
    if (i > 0 && (g.fcodeName.match(/administrative division/) || !(g.countryName.match(/Czechia|Slovakia/))))
      continue;                        

    // get distance between received coords and our original coords
    var distance = geoname_lookup_coords.coords.distance(SMap.Coords.fromWGS(g.lat, g.lng)) / 1000;
    
    // set new coords if distance is lower then the previous one
    if (!new_coords || distance < min_distance)
    {
      new_coords = g;
      min_distance = distance;
    }
    */
    
    new_coords = g;
  }

  new_coords = SMap.Coords.fromWGS(new_coords.lat, new_coords.lng);
  
  // if we are in projectionUTM33 and target is outside Europe, switch to Google Maps
  if (new_coords.isValidUTM33() == false && m.getProjection() === SMap.projUTM33)
    m.switchMap("GSatellite_ID");

  // set new center
  m.setCenter(new_coords);
}

//==============================================
//============ GeoNames Geocoder ===============
//==============================================

function geonamesCallback(jData)
{ 
  parseLocationSearchResult(jData);
}

function geonamesGeocoder(location_name)
{
  // info: http://www.geonames.org/export/geonames-search.html
  //request = 'http://api.geonames.org/searchJSON?formatted=true&name_startsWith=' + encodeURIComponent(location_name) + '&maxRows=1&username=gsakmapymakro&style=full&countryBias=CZ&lang=cs&callback=geonamesCallback'
  loadJS('http://api.geonames.org/searchJSON?name=' + encodeURIComponent(location_name) + '&maxRows=100&countryBias=CZ&username=gsakmapymakro&callback=geonamesCallback');
}

//==============================================
//============== Seznam Geocoder ===============
//==============================================

function seznamCallback (result) 
{
  // received search results
  var vysledky = result.getResults()[0].results || 0;

  // declare our new array (that we send to GeoName processing function)
  var jData = {geonames: []};
   
  // go through array and convert all items to GeoName compatible format
  //for (i = vysledky.length - 1; i >= 0; i--)
  for (i = 0; i < vysledky.length; i++) 
  {
    var item = vysledky[i];
    item.lat = item.coords.lat();
    item.lng = item.coords.lon();
    item.fcodeName = "";
    item.countryName = ((item.label).match(/(esk..republika)/)) ? "Czechia" 
      : (((item.label).match(/Slovensko/)) ? "Slovakia" : "");
      
    // only CZ and SK - use geoname for everything else
    /*if (item.countryName != "")*/ jData.geonames.push(item);
    
    //log_object(vysledky[i]);
    break;
  }
  
  jData.totalResultsCount = jData.geonames.length;

  // if we have results, parse them 
  if (jData.geonames.length)
    parseLocationSearchResult(jData);
    
  // no results? run GeoNames
  else
    geonamesGeocoder(geoname_lookup_coords.location);
}

function seznamGeocoder (location_name) 
{
  return(new SMap.Geocoder(location_name, seznamCallback, {url:"geocode.php"}))
}

//==============================================
//============== main function =================
//==============================================

function LocationLookup(location_name, lat, lon)
{ 
  // we can't pass parameters to the callback, so send them through global variable
  window.geoname_lookup_coords = {location: location_name, coords: new SMap.Coords.fromWGS(lat || 50, lon || 15)};
  
  try
  {
    // SMap Geocoder fails in IE8 and older (because of xmlHttpRequest)
    new SMap.Geocoder(location_name, seznamCallback, {url:"geocode.php"});
  }
  catch(ex)
  {
    // if seznam fails, use GeoNames
    geonamesGeocoder(location_name);
  }  
}