// encapsulate original function
function encapsulate (object, variable, fn)
{
  object[variable + "Orig"] = object[variable];
  object[variable] = fn;
}

SMap.Util.mergeParams = function(from, to)
{
  for (var p in from)
  {
    if (!(p in to))
      to[p] = from[p];
  }
}

// ************
// function setCenterZoom has no business calling setProjection
// (fixes reseting projection to UTM33 where zoom 19 was set)
// - our method may have issues, so replace the calling function instead
encapsulate(SMap.prototype, "setProjection", function(projection)
{
  if(arguments.callee.caller != this.setCenterZoom)
    this.setProjectionOrig(projection);
  // zoom may not be updated properly after leaving zoom 19, so fix it
  else
    this.redraw();
});

// setProjection without redraw
 SMap.prototype.setProjection2 = function(projection, redraw)
 {
 	if (this._projection) { this._projection.setOwner(null); }
 	this._projection = projection;
 	this._projection.setOwner(this);
  
//   if (redraw)
//  	  this._redraw(true); /* po zmene orientace je nutne plne prekresleni */

 	return this;
}

// create default projections (so that we don't have to create them each time we change a map
SMap.projUTM33 = new SMap.Projection.UTM33();
SMap.projMercator = new SMap.Projection.Mercator();

// switches map (and updates layer switch)
SMap.prototype.switchMap = function(id)
{
  JAK.DOM.removeClass(document.querySelector(".layer-switch .active"), "active");
  this.getLayer(this.CurrentMapId).disable();
  this.CurrentMapId = id;
  this.getLayer(this.CurrentMapId).enable();
}

// send redraw event also before the fact, not only after it
encapsulate(SMap.prototype, "_redraw", function(full)
{
  this._signals.makeEvent("map-redraw-before", this, {full:full});
  return(this._redrawOrig(full));
});

// this functions sometimes fails in IE7 and maybe IE8 - attempt to fix it
encapsulate(JAK.VML.prototype, "setCenterRadius", function(element, center, radius)
{
  try
  {
    return(this.setCenterRadiusOrig(element, center, radius));
  }
  catch(ex){/*log("JAK.VML setCenterRadius error");*/};
});



// ************
// get current layer + automatically switch projection + let us know before turning on the layer
encapsulate(SMap.Control.Layer.prototype, "_itemClick", function(e, elm)
{
  for (var id in this._layers) 
  {
  	if (this._layers[id] == elm)
    {
      var map = this.getMap();

      // set nearest zoom level
      map._zoom = map.getNearestZoom({layerTo: id});

      // set new ID
      map.CurrentMapId = id;
      
      // set new projection (needed for correct copyright get scale - otherwise would use old projection)
      var conf = SMap.CONFIG[id];
      map.setProjection2((conf.options && conf.options.defaultProjection /* override default projection and reproject tiles to it */)
       || /* projection in CONFIG settings */ conf.projection || /* default projection */ SMap.projUTM33);
    }
  }

  this._itemClickOrig(e, elm);
});

encapsulate(SMap.Layer.prototype, "enable", function()
{
  if (SMap.CONFIG[this.getId()] && SMap.CONFIG[this.getId()].preview)
  {
    // no ID set yet? then set it (will be updated in _itemClick)
    this.getMap().CurrentMapId = this.getMap().CurrentMapId || this.getId();

    // set zoom range for the new map
    var zoomRange = SMap.CONFIG[this.getId()].zoom;
    
    if (zoomRange && zoomRange.length)  
      this.getMap().setZoomRange.apply(this.getMap(), zoomRange);
   
    // change layer
    if (this.getId() == this.getMap().CurrentMapId)
    {
      // set new projection again (in case this is our first run)
      var conf = SMap.CONFIG[this.getId()];
      this.getMap().setProjection2((conf.options && conf.options.defaultProjection /* override default projection and reproject tiles to it */)
       || /* projection in CONFIG settings */ conf.projection || /* default projection */ SMap.projUTM33);

      this.getMap().getSignals().makeEvent("layer-change-before", this.getId());
      
      // redraw all marker and geometry layers (so that they are not off when projection changes)
      // - taken from SMap.prototype._redraw
      var map = this.getMap();
     	var cont = map._geometryCanvas.getContainer();
     	cont.style.left = -(map._offset.x+map._sizeHalf.x) + "px";
     	cont.style.top = -(map._offset.y+map._sizeHalf.y) + "px"; 

     	map._layers.forEach(function(layer)
      {
        if (!(layer.getId() in SMap.CONFIG)) /* skip map layers */
          layer.redraw(true);
      });
      
     	if (map._card) { map._card.anchorTo(map._card.getAnchor()); } /* pokud je vizitka, tak jeji ukotveni */
    }
  }
  
  this.enableOrig(this);
});

// převede id chytré vrstvy na id podřízené hlavní vrstvy, se kterou už můžeme pracovat
SMap.prototype.idFromSmartId = function(idIn)
{
  var idOut = idIn;
  if (idIn == SMap.DEF_SMART_BASE)   idOut = SMap.DEF_BASE;
  if (idIn == SMap.DEF_SMART_OPHOTO) idOut = SMap.DEF_OPHOTO;
  if (idIn == SMap.DEF_SMART_TURIST) idOut = SMap.DEF_TURIST;
  if (idIn == SMap.DEF_SMART_WINTER) idOut = SMap.DEF_TURIST_WINTER;
  if (idIn == SMap.DEF_SMART_SUMMER) idOut = SMap.DEF_SUMMER;

  return idOut;
}

SMap.prototype.isLayerSmart = function(id)
{
  id = id || this.CurrentMapId;
  return (this.idFromSmartId(id) != id)
}

// getProjection -> přidání možnosti získat projekci zadané vrstvy
encapsulate(SMap.prototype, "getProjection", function(layerId)
{
  if (typeof layerId === 'undefined' || !layerId)
    return this.getProjectionOrig();
    
  else
    return SMap.CONFIG[this.idFromSmartId(layerId)].projection || SMap.projUTM33;
});

// getZoomRange -> přidání možnosti získat rozsah zoomu zadané vrstvy
encapsulate(SMap.prototype, "getZoomRange", function(layerId)
{
  if (typeof layerId === 'undefined' || !layerId)
  {
    return this.getZoomRangeOrig();
  }
    
  else
  { 
    return SMap.CONFIG[this.idFromSmartId(layerId)].zoom || /* Mapy.cz transparent layers */ [3, 15];
  }
});

// computeCenterZoom: compute center zoom based on projection
encapsulate(SMap.prototype, "computeCenterZoom", function(arr /* soubor bodů */, usePadding)
{
  // najdeme hranice skupiny bodů (nejmenší/největší lat/lon)
  var minX = Infinity;
  var minY = Infinity;
  var maxX = -Infinity;
  var maxY = -Infinity;  
  
  for (var i = 0; i < arr.length; i++)
  {
    minX = Math.min(minX, arr[i].lon());
    minY = Math.min(minY, arr[i].lat());
    maxX = Math.max(maxX, arr[i].lon());
    maxY = Math.max(maxY, arr[i].lat());
  }

  // střed viewportu
  var center = SMap.Coords.fromWGS((minY + maxY) / 2, (minX + maxX) / 2);

  // poloviční width a height viewportu
 	var w = this._size.x;
  var h = this._size.y; 

  if (usePadding)
  { 
    w -= this._padding.left;
    w -= this._padding.right;
    h -= this._padding.top;
    h -= this._padding.bottom;
  }
  
  w /= 2;
  h /= 2;
  
  // načtení rozsahu zoomu
  //var zoomRange = this.getMap().getZoomRange();
  var zoomRange = this.getMap().getZoomRange();
  var minZoom = zoomRange[0];
  var maxZoom = zoomRange[zoomRange.length - 1];
  var proj = this.getMap().getProjection();
  var o = this.getMap().getOrientation();  

  for (var z = maxZoom; z >= minZoom; z--)
  {
    // pokud máme oblique projekci, tak projekci překlopíme na UTM33 pro zoomy menší než 19
    var proj2 = (proj.getCode() == "oblique" && z >= 19) ? SMap.projUTM33 : proj;
  
    // hranice oblasti v pixelech
    var px1 = proj2.coordsToPixel(SMap.Coords.fromWGS(minY, minX), center, z, o);
    var px2 = proj2.coordsToPixel(SMap.Coords.fromWGS(maxY, minX), center, z, o);
    var px3 = proj2.coordsToPixel(SMap.Coords.fromWGS(minY, maxX), center, z, o);
    var px4 = proj2.coordsToPixel(SMap.Coords.fromWGS(maxY, maxX), center, z, o);

    // zkontrolujeme, zda všechny body hranice leží ve viewportu pro daný zoom
    // -- pokud ano, tak ukončíme; pokud ne, tak pokračujeme s dalším nižším zoomem
    if (px1.x >= -w && px1.x <= w && px1.y >= -h && px1.y <= h
     && px2.x >= -w && px2.x <= w && px2.y >= -h && px2.y <= h
     && px3.x >= -w && px3.x <= w && px3.y >= -h && px3.y <= h
     && px4.x >= -w && px4.x <= w && px4.y >= -h && px4.y <= h)
     {
        // našli jsme největší možný zoom, kam všechny hraniční body pasují
        return [center, z];   
     }
  }

  // pokud jsme nenašli vhodný zoom, tak vrátíme ten nejmenší
  return [center, minZoom];  
});

// ******* smooth out zoom level change (gets nearest zoom of another layer)
SMap.prototype.getNearestZoom = function(options)
{
  options = options || {};

  var optionsDefault =
  {
    layerFrom: null /* aktuální vrstva */,
    zoomFrom: this.getZoom(),
    orientationFrom: this.getOrientation(),
    layerTo: null, /* aktuální vrstva */
    lower: false,  /* nastaví cílový zoom o 1 nižší */
    higher: false  /* nastaví cílový zoom o 1 vyšší */
  };

  SMap.Util.mergeParams(optionsDefault, options);

  // získame rozsah zoomů cílové vrstvy
  var zoomRange = this.getZoomRange(options.layerTo);
  var minZoom = zoomRange[0];
  var maxZoom = zoomRange[zoomRange.length - 1];

  // najdeme nejbližší zoom v cílové vrstvě
  var scaleFrom = this.getScale({layer: options.layerFrom, zoom: options.zoomFrom, orientation: options.orientationFrom});
  var zoomBest;
  var distBest = Infinity;
  
  for (var z = minZoom; z <= maxZoom; z++)
  {
     var scaleTo = this.getScale({layer: options.layerTo, zoom: z, orientation: options.orientationFrom});
     
     // vezmeme poměr meřítek a pak logaritmus, protože log(a/b) je roven -log(b/a)
     var dist = Math.log(scaleFrom / scaleTo);
          
     if (Math.abs(dist) < Math.abs(distBest))
     {
      distBest = dist; /* zachováme znaménko, kvůli options.lower a options.higher */
      zoomBest = z;
     }
  }

  // u některých vrstev je třeba manuálně nastavit o úroveň vyšší či nižší zoom
  if (options.lower && zoomBest < maxZoom && distBest > 0.05)
    zoomBest--;

  if (options.higher && zoomBest > minZoom && distBest < -0.05)
    zoomBest++;

  return zoomBest;
}

// get scale (options can have params: layer, zoom, round)
SMap.prototype.getScale = function(options)
{
  options = options || {};
  
  var optionsDefault =
  {
    layer: null /* současná vrstva */,
    zoom: this.getZoom(),
    orientation: this.getOrientation(),
    round: false /* zaokrouhlit měřítko? */ 
  }
  
  SMap.Util.mergeParams(optionsDefault, options);

  // pokud byla zadána vrstva, tak si vezmeme její projekci (nebo UTM33 pokud žádnou u sebe nemá),
  // -- pokud nebyla zadána, tak vezmeme projekci aktuální mapy
  var proj = this.getProjection(options.layer);
  
  // spočteme vzdálenost pixelů (0,0) a (100,100)
  var px1 = new SMap.Pixel(0, 0);
  var px2 = new SMap.Pixel(100, 100);

  var p1 = proj.pixelToCoords(px1, this.getCenter(), options.zoom, options.orientation);
  var p2 = proj.pixelToCoords(px2, this.getCenter(), options.zoom, options.orientation);
  var scale = p1.distance(p2) / px2.norm(); /* vydělíme, jelikož chceme délku strany a ne úhlopříčky, navíc znormovanou */
  scale *= 95/160; /* poměr, abychom to dostali přibližně na seznamácké hodnoty */

  // zaokrouhlování
  if (options.round)
  {
    // měřítka v řádu tisíců zaokrouhlí na tisíce (např. 75248 --> 75000)
    if (scale > 0.9)
      scale = Math.round(scale * 10) * 1000;
    // v řádu set na stovky
    else if (scale > 0.09)
      scale = Math.round(scale * 100) * 100;
    // jinak zaokrouhlíme na jednotky
    else
      scale = Math.round(scale * 10000);  
  }
  else
    scale *= 10000;

  return scale;
}

// přepíše funkci na výpočet měřítka
SMap.Control.Copyright.prototype._computeScale = function()
{
  return ("1 : " + this.getMap().getScale({round: true}));
}

// opravuje chybu ve verzi 2014-8-24, až bude chyba opravena, odstraníme
encapsulate(SMap.Control.Copyright.prototype, "_redraw", function()
{
  this._redrawOrig();
  this._dom.container.innerHTML = this.getMap().getZoom() + " / " + this._computeScale() + " " + this._dom.container.innerHTML; 
});


// fromUTM33 - just for monitoring when needed
// encapsulate(SMap.Coords, "fromUTM33", function(x, y)
// {
//  var latlon = SMap.Coords.fromUTM33Orig(x, y);
//  //log("fromUTM33: " + x + "_" + y + " >>> " + latlon);
//  return latlon;
// });



// ************
// fix local XHR (for oblique layer)
if (JAK.Browser.client == "ie" && document.protocol == "File Protocol")   
{
  // remove layer 19 for IE8-
  // doesn't work also in IE9 when using MOTW
  // (http://msdn.microsoft.com/en-us/library/ms537628(v=vs.85).aspx)
  if (JAK.Browser.version <= 8)
     SMap.CONFIG.obliqueExtents = [];

  // in IE9+, change XDomainRequest to XMLHttpRequest (otherwise calls will be denied)  
  else
    XDomainRequest = XMLHttpRequest;
}

// note: in webkit, marker title can't coexist with draggable markers (displaying card then often fails)
// (fixed by displaying div on hover instead)

// ************
// addDefaultLayer - add custom build url
encapsulate(SMap.prototype, "addDefaultLayer", function(id)
{
  var layer = this.addDefaultLayerOrig(id);
  layer._buildURLOrig = layer._buildURL;
  layer._buildURL = SMap.CONFIG[id].buildUrl || layer._buildURL;
  layer.redraw = SMap.CONFIG[id].redraw || layer.redraw;  
  return layer;
});

// **** don't render (sub)layers that are outside of zoom bounds
encapsulate(SMap.Layer.Tile.prototype, "_render", function(tiles)
{//log("render - " + this.getId());

  var zoom = this.getMap().getZoom();
  var zoomRange = SMap.CONFIG[this.getId()].zoom;   /* can't use getZoomRange, would break oblique */

  if (zoomRange && zoomRange.length && (zoom < zoomRange[0] || zoom > zoomRange[1]))
    return;

//   //*********************
//   // pokusíme se odfiltrovat některé dílky, pokud jsou souřadnice mimo (dočasné řešení)
//   var map = this.getMap();
//   
//   // výraz z _render()
//   var tileOffset = new SMap.Pixel(0, 0).toTile(map, this._options.tileSize).toPixel(map).minus(map.getOffset()).toCoords(map, map.getZoom());
// 
//   if ((!tileOffset || tileOffset.lat() > 130 || tileOffset.lat() < -90 || tileOffset.lon() < -720 || tileOffset.lat() > 720)
//     && map.getProjection() !== SMap.projMercator)
//   {
//     return;
//   }  
  //**********************
  

  this._renderOrig.apply(this, arguments)
  
  // my code
  if (!this.same)
  {
    var map = this.getMap();
        
    for (var i = 0; i < tiles.length; i++)
    {
      var obj = tiles[i];
      var t = obj.tile;
      var node = obj.node;
      
      if (!node)
        continue;
      
      // tl - top left, br - bottom right, tr - top right; tlp = top left pixel, ...
      // get source corners
      var tl = t.topLeftCoords(map);
      var br = t.bottomRightCoords(map);

      var tlp = tl.toPixel(map);
      var brp = br.toPixel(map);

      var trp = new SMap.Pixel(brp.x, tlp.y);
      var tr = trp.toCoords(map);
      var blp = new SMap.Pixel(tlp.x, brp.y);
      var bl = blp.toCoords(map);
      
      // get target corners
      var tlp2 = this.projOld.coordsToPixel(tl, map.getCenter(), map.getZoom() + this.zoomDelta, map.getOrientation());
      var brp2 = this.projOld.coordsToPixel(br, map.getCenter(), map.getZoom() + this.zoomDelta, map.getOrientation());
      var trp2 = this.projOld.coordsToPixel(tr, map.getCenter(), map.getZoom() + this.zoomDelta, map.getOrientation());
      var blp2 = this.projOld.coordsToPixel(bl, map.getCenter(), map.getZoom() + this.zoomDelta, map.getOrientation());

      // get translation
      var diffX = ((tlp2.x - tlp.x) + (brp2.x - brp.x) + (trp2.x - trp.x) + (blp2.x - blp.x)) / 4;
      var diffY = ((tlp2.y - tlp.y) + (brp2.y - brp.y) + (trp2.y - trp.y) + (blp2.y - blp.y)) / 4;
      
      // get scaleX and scaleY
      function dist(a, b) {return Math.sqrt(Math.pow(a.x - b.x, 2) + Math.pow(a.y - b.y, 2))};
      var scaleX = (dist(tlp2, trp2) + dist(blp2, brp2)) / (dist(tlp, trp) + dist(blp, brp));
      var scaleY = (dist(trp2, brp2) + dist(tlp2, blp2)) / (dist(trp, brp) + dist(tlp, blp));

      // get rotation
      var a = Math.atan2(trp2.y - tlp2.y, trp2.x - tlp2.x);
      var b = Math.atan2(brp2.y - blp2.y, brp2.x - blp2.x);      
      var angle = ((a + b) / 2) * 180 / Math.PI;
      angle += (map.getOrientation() == SMap.WEST || map.getOrientation() == SMap.EAST) ? 90 : 0;

      //nastavení transformace
      // -- dalo by se i přes this._dom.cOld.style[SMap.TRANSFORM], ale vytváří to grafické glitche a navíc je to různě přepisováno
      node.style[SMap.TRANSFORM] = "translate(" + diffX + "px, " + diffY + "px) scale(" + scaleX + "," + scaleY + ") rotate(" + angle + "deg)";
    }
  }  
});

// **** reproject tiles when layers have incompatible projection

encapsulate(SMap.Layer.Tile.prototype, "redraw", function(full)
{
  if (!this._active)
    return;

//  log(this.getId());
 
  var map = this.getMap();
  this.proj = SMap.CONFIG[this.getId()].projection || SMap.projUTM33;
  this.projOld = map.getProjection();
  this.same = (this.projOld === this.proj
    /* don't reproject layers from ČÚZK, that can be reprojected directly on WMS server */ 
    || (this._options.noReprojection && this._options.noReprojection(map)));

  // don't reproject anything within the oblique layer
  // -- EPSG:32633 crashes javascript
  if (this.projOld.getCode() == "oblique" /*&& this.proj.getCode() == "EPSG:32633"*/)
  { 
    this.same = true;
  }

  // don't reproject when transform CSS property not supported
  if (!SMap.TRANSFORM)
    this.same = true;

  if (!this.same)
  {
    this.zoomDelta = map.getZoom() - map.getNearestZoom({
      layerTo: this.getId(),
      higher: this._options.reprojectionZoomHigher,  /* sometimes choose only higher or lower zoom for rendering nicety reasons */
      lower: this._options.reprojectionZoomLower});

    map._zoom -= this.zoomDelta;

    // increase prefetch to cover entire viewport (in case we scale down or significantly rotate tiles)
    this._options.prefetch += 1;
    
    // remove prefetchDelay
    var prefetchDelayBackup = this._options.prefetchDelay;
    this._options.prefetchDelay = 0;
    
    this.getMap().setProjection2(this.proj);
  }

  var ret = this.redrawOrig(full);

  if (!this.same)
  {
    this.getMap().setProjection2(this.projOld); 
    map._zoom += this.zoomDelta;
    this._options.prefetch -= 1;
    this._options.prefetchDelay = prefetchDelayBackup;
  }
  
  this.same = false;
  
  //return ret;
});

// attempt to improve loading speed of ZM10 tiles
encapsulate(SMap.Layer.Tile.prototype, "_buildElement", function(tile)
{
  var node = this._buildElementOrig(tile);
 
  if (node)
  {
    node.onerror = function()
    {
      if (/cuzk/.test(node.src))
      {
        node.src = node.src + "&rand=" + Math.random();
        node.onerror = null; //function() {node.style.display = "none"};
      }
      
      // hide altogether on error
      else
      {
        node.style.display = "none";
      }
    };  
  }
  
  //log_object(this._options);
  
  if ("opacity" in this._options)
  {
    node.style.opacity = this._options.opacity;

    // doesn't work (will probably work in the future, when seznam.cz adds ability to change opacity)
    if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
      node.style.filter = "alpha(opacity=" + (this._options.opacity * 100) + ")";
  }
  
  return node;
});

// prevent random error in tile garbage collector
encapsulate(SMap.Layer.Tile.prototype, "_gc", function(tiles)
{
  try{this._gcOrig.apply(this, arguments)} catch(ex){};
});



// implement ability to be able to retrieve coordinates from Tile

/* get topLeft and bottomRight coords for both UTM33 and Mercator */
SMap.Tile.prototype.topLeftCoords = function(map, projection, zoom)
{
  var ts = this.tileSize;
  var proj = projection || map.getProjection(); 
  var coef = proj._screenCoefficients;
  var x = this.x * ts;
  var y = this.y * ts + ts * (coef.y == 1 ? 1 : 0);
  
  var coord = proj._absolutePixelToCoords(new SMap.Pixel(x, y), zoom || map.getZoom());
  return coord; 
}

SMap.Tile.prototype.bottomRightCoords = function(map, projection, zoom)
{
  var proj = projection || map.getProjection();
  var coef = proj._screenCoefficients;
  var px = new SMap.Pixel(this.tileSize * coef.x /* test this when we have map with -x */, this.tileSize * coef.y * -1);
  var coord = proj.shiftCoords(this.topLeftCoords(map), zoom || map.getZoom(), px);
  return coord;
}

SMap.Tile.prototype.getBBOX = function(map, version)
{
  var proj = map.getProjection();
  var tl = this.topLeftCoords(map);
  var br = this.bottomRightCoords(map);

  // UTM33
  if (proj.getCode() == "EPSG:32633")
  {
    tl = tl.toUTM33(); /* [lon, lat] */
    br = br.toUTM33();
    return [tl[0], tl[1], br[0], br[1]].join();
  }  

  // WGS84
  if (proj.getCode() == "EPSG:4326")
  {
    // 1.1.1 requires Lon,Lat; new versions require Lat,Lon
    if (version == "1.1.1")
      return [tl.lon(), tl.lat(), br.lon(), br.lat()].join();
    else
      return [tl.lat(), tl.lon(), br.lat(), br.lon()].join();
  }  
    
  // S-JTSK
  else if (proj.getCode() == "EPSG:102067")
  {
    tl = tl.toSJTSK();
    br = br.toSJTSK();

    return [-tl.x, -tl.y, -br.x, -br.y].join();
  }
  
  return "";
}

//******************************************


/*** fix listener errors, when we double click on a marker ***/
encapsulate(SMap.prototype, "animate", function(c, options)
{
  if (JAK.Timekeeper.getInstance()._findListener(this) == -1)
    return (this.animateOrig(c, options));
});

encapsulate(SMap.prototype, "setZoom", function(z, fixedPoint, animate)
{
  if (JAK.Timekeeper.getInstance()._findListener(this) == -1)  
    return (this.setZoomOrig(z, fixedPoint, animate));
});


// ************
// SMap.Control.Selection - modify to right click


// set more params
encapsulate(SMap.Control.Selection.prototype, "$constructor", function(thickness)
{
  // call and backup $super()
  this.$super();
  var temp = this.$super;
  this.$super = function(){};
  
  // note: Firefox and SHIFT+right click launches app context menu (can't be overridden)
  // - customStartDrag - called on mousedown
  // - customRedraw - called on mousemove 
  // - customHandler - called on draw finish (mouse up)
  // - customFailDrag - called on finish, but when mouse movement was too small and customHandler wouldn't be triggered
  var options = (typeof thickness === 'object') ? thickness : {thickness: thickness};
  this.customHandler = options.customHandler || null;
  this.fillColor = options.fillColor || null;
  this.cursor = options.cursor || null;
  this.customId = options.customId || null;
  this.shift = options.shift || false;
  this.ctrl = options.ctrl || false;
  this.alt = options.alt || false;
  this.meta = options.meta || false;
  this.mouseButton = (typeof options.mouseButton !== 'undefined') ? options.mouseButton : JAK.Browser.mouse.right;
  this._backupRedraw = this._redraw;   // enable us to call original redraw from our functions
  this._redraw = options.customRedraw || this._redraw;
  this.customStartDrag = options.customStartDrag || null;
  this.customFailDrag = options.customFailDrag || null; // called when dragging fails (no mouse movement)

  // call main function
  this.$constructorOrig(options.thickness);
  
  // restore $super()
  this.$super = temp;
});

// start selection only on dragging content (not any controls)
encapsulate(SMap.Control.Selection.prototype, "setOwner", function(owner)
{
  this.$super(owner);
  
  // version 2014-8-23 made owner sometimes null
  if (!owner)
    return;

  var elm = owner.getContent();
  this._ec.push(JAK.Events.addListener(elm, "mousedown", this, "_mousedown"));
});

// react on right click
encapsulate(SMap.Control.Selection.prototype, "_mousedown", function(e, elm)
{ 
  // handle special keys (allowing for custom handling)
  if (this.shift != !!e.shiftKey || this.ctrl != !!e.ctrlKey || this.alt != !!e.altKey || this.meta != !!e.metaKey
  || this.mouseButton != e.button || window.selectionActive)
    return;

  // Maxthon 2 fires mousedown on mouseup (on right click with SHIFT etc. pressed), prevent it
  if (this.mouseIsDown)
    return false;

  this.mouseIsDown = true;
  
  // prevent multiple selections being drawn at a time
  window.selectionActive = true;

  // Opera
  if (JAK.Browser.client == "opera")
    EventUtil.preventDefault(e);

  // custom function on start
  if (this.customStartDrag)
    this.customStartDrag(e, elm);

  // change cursor
  this.getMap().setCursor(this.cursor || "pointer");
 
  // *** prevent context menu from appearing ***
  var that = this;
  
  this.handleMouseUp = function(e, target)
  {
    if (that._point2 !== that._point1)
    {
      // context menu stops working on repeated righ-click selections;
      // attempt to fix it by turning it off in Opera (this would show context menu in IE, at least without a CTRL/SHIFT/ALT)
      // - still needs fixing (perhaps turn this on only for non-alt+ctrl+shift stuff???
      if (JAK.Browser.client != "opera")
        EventUtil.stopPropagation(e);
      
      EventUtil.preventDefault(e);  
    }

    if (e.type == "contextmenu")
    {
      while (that.contextMenuHandlers.length)
        EventUtil.removeHandler(that.contextMenuHandlers.pop());
    }
    
    else if (e.type == "mouseup")
    {
      while (that.mouseUpMenuHandlers.length)
        EventUtil.removeHandler(that.mouseUpMenuHandlers.pop());

      // remove handler for context menu here as well
      if (that.mouseButton != JAK.Browser.mouse.right)
      {
        while (that.contextMenuHandlers.length)
          EventUtil.removeHandler(that.contextMenuHandlers.pop());
      }
  
      // call mouseup function (only if any move happened)
      if (that.movedMouse)
        that._mouseup.call(that, e, target);
    }
  }
 
  // prevent context menu bubbling
  // IE7-8 --> guess map layer index (should be 0), IE9+ - use capture mode
  this.contextMenuHandlers = [];
  
  for (var i = 0; i < elm.childNodes.length; i++)
    this.contextMenuHandlers.push(EventUtil.addHandler(elm.childNodes[i], "contextmenu", this.handleMouseUp, false));

  this.contextMenuHandlers.push(EventUtil.addHandler(document.documentElement, "contextmenu", this.handleMouseUp, true));
  
  // prevent middle click bubbling (note: don't do this in IE7-8, would cause a bug)
  this.mouseUpMenuHandlers = [];
  this.mouseUpMenuHandlers.push(EventUtil.addHandler(document.documentElement, "mouseup", this.handleMouseUp, true));

  // *** call original function (e can't be modified, so create a new one) ***
  this._mousedownOrig({button: JAK.Browser.mouse.left, ctrlKey: 1, metaKey: 0, clientX: e.clientX, clientY: e.clientY}, elm);
  
  // change fill color (if set)
  if (this.fillColor)
    this._dom.container.style.backgroundColor = this.fillColor;
});

// mouse up
encapsulate(SMap.Control.Selection.prototype, "_mouseup", function(e, elm)
{
  EventUtil.preventDefault(e);
  EventUtil.stopPropagation(e);
  this.movedMouse = false;
  this.mouseIsDown = false;  /* Maxthon 2 fix */
  
  // restore cursor
  this.getMap().setCursor(null);

  // custom handler
  if (this.customHandler)
  {
    // get left-bottom/right-top points and direction where we started
    var direction = ((this._point1.x <= this._point2.x) ? "l" : "r") + ((this._point1.y <= this._point2.y) ? "t" : "b")
    var p1 = new SMap.Pixel(Math.min(this._point1.x, this._point2.x), Math.min(this._point1.y, this._point2.y));
    var p2 = new SMap.Pixel(Math.max(this._point1.x, this._point2.x), Math.max(this._point1.y, this._point2.y));

    // original cleanup
  	this._ecTmp.forEach(JAK.Events.removeListener, JAK.Events);
  	this._ecTmp = [];
   	this._hide();    

    // make sure something is selected
    var selectedSomething = (p1.x != p2.x && p1.y != p2.y);
    
    if (selectedSomething)
      this.customHandler(p1, p2, this.customId, direction);
    else if (this.customFailDrag)
      this.customFailDrag(p1, p2, this.customId, direction);
  }
  
  // default: zoom to selection
  else
  {
    this._mouseupOrig(e, elm);
  }

  window.selectionActive = false;
});

// handle mouse move
encapsulate(SMap.Control.Selection.prototype, "_mousemove", function(e, elm)
{
  this.movedMouse = true;
  return(this._mousemoveOrig(e, elm));
})

// stop moving map using mouse if alt or shift is pressed (to enable setting selection on any mouse/key combination)
encapsulate(SMap.Control.Mouse.prototype, "_mouseDown", function(e, elm)
{
  if (e.shiftKey || e.altKey)
    return;
    
  return(this._mouseDownOrig(e, elm));
});

//----------------------------------------------------------------------------------------
// allow zooming 3 levels on Shift/Alt + middle scroll

encapsulate(SMap.Control.Mouse.prototype, "_scroll", function(e, elm)
{
  // Seznam code
 	if (e.wheelDeltaX) { return; } /* posun po vodorovne ose */
 	JAK.Events.cancelDef(e);
 	if (!this._checkScrollFrequency()) { return; }
 
 	var delta = e.wheelDelta || -e.detail;
 	var x = e.clientX;
 	var y = e.clientY;
 	
  // my code
  if (e.altKey || e.shiftKey)
  {
    this._zoom(delta > 0 ? "3" : "-3", x, y);
  }
  else
  // Seznam code
 	  this._zoom(delta > 0 ? "1" : "-1", x, y);
});


//----------------------------------------------


// global disabling/enabling of marker dragging (fixes some bugs in free draw polygon mode)
SMap.prototype.disableMarkerDragging = function()
{
  this.markerDraggingDisabled = true;
  
  this.markerDraggingFnBackup = SMap.Marker.Feature.Draggable.prototype._dragStart;
  SMap.Marker.Feature.Draggable.prototype._dragStart = function(){};
}

SMap.prototype.enableMarkerDragging = function()
{
  if (!this.markerDraggingFnBackup)
    return;

  this.markerDraggingDisabled = false;
  SMap.Marker.Feature.Draggable.prototype._dragStart = this.markerDraggingFnBackup;
}

/****** add marker dblclick ******/
encapsulate(SMap.Layer.Marker.prototype, "_build", function()
{
  this._buildOrig();
  this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_ACTIVE], "dblclick", this, "_dblclick"));
  this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_MARKER], "dblclick", this, "_dblclick"));
});

SMap.Layer.Marker.prototype._dblclick = function(e, elm) 
{
 	var t = JAK.Events.getTarget(e);
 	var tops = [];
 	for (var p in this._dom.container) { tops.push(this._dom.container[p]); }
 	
 	for (var id in this._markers) {
 		var m = this._markers[id].marker;
 		var node = t;
 		var a = m.getActive();
 		do {
 			if (node == a) 
       {
 				m.dblclick(e, elm, "x");
 				return;
 			}
 			node = node.parentNode;
 		} while (tops.indexOf(node) == -1);
 	}
}

SMap.Marker.prototype.dblclick = function(e, elm)
{
  this.getMap().getSignals().makeEvent("marker-dblclick", this, {event:e});
};



//******************
// support contexmenu event for geometry-click

encapsulate(SMap.Layer.Geometry.prototype, "_build", function()
{
  // original
 	this._canvas = JAK.Vector.getCanvas(1, 1); /* tenhle canvas pouzijeme jen pro vyrobu prvku, ale cpeme je do rodicovskeho */
 	var group = this._canvas.group();
 	group.id = this.id+"-"+SMap.LAYER_GEOMETRY;
 
 	this._ec.push(JAK.Events.addListener(group, "click", this, "_click"));
 	this._ec.push(JAK.Events.addListener(group, "mousedown", this, "_mouseDown"));
 
 	this._dom.container[SMap.LAYER_GEOMETRY] = group;
 	this._canvas.setContent(group);

  // our
  this._ec.push(JAK.Events.addListener(group, "contextmenu", this, "_click"));
});

//******************************************************

SMap.Projection.ArcGIS = JAK.ClassMaker.makeClass(
{
	NAME: "SMap.Projection.ArcGIS",
 	VERSION: "1.0",
 	EXTEND: SMap.Projection
});

SMap.Projection.ArcGIS.prototype.$constructor = function(tileInfo, fullExtent, delta)
{
  this._owner = null;
  
  this.tileInfo = tileInfo;
  this.fullExtent = fullExtent;

  this._code = "EPSG:" + this.tileInfo.spatialReference.wkid;
  
  // offset to normal zoom (so that we don't start with 0)
  this.delta = delta || 8;
  
  this._screenCoefficients = {x:1, y:1};  
}

SMap.Projection.ArcGIS.prototype._absolutePixelToCoords = function(pixel, zoom)
{
  var pixelSize = this.tileInfo.lods[zoom - this.delta].resolution;
  var x = pixel.x * pixelSize + this.tileInfo.origin.x;
  var y = -pixel.y * pixelSize + this.tileInfo.origin.y;

  // S-JTSK
  if (this.getCode() == "EPSG:102067")
    return SMap.Coords.fromSJTSK(-x, -y);
    
  // Puwg1992 (Poland)
  else if (this.getCode() == "EPSG:2180")
    return SMap.Coords.fromPuwg1992(x, y);
}

SMap.Projection.ArcGIS.prototype._coordsToAbsolutePixel = function(coords, zoom)
{
  if (zoom - this.delta < 0)
    throw("zoom error (ArcGIS)");
    
  var x, y;

  if (this.getCode() == "EPSG:102067")
  {  
    var coord = coords.toSJTSK();
    x = -coord.x;
    y = -coord.y; 
  }
  else if (this.getCode() == "EPSG:2180")
  {
    var coord = coords.toPuwg1992();
    x = coord.x;
    y = coord.y;
  }

 //   try{
  var pixelSize = this.tileInfo.lods[zoom - this.delta].resolution;
 //  } catch(ex){alert(zoom + " / " + this.delta + " / " + this.getMap().getProjection().getCode())};

  x = (x - this.tileInfo.origin.x) / pixelSize;
  y = -(y - this.tileInfo.origin.y) / pixelSize;

  return new SMap.Pixel(Math.round(x), Math.round(y)); 
}

//------------------------
//******************************************
SMap.Bubble = JAK.ClassMaker.makeClass
({
  NAME: "SMap.Bubble",
  VERSION: "1.0",
  IMPLEMENT: [SMap.IOwned],
  /*DEPEND: [
  {
  	sClass: JAK.Window,
  	ver: "2.0"
  }]*/
});
 
SMap.Bubble.prototype.$constructor = function(options)
{
  // set options
  this.options = 
  {
    html: "",
    background: "#C4E2F7",
    fontSize: 12,
    minWidth: 100,
    timeout: 100, /* bubble is hidden after mouseout after timeout ms */
    alwaysShow: false  
  }
  
  for (var param in (options || {}))
    this.options[param] = options[param];
  
  // create bubble div
  this.dom = JAK.mel("div", {className: "markerBubble"},
    {display: "none", background: this.options.background, fontSize: this.options.fontSize + "px",
    minWidth: this.options.minWidth + "px"});
  
  this.dom.innerHTML = this.options.html;
}

/* set bubble HTML */
SMap.Bubble.prototype.setHTML = function(html)
{
  this.options.html = html || "";
  this.dom.innerHTML = this.options.html;
}

/* force show */
SMap.Bubble.prototype.show = function()
{
  this.dom.style.display = "";
  this.afterMouseOut = false;
  
  // old IE would select all text on the page --> prevent that
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 9)
    document.selection.empty();  
}

/* force hide */
SMap.Bubble.prototype.hide = function()
{
  this.dom.style.display = "none";
}

SMap.Marker.Feature.Bubble = JAK.ClassMaker.makeSingleton(
{
  NAME: "SMap.Marker.Feature.Bubble",
  VERSION: "1.0",
  EXTEND: JAK.AbstractDecorator
});
 
SMap.Marker.Feature.Bubble.prototype.decorate = function(marker, bubble)
{
  this.$super(marker);

  // bubble already exists?
  if (marker.bubble)
    throw("Marker already has a bubble!");
                                  
  marker.bubble = bubble;

  // add bubble div to marker
  var markerDiv = marker.getContainer()[SMap.LAYER_MARKER];
  markerDiv.appendChild(bubble.dom);

  // set display/hide handlers
  if (!marker.bubble.options.alwaysShow)
  {
    marker._bubbleOver = this._mouseOver;
    marker._bubbleOut = this._mouseOut;
    marker._ec.bubble_over = JAK.Events.addListener(markerDiv, "mouseover", marker, "_bubbleOver");
    marker._ec.bubble_out = JAK.Events.addListener(markerDiv, "mouseout", marker, "_bubbleOut");
    
    marker.oldZIndex = NaN;
  }
  
  return marker;
}

/* mouseover - show */
SMap.Marker.Feature.Bubble.prototype._mouseOver = function(e, elm)
{
  this.bubble.dom.style.display = "";
  
  // old IE would select all text on the page --> prevent that
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 9)
    document.selection.empty();
  
  /* timeout after mouseout */
  this.bubble.afterMouseOut = false;
  
  // beef up marker z-index, so that the bubble is always on top
  var markerDiv = this.getContainer()[SMap.LAYER_MARKER];
  
  if (isNaN(this.oldZIndex))
    this.oldZIndex = markerDiv.style.zIndex;

  markerDiv.style.zIndex = 10;
}

/* mouseout - hide */
SMap.Marker.Feature.Bubble.prototype._mouseOut = function(e, elm)
{
  this.bubble.afterMouseOut = true;
  var that = this;

  window.setTimeout(function()
  {
    if (that.bubble.afterMouseOut)
    {
      that.bubble.dom.style.display = "none";
      
      // restore marker z-index
      if (!isNaN(that.oldZIndex))
      {
        that.getContainer()[SMap.LAYER_MARKER].style.zIndex = that.oldZIndex;
        that.oldZIndex = NaN;
      }      
    }
  }, this.bubble.options.timeout);
}

window.mapyCZ_Mod_Loaded = true;