function EventTarget(){
    this.handlers = {};    
}
                   
EventTarget.prototype = {
    constructor: EventTarget,
                   
    addHandler: function(type, handler){
        if (typeof this.handlers[type] == "undefined"){
            this.handlers[type] = [];
        }
                   
        this.handlers[type].push(handler);
    },
    
    fire: function(event){
        if (!event.target){
            event.target = this;
        }
        if (this.handlers[event.type] instanceof Array){
            var handlers = this.handlers[event.type];
            for (var i=0, len=handlers.length; i < len; i++){
                handlers[i](event);
            }
        }            
    },
                   
    removeHandler: function(type, handler){
        if (this.handlers[type] instanceof Array){
            var handlers = this.handlers[type];
            for (var i=0, len=handlers.length; i < len; i++){
                if (handlers[i] === handler){
                    break;
                }
            }
            
            handlers.splice(i, 1);
        }            
    }
};

var DragDrop = function()
{
  var dragdrop = new EventTarget(),
      dragging = null,
      diffX = 0,
      diffY = 0;
      
  var offsetStartX, offsetStartY;
  
  function handleEvent(event, target)
  {
    //determine the type of event
    switch(event.type)
    {
      case "mousedown":
        var hasClass = false;
        try {hasClass = contains_class(target, "isDraggable");} catch(ex){};
      
        if (hasClass)
        {
          dragging = target;
          diffX = event.clientX - target.offsetLeft;
          diffY = event.clientY - target.offsetTop;
          dragdrop.fire({type:"dragstart", target: dragging, x: event.clientX, y: event.clientY});
          offsetStartX = event.clientX;
          offsetStartY = event.clientY;
        }                     
        break;
          
      case "mousemove":
        if (dragging !== null)
        {
          //assign location
          //dragging.style.left = (event.clientX - diffX) + "px";
          //dragging.style.top = (event.clientY - diffY) + "px";   
         
          //fire custom event
          dragdrop.fire({type:"drag", target: dragging, x: event.clientX, y: event.clientY, 
            deltaX: event.clientX - offsetStartX, deltaY: event.clientY - offsetStartY});
        }                    
        break;
          
      case "mouseup":
        if (dragging !== null)
        {
          dragdrop.fire({type:"dragend", target: dragging, x: event.clientX, y: event.clientY,
            deltaX: event.clientX - offsetStartX, deltaY: event.clientY - offsetStartY});
          dragging = null;
        }
        break;
    }
  };
  
  //public interface
  dragdrop.enable = function()
  {
    EventUtil.addHandler(document, "mousedown", handleEvent);
    EventUtil.addHandler(document, "mousemove", handleEvent);
    EventUtil.addHandler(document, "mouseup", handleEvent);
  };
      
  dragdrop.disable = function()
  {
    EventUtil.removeHandler(document, "mousedown", handleEvent);
    EventUtil.removeHandler(document, "mousemove", handleEvent);
    EventUtil.removeHandler(document, "mouseup", handleEvent);
  };
  
  return dragdrop;
}();


// // minimap._dom.container.style.width = "400px";
// minimap._dom.container.firstChild.style.width = "400px";
// minimap._dom.container.draggable = true;
// minimap._dom.container.firstChild.draggable = true;

 /**
  * @class Náhledové mini-api s viditelným průhledem
  * @augments SMap.Control.Visible
  */
 SMap.Control.Minimap = JAK.ClassMaker.makeClass({
 	NAME: "SMap.Control.Minimap",
 	VERSION: "1.0",
 	EXTEND: SMap.Control.Visible
 });
 
 /**
  * @param {int} [width=150] Šířka
  * @param {int} [height=150] Výška
  * @param {object} [options] Konfigurace
  * @param {int} [options.diff=-4] Rozdíl zoomu oproti aktuálnímu
  * @param {int} [options.layer=SMap.DEF_BASE] Podklad
  * @param {string} [options.color="gray"] Barva
  * @param {string} [options.opacity=0.5] Průhlednost
  */


SMap.Control.Minimap.prototype.$constructor = function(width, height, options) 
{
 	this.$super();
 	this._options = 
  {
  	diff: -4,
  	opacity: 0.5,
  	layer: SMap.DEF_BASE,
  	color: "gray"
 	}
 	
 	var w = width || 150;
 	var h = height || 150;
 	this._map = null;
 	this._layer = new SMap.Layer.Geometry();
 
 	this._dom.container = JAK.mel("div", {className:"minimap"}, {width:w+"px", height:h+"px"});

  // handle zoom settings
  options.relativeZoom = (/[+\-]/.test(options.zoom));
  options.negativeZoomDif = (/[\-]/.test(options.zoom));
  options.zoom = parseInt(options.zoom);  
 
  // minimap style  
  if (options.borderWidth)
    this._dom.container.style.borderWidth = options.borderWidth;
  
  this._dom.container.style.background = "white";

  // move width / height to _options
  this._options.width = w;
  this._options.height = h;

  var that = this;
  this.resizeDiv = JAK.mel("div", {className: "resizeDiv"});
  this._dom.container.appendChild(this.resizeDiv);

  // *** change minimap size on dragging the resize button ***
  add_class(this.resizeDiv, "isDraggable");
  DragDrop.enable();
  
  // handlers for removal
  this.dragHandlers = [];
  this.eventUtilHandlers = [];
  
  this.dragHandlers.push(DragDrop.addHandler("drag", function(e)
  {
    if (e.target !== that.resizeDiv)
      return;

    that.wasDragged = true;

    var newWidth = that.dragStartWidth - e.deltaX;
    var newHeight = that.dragStartHeight - e.deltaY;
    
    // minimum size
    newWidth = Math.max(newWidth, 100);
    newHeight = Math.max(newHeight, 100);
        
    // maximum size (smaller than screen size)
    newWidth = Math.min(newWidth, that.getMap().getContainer().offsetWidth - 1 /* border */);
    newHeight = Math.min(newHeight, that.getMap().getContainer().offsetHeight - 1 /* border */);

    that.setOptions({width: newWidth, height: newHeight});
  }));
  
  this.dragHandlers.push(DragDrop.addHandler("dragstart", function(e, target)
  {
    if (e.target !== that.resizeDiv)
      return;  
  
    that.dragStartWidth = that._dom.container.offsetWidth;
    that.dragStartHeight = that._dom.container.offsetHeight;
    that.wasDragged = false;
    that.resizeDiv.style.cursor = "move";
  }));  
  
  this.dragHandlers.push(DragDrop.addHandler("dragend", function(e)
  {
    if (e.target !== that.resizeDiv)
      return;  

    // close on click
    if (!that.wasDragged)
    {
      closeMinimap();
    }
    
    that.resizeDiv.style.cursor = "pointer";
  }));    
  
  // *** center movable div ***
  this.centerDiv = JAK.mel("div", {className: "minimapCenter"});
    
  this._dom.container.appendChild(this.centerDiv);
  add_class(this.centerDiv, "isDraggable");
  
  this.dragHandlers.push(DragDrop.addHandler("drag", function(e)
  {
    if (e.target !== that.centerDiv)
      return;

    that.getMap().setCenter((new SMap.Pixel(e.deltaX, e.deltaY)).toCoords(that._map));

    // old IE would select all text on the page --> prevent that
    if (JAK.Browser.client == "ie" && JAK.Browser.version <= 9)
      document.selection.empty();    
    
  }));

  this.dragHandlers.push(DragDrop.addHandler("dragstart", function(e)
  {
    if (e.target !== that.centerDiv)
      return;  
  
    that.dragCenterActive = true;
    that.centerDiv.style.cursor = "move";
  }));

  this.dragHandlers.push(DragDrop.addHandler("dragend", function(e)
  {
    if (e.target !== that.centerDiv)
      return;

    that.dragCenterActive = false;
    that.centerDiv.style.cursor = "pointer";
    that._mapRedraw();
    
  }));

  // original
  this.setOptions(options);
}

SMap.Control.Minimap.prototype.addTransparentLayers = function()
{
  for (var layer in transparentLayers)
  {
    // change SMap.DEF_OPHOTO to SMap.DEF_SMART_OPHOTO etc.
    var mainLayer = this.smartizeMapLayer(this._options.layer);

    if (settings.transparent_layers[mainLayer][layer])
    {
      // display only some transparent layers
       if (layer == "hybrid" || layer == "trail" || layer == "bike"
         || layer == "g_hybrid" /*|| layer == "amapy_tur" || layer == "amapy_cyklo"*/)
        this._map.addDefaultLayer(transparentLayers[layer]).enable();
    }
  }
}

SMap.Control.Minimap.prototype.removeTransparentLayers = function()
{
  if (!this._map)
    return;

  for (var i in transparentLayers)
  {
  	var layer = this._map.getLayer(transparentLayers[i]);
    
    if (!layer)
      continue;
    
  	layer.disable();
  	this._map.removeLayer(layer);
  	layer.$destructor();
  }
}

SMap.Control.Minimap.prototype.setOwner = function(owner)
{
 	this.$super(owner);
  
  // fix for 2014-8-23 version
  if(!owner)
    return;
  
 	var map = this.getMap();

 	var signals = map.getSignals();
 	this._sc.push(signals.addListener(this, "map-redraw", "_mapRedraw"));
  
 	this._map = new SMap(this._dom.container, map.getCenter(), 14);
 	while (1) {
 		var controls = this._map.getControls();
 		if (!controls.length) { break; }
 		this._map.removeControl(controls[0]);
 	}
  
  // allow mouse pan
  this._map.addControl(new SMap.Control.Mouse(SMap.MOUSE_PAN));

  // *** change position on click ***
  var that = this;
  
  this._map.getSignals().addListener(this, "map-click", function(e, target)
  {
    if (!window.contextMenuActive)
      that.getMap().setCenter(new SMap.Coords.fromEvent(e.data.event, that._map));
  });
 
  // *** monitor mouse move ***
  this.lastMouseMoveCoords = map.getCenter();
  
  this.eventUtilHandlers.push(EventUtil.addHandler(map.getContent(), "mousemove", function(e, target)
  {
    that.lastMouseMoveCoords = new SMap.Coords.fromEvent(e, map);
    
    if (that._map.getZoom() > that.getMap().getZoom() || that._options.zoom === "+0")
      that._mapRedraw();
    
    // set center marker
    if (that.markerLayer)
    {
      that._map.removeLayer(that.markerLayer);
      that.markerLayer = null;
    }

    if (that._map.getZoom() >= that.getMap().getZoom())
    {
      that.markerLayer = new SMap.Layer.Marker();
      that._map.addLayer(that.markerLayer).enable();
      that.markerLayer.enable();
      
      // create marker
      var markerDiv = JAK.mel("img", {src: basePath + "Images/Minimap/RedCross.png", className: "redCrossMarker"});
      var marker = new SMap.Marker(that.lastMouseMoveCoords, null, {url: markerDiv, anchor: {left: 15, top: 15}});
      that.markerLayer.addMarker(marker);
    }
  }));
  
  // *** monitor mouseup (to reposition drag rectangle on map drag)
  this.eventUtilHandlers.push(EventUtil.addHandler(window, "mouseup", function(e, target)
  {
    that.positionDragRectangle();
  }));

  // *** create context menu ***
  translations["_minimap_type"] = {"Czech": "Vyber typ mapy", "English": "Choose map type"};
  translations["_minimap_same_map"] = {"Czech": "jako hlavní mapa", "English": "like the main map"};
  translations["_minimap_zoom"] = {"Czech": "Vyber zoom mapy", "English": "Choose map zoom"};
  translations["_minimap_relative_zoom"] = {"Czech": "Relativní k mapě", "English": "Relative to map&nbsp;&nbsp;&nbsp;" /* make same length */};
  translations["_minimap_absolute_zoom"] = {"Czech": "Absolutní", "English": "Absolute"};
  translations["_minimap_close"] = {"Czech": "Zavřít", "English": "Close"};
  translations["_minimap_help"] = {"Czech": "Nápověda", "English": "Help"};  

  translations["_minimap_help_text"] = 
  {
    "Czech": 
      "Nacházíte se v nápovědě minimapy.\n" +
      "\n" +
      "Typ mapy, který minimapa používá, můžete změnit na pravém tlačítku pod položkou \"Vyber typ mapy\". Položka \"jako hlavní mapa\" znamená, že typ minimapy bude synchronizován s hlavní mapou, ostatní položky nastaví typ minimapy napevno.\n" +
      "\n" +
      "Zoom můžete nastavit také na pravém tlačítku. Zde můžete nastavit buď absolutní hodnoty nebo hodnoty relativní (tedy přičítající se k zoomu hlavní mapy). Minimapa funguje ve dvou různých módech - pokud je zoom minimapy menší nebo stejný než zoom mapy, tak se střed minimapy nastaví na střed mapy; v opačném případě se střed mapy nastaví tam, kde je právě myš. Rozdíl mezí +0 a -0 je právě tento. Zoomovat je možno i kolečkem. Typ zoomu (tedy relativní či absolutní) se při skrolování zachovává.\n" +
      "\n" +
      "V prvním módu (tedy zoom mapy je větší než zoom minimapy) máte výřez mapy zobrazen v červeném obdélníčku. Tento můžete tažením posouvat a zobrazovat tak to, co je momentálně pod obdélníčkem. Po puštění myši se na nové místo vycentruje jak mapa, tak minimapa.\n" +
      "\n" +
      "Kliknutím na místo na mapě (mimo červený obdélníček) se na daném místě obě mapy vycentrují.\n" +
      "\n" +
      "Tažením levým tlačítkem je také možno se posouvat za hranice minimapy. Tam můžete využít i výše zmíněnou funkci.\n" +
      "\n" +
      "Pokud má minimapa zoom stejný nebo vyšší než mapa, tak se na místě myši automaticky objevuje červený kříž. Ten můžete využít účelně třeba v menu track pro přidávání nebo přesun bodů.\n" +
      "\n" +
      "Velikost minimapy můžete libovolně měnit tažením ikonky s modrými šipkami. Kliknutí na tuto ikonku minimapu otevírá a zavírá.\n" +
      "\n" +
      "Nastavení se mezi jednotlivými spuštěními mapy ukládají. Pokud používáte verzi v GSAKu s malou mapou, tak tato má svoje vlastní nastavení nezávislé na nastavení \"velké\" mapy.",  

   "English":
      "You can set map type under \"Choose map type\" option in minimap context menu. The options \"like the main map\" means, that the minimap type will be synchronized with the main map; otherwise it will stay no matter the main map type.\n" +
      "\n" +
      "Zoom can also be set in the context menu. You can set either absolute value or a relative one (added to the main map's zoom). The minimap works in two modes - if map's zoom is higher than that of the minimap, then minimap center will be synchronized with mouse position. Otherwise it will be synchronized with the map center. This is also the difference between +0 and -0. You can also change zoom by scrolling. If you scroll while relative value is active, it will scroll it to relative value as well and vice-versa.\n" +
      "\n" +
      "In the first mode (map zoom higher) you can see a red rectangle showing where the map is. You can drag this rectangle to move the main map. On releasing the mouse, both map will be centered at the last position.\n" +
      "\n" +
      "Clicking on the map (outside the red rectangle) centers both maps there.\n" +
      "\n" +
      "You can also move outside of the minimap borders by left dragging. You can use the above function there.\n" +
      "\n" +
      "If the minimap has higher zoom than the map, a red cross appears on the map. Its position is synchronized with the mouse position.\n" +
      "\n" +
      "Minimap's size can be changed by dragging the icon with two blue arrows. The blue arrows icon is also used for showing/hiding the minimap (on click).\n" +
      "\n" +
      "Minimap settings are saved between sessions. Withing GSAK window, if you launch the \"small map\" (which is default), the minimap has separate settings there."
  }
 
  var menu_string = ""
  + "<ul class=\"minimap_context_menu context_menu\">"
  + " <li data-tr>_minimap_type<\/li>"
  + " <select class=minimap_choose_map>"
    + "<option value=_same_as_map selected data-tr>_minimap_same_map</option>"
    + "<option>---------------</option>"
    + "<option value=Letecká data-id=" + SMap.DEF_OPHOTO + ">Letecká</option>"
    + "<option value=Turistická data-id=" + SMap.DEF_TURIST + ">Turistická</option>"
    + "<option value=\"Turistická zimní\" data-id=" + SMap.DEF_TURIST_WINTER + ">Turistická - Zimní</option>"
    + "<option value=\"Turistická letní\" data-id=" + SMap.DEF_SUMMER + ">Turistická - Letní</option>"
    + "<option value=\"Zeměpisná\" data-id=" + SMap.DEF_GEOGRAPHY + ">Zeměpisná</option>"    
    + "<option value=Obecná data-id=" + SMap.DEF_BASE + ">Obecná</option>"
    + "<option value=\"Letecká '06\" data-id=" + SMap.DEF_OPHOTO0406 + ">Letecká '06</option>"
    + "<option value=\"Letecká '03\" data-id=" + SMap.DEF_OPHOTO0203 + ">Letecká '03</option>"
    //+ "<option value=\"Ortofoto 50. léta\" data-id=Orto_50_ID>Ortofoto 50. léta</option>"
    + "<option value=ZM10 data-id=ZM10_ID>ZM10</option>"
    + "<option value=\"Ortofoto ČR\" data-id=Orto_CZ_ID>Ortofoto ČR</option>"    
    + "<option value=IZS data-id=IZS_ID>IZS</option>"
    + "<option value=\"Vojenská mapa\" data-id=Vojenska_ID>Vojenska mapa</option>"
    + "<option value=\"Google Satellite\" data-id=GSatellite_ID>Google Satellite</option>"
    + "<option value=\"Google Maps\" data-id=GMaps_ID>Google Maps</option>"
    + "<option value=\"Google Terrain\" data-id=GTerrain_ID>Google Terrain</option>"
    + "<option value=OSM data-id=OSM_ID>OSM</option>"
    + "<option value=\"OSM Cyclo\" data-id=OSMCyclo_ID>OSM Cyclo</option>"
    + "<option value=\"OSM Cyclo\" data-id=MTBMap_ID>MTB Map</option>"
    + "<option value=\"Freemap SK\" data-id=FreemapSK_ID>Freemap SK</option>"
    + "<option value=\"Müllerova mapa\" data-id=Muller_ID>Müllerova mapa</option>"    
    + "<option value=\"II. voj. mapování\" data-id=" + SMap.DEF_HISTORIC + ">II. voj. mapování</option>"
    + "<option value=\"III. voj. mapování\" data-id=VojMapIII_ID>III. voj. mapování</option>"    
    //+ "<option value=\"aMapy\" data-id=aMapy_ID>aMapy</option>"
    //+ "<option value=\"ZM Slovensko\" data-id=ZM10_Slovakia_ID>ZM Slovensko</option>"
    //+ "<option value=\"Polsko Orto\" data-id=Poland_ID>Polsko Orto</option>"
  + " </select>"  
  + " <li data-tr>_minimap_zoom<\/li>"
  + " <select class=minimap_choose_zoom>"
    + "<option data-tr>_minimap_relative_zoom</option>"
    + "<option value=\"-0\">-0</option>"
    + "<option value=\"-1\">-1</option>"
    + "<option value=\"-2\">-2</option>"
    + "<option value=\"-3\" selected>-3</option>"
    + "<option value=\"-4\">-4</option>"
    + "<option value=\"-5\">-5</option>"
    + "<option value=\"-6\">-6</option>"
    + "<option value=\"-7\">-7</option>"
    + "<option value=\"-8\">-8</option>"
    + "<option value=\"-9\">-9</option>"
    + "<option value=\"-10\">-10</option>"
//     + "<option value=\"-11\">-11</option>"
//     + "<option value=\"-12\">-12</option>"
//     + "<option value=\"-13\">-13</option>"
//     + "<option value=\"-14\">-14</option>"
//     + "<option value=\"-15\">-15</option>"
//     + "<option value=\"-16\">-16</option>"
//     + "<option value=\"-17\">-17</option>"    
    + "<option>---------------</option>"
    + "<option value=\"+0\">+0</option>"
    + "<option value=\"+1\">+1</option>"
    + "<option value=\"+2\">+2</option>"
    + "<option value=\"+3\">+3</option>"
    + "<option value=\"+4\">+4</option>"
    + "<option value=\"+5\">+5</option>"
    + "<option value=\"+6\">+6</option>"
    + "<option value=\"+7\">+7</option>"
    + "<option value=\"+8\">+8</option>"
    + "<option value=\"+9\">+9</option>"
    + "<option value=\"+10\">+10</option>"
//     + "<option value=\"+11\">+11</option>"
//     + "<option value=\"+12\">+12</option>"
//     + "<option value=\"+13\">+13</option>"
//     + "<option value=\"+14\">+14</option>"
//     + "<option value=\"+15\">+15</option>"
//     + "<option value=\"+16\">+16</option>"
//     + "<option value=\"+17\">+17</option>"      
    + "<option>---------------</option>"
    + "<option data-tr>_minimap_absolute_zoom</option>"
    + "<option value=3>3</option>"
    + "<option value=4>4</option>"
    + "<option value=5>5</option>"
    + "<option value=6>6</option>"
    + "<option value=7>7</option>"
    + "<option value=8>8</option>"
    + "<option value=9>9</option>"
    + "<option value=10>10</option>"
    + "<option value=11>11</option>"
    + "<option value=12>12</option>"
    + "<option value=13>13</option>"
    + "<option value=14>14</option>"
    + "<option value=15>15</option>"
    + "<option value=16>16</option>"
    + "<option value=17>17</option>"
    + "<option value=18>18</option>"
    + "<option value=19>19</option>"
    + "<option value=20>20</option>"
  + " </select>"
  + " <li class=_minimap_help data-tr>_minimap_help<\/li>"    
  + " <li class=_minimap_close data-tr>_minimap_close<\/li>"  
  + "<\/ul>";

  this.contextMenuDiv = JAK.mel("div", {innerHTML: menu_string}).firstChild;
  document.body.appendChild(this.contextMenuDiv);
  translate_strings();
  
  this.contextMenu = contextMenu.addContextMenu(this._dom.container, this.contextMenuDiv, 
  function (e, target, menu, options)
  {
    // remove keypress propagation
    that.eventUtilHandlers.push(EventUtil.addHandler(that.contextMenuDiv, "keydown", function(e) {EventUtil.stopPropagation(e);}));
    that.eventUtilHandlers.push(EventUtil.addHandler(that.contextMenuDiv, "keyup", function(e) {EventUtil.stopPropagation(e);}));
    that.eventUtilHandlers.push(EventUtil.addHandler(that.contextMenuDiv, "keypress", function(e) {EventUtil.stopPropagation(e);}));
  },
  
  function (e, target, menu, options) 
  {
    if (contains_class(target, "_minimap_help"))
      ui("copybox", {text: tr("_minimap_help_text"), width: 600, height: 200});

    else if (contains_class(target, "_minimap_close"))
    ;

    else
      return true;
  });

  // handle map select box change
  var selectMap = this.contextMenuDiv.getElementsByTagName("select")[0];
  
  that.eventUtilHandlers.push(EventUtil.addHandler(this.contextMenuDiv.getElementsByTagName("select")[0], "change", function(e, target)
  {
    var id = target[target.selectedIndex].getAttribute("data-id");
    
    if (target.value == "_same_as_map")
      that._options.map = "_same_as_map";
    else if (id)
      that._options.map = id;

    that.setOptions(that._options);
  }));
  
  // handle zoom select box change
  that.eventUtilHandlers.push(EventUtil.addHandler(this.contextMenuDiv.getElementsByTagName("select")[1], "change", function(e, target)
  {
    if ((target.value || target.value === "-0" || target.value === "+0") /* IE7 */ && !isNaN(target.value) /* modern browsers */)
      that._options.zoom = parseInt(target.value);

    that._options.relativeZoom = (/[+\-]/.test(target.value));
    that._options.negativeZoomDif = (/[\-]/.test(target.value));
    //log(that._options.zoom + " / " + that._options.negativeZoomDif + " / " + that._options.relativeZoom);
    that.setOptions(that._options);
  }));  
 
  // set map on map change
  this._sc.push(signals.addListener(this, "layer-change-before", function(e, elm)
  //this._sc.push(signals.addListener(this, "layer-enable", function(e, elm)
  {
    that.setOptions(that._options);
  }));
  
  // set map also on checkbox change in sidebar (where we can change sub-layer on/off state)
  this._sc.push(signals.addListener(this, "sidebar-checkbox-change", function(e, elm)
  {
    if (that.desmartizeMapLayer(that.getMap().CurrentMapId) == that._options.layer)
      that.setOptions(that._options);
  }));  

  // change zoom on scroll
  function adjustZoomOnWheelScroll(e)
  {
    var wheelDelta = EventUtil.getWheelDelta(e);

    // relative zoom
    if (that._options.relativeZoom)
    {
      var delta = wheelDelta / Math.abs(wheelDelta);
      
      // +0 <-> -0
      if (that._options.zoom == 0 && that._options.negativeZoomDif != (delta < 0))
        that._options.negativeZoomDif = !that._options.negativeZoomDif;

      // all else
      else
        that._options.zoom = Math.min(Math.max(that._options.zoom + delta, -10), 10);
        
      //log("delta: " + (that._options.negativeZoomDif ? "-" : "+") + Math.abs(that._options.zoom))
    }

    // absolute zoom
    else
    {
      that._options.zoom += wheelDelta / 120;
      that._options.zoom = Math.min(Math.max(that._options.zoom, 3), 20);    
    }

    EventUtil.preventDefault(e);
    that.setOptions(that._options);
  }
  
  this._ec.push(JAK.Events.addListener(this._dom.container, "DOMMouseScroll", adjustZoomOnWheelScroll));
  this._ec.push(JAK.Events.addListener(this._dom.container, "mousewheel", adjustZoomOnWheelScroll));
  
  // write values to select on first run
  this.writeMapZoomToSelect();

  // redraw
  this._map.addDefaultLayer(this._options.layer);
  this._map.addLayer(this._layer).enable();
  this.setOptions();
}

SMap.Control.Minimap.prototype.writeMapZoomToSelect = function()
{
  // set map (we need to find option by data-id)
  var select = this.contextMenuDiv.getElementsByTagName("select")[0];
  
  for (var i = 0; i < select.length; i++)
  {
    if (select[i].getAttribute("data-id") == this._options.map)
    {
      select.value = select[i].value;
      break;
    }
  }

  if (this._options.map == "_same_as_map")
    select.value = this._options.map;

  // set zoom
  if (this._options.relativeZoom)
    var zoom = (this._options.negativeZoomDif ? "-" : "+") + Math.abs(this._options.zoom);
  else
    var zoom = this._options.zoom;

  this.contextMenuDiv.getElementsByTagName("select")[1].value = zoom;
}

// changes smart layer to a normal layer
SMap.Control.Minimap.prototype.desmartizeMapLayer = function(layer)
{
  if (layer == SMap.DEF_SMART_OPHOTO)
    return SMap.DEF_OPHOTO;
  else if (layer == SMap.DEF_SMART_BASE)
    return SMap.DEF_BASE;  
  else if (layer == SMap.DEF_SMART_TURIST)
    return SMap.DEF_TURIST;
  else
    return layer;
}

// ... and vice-versa
SMap.Control.Minimap.prototype.smartizeMapLayer = function(layer)
{
  if (layer == SMap.DEF_OPHOTO)
    return SMap.DEF_SMART_OPHOTO;
//   else if (layer == SMap.DEF_BASE)
//     return SMap.DEF_SMART_BASE;  
//   else if (layer == SMap.DEF_TURIST)
//     return SMap.DEF_SMART_TURIST;
  else
    return layer;
}

SMap.Control.Minimap.prototype.setMap = function()
{
  this._options.layer = (this._options.map == "_same_as_map") ? this.getMap().CurrentMapId : this._options.map;
  
  //alert(this._options.layer);
//   if (this._options.layer == 15)
//     this._options.layer = 3;
  
  
  this._options.layer = this.desmartizeMapLayer(this._options.layer);
  
  this._map.setProjection2(SMap.CONFIG[this._options.layer].projection || SMap.projUTM33);
}
 
/**
* Změna nastavení za běhu
* @param {object} options Viz poslední parametr konstruktoru
*/
SMap.Control.Minimap.prototype.setOptions = function(options)
{
  if (this._map)
  {
  	var layer = this._map.getLayer(this._options.layer);
  	layer.disable();
  	this._map.removeLayer(layer);
  	layer.$destructor();
  }

  // remove transparent layer (if any)
  this.removeTransparentLayers();

  for (var p in options) {this._options[p] = options[p];}
  this._dom.container.borderColor = this._options.color;

  // handle width/height change
  this._dom.container.style.width = this._options.width + "px";
  this._dom.container.style.height = this._options.height + "px";

  if (this._map)
  {
    this._map.syncPort();
    this.setMap();
    
    var l = this._map.addDefaultLayer(this._options.layer);

  	this._mapRedraw(l);
    
    // add transparent layers
    this.addTransparentLayers();
        
    l.enable();

    this.writeMapZoomToSelect();

    // on resize
    this.getMap()._signals.makeEvent("minimap-resize", this._dom.container); 
  }
}

// zooming min/max restrictions don't work in minimap, fix it
SMap.Control.Minimap.prototype.fixZoom = function(zoom)
{
  var mapConfig = SMap.CONFIG[this._options.layer].zoom;

  if (mapConfig)
  {
    zoom = Math.min(mapConfig[1], zoom);
    zoom = Math.max(zoom, mapConfig[0]);
  }
  
  return zoom;
}

// alternative minimap mode: zoom based on map move
SMap.Control.Minimap.prototype.magnify = function()
{
  var map = this.getMap();
  var newZoom = (this._options.relativeZoom ? map.getZoom() : 0) + this._options.zoom;

  if (newZoom < map.getZoom() || (this._options.relativeZoom && this._options.negativeZoomDif))
    return false;

  if (typeof this.lastMouseMoveCoords !== 'undefined')
  {
    this._map.setCenterZoom(this.lastMouseMoveCoords, this.fixZoom(newZoom));
  }

  this._layer.removeAll(); 
 
//   // turn off selection
//   if (window.getSelection && JAK.Browser.client != "opera" && JAK.Browser.client != "chrome")
//     window.getSelection().deleteFromDocument();

  return true;
}

// re-position drag rectangle
SMap.Control.Minimap.prototype.positionDragRectangle = function()
{
  var rect = this.rectInfo;
  
  if (!rect)
    return;
  
  var ltPix = rect.lt.toPixel(this._map);

  this.centerDiv.style.width = ((rect.wWhole ? rect.rb1 : rect.rb).toPixel(this._map).x - ltPix.x) + "px";
  this.centerDiv.style.height = ((rect.hWhole ? rect.rb3 : rect.rb).toPixel(this._map).y - ltPix.y) + "px";  
  this.centerDiv.style.top = (this._map.getContainer().offsetHeight / 2 + ltPix.y) + "px";
  this.centerDiv.style.left = (this._map.getContainer().offsetWidth / 2 + ltPix.x) + "px";
}

SMap.Control.Minimap.prototype._mapRedraw = function() 
{
//   if (this.redrawTimeoutBlocked)
//     return;
//   
//   this.redrawTimeoutBlocked = true;
//   var that = this;
//     
//   window.setTimeout(function()
//   {
//     that.redrawTimeoutBlocked = false;
//   }, 200);


  //log(this._sc.length);
  
  // turn off red cross
  if (this.markerLayer)
  {
    this._map.removeLayer(this.markerLayer);
    this.markerLayer = null;
  }

  if (this.magnify())
    return false;

  var map = this.getMap();
  
  // add difference for relative zoom
  if (this._options.relativeZoom)
    var zoom = map.getZoom() + this._options.zoom;
  // otherwise set absolute zoom (degrades gracefully when zoom not supported)
  else
    var zoom = this._options.zoom;
    
  zoom = this.fixZoom(zoom)

  //if (zoom == 19) { zoom = 18; }
  //zoom += this._options.diff;

  this._layer.removeAll();
  var size = map.getSize();
  var w = Math.round(size.x/2);
  var h = Math.round(size.y/2);

  var width = this._dom.container.offsetWidth;
  var height = this._dom.container.offsetHeight;
  
  var lt = new SMap.Pixel(-w, -h).toCoords(map);
  var rt = new SMap.Pixel( w, -h).toCoords(map);
  var lb = new SMap.Pixel(-w,  h).toCoords(map);
  var rb = new SMap.Pixel( w,  h).toCoords(map);

  var rb1 = new SMap.Pixel(w - width, h).toCoords(map);
  var rb2 = new SMap.Pixel(w - width, h - height).toCoords(map);
  var rb3 = new SMap.Pixel(w, h - height).toCoords(map);

  // don't change position when drag is active
  var hWhole = 0//(Math.abs(width - w * 2) < 20);
  var wWhole = 0//(Math.abs(height - h * 2) < 20);
  
  
  if (!this.dragCenterActive)
  {
    // adjust width and height if we cover entire (or almost entire) screen width or height
    var center = map.getCenter().toPixel(this._map, zoom).plus(new SMap.Pixel(wWhole ? (-width / 2) : 0, hWhole ? (-height / 2) : 0)).toCoords(this._map, zoom);
   
    this._map.setCenterZoom(center, zoom);
  }    

  // set draggable center
  if (!this.dragCenterActive)
  {
    var ltPix = lt.toPixel(this._map);

    this.centerDiv.style.width = ((wWhole ? rb1 : rb).toPixel(this._map).x - ltPix.x) + "px";
    this.centerDiv.style.height = ((hWhole ? rb3 : rb).toPixel(this._map).y - ltPix.y) + "px";  
    this.centerDiv.style.top = (this._map.getContainer().offsetHeight / 2 + ltPix.y) + "px";
    this.centerDiv.style.left = (this._map.getContainer().offsetWidth / 2 + ltPix.x) + "px";
  }
  
  this.rectInfo = {lt: lt, rt: rt, lb: lb, rb: rb, rb1: rb1, rb2: rb2, rb3: rb3, hWhole: hWhole, wWhole: wWhole};

  size = this._map.getSize();
  var w = Math.round(size.x/2);
  var h = Math.round(size.y/2);

  var LT = new SMap.Pixel(-w, -h).toCoords(this._map);
  var RT = new SMap.Pixel( w, -h).toCoords(this._map);
  var LB = new SMap.Pixel(-w,  h).toCoords(this._map);
  var RB = new SMap.Pixel( w,  h).toCoords(this._map);
 
  this._layer.disable();
 
  var g = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, [lt, rt, rb3, rb2, rb1, lb, lt, LT, LB, RB, RT, LT], {outlineOpacity:0, color:/*this._options.color*/ "grey", opacity:this._options.opacity});
  this._layer.addGeometry(g);

  var g = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, [lt, rt, rb3, rb2, rb1, lb], {outlineColor:this._options.color, outlineOpacity:0.8, color: "transparent", opacity: 0});
  this._layer.addGeometry(g);
  
  this._layer.enable();
  
//   // turn off selection
//   if (window.getSelection && JAK.Browser.client != "opera" && JAK.Browser.client != "chrome")
//     window.getSelection().deleteFromDocument();

  //this._map.unlock();
  //this._map.redraw();
}


//  SMap.Control.Minimap.prototype.setOwner = function(owner) {
//  	this.$super(owner);
//  	var map = this.getMap();
//  	var signals = map.getSignals();
//  	this._sc.push(signals.addListener(this, "map-redraw", "_mapRedraw"));
//  	
//  	this._map = new SMap(this._dom.container, map.getCenter(), 14);
//  	while (1) {
//  		var controls = this._map.getControls();
//  		if (!controls.length) { break; }
//  		this._map.removeControl(controls[0]);
//  	}
//  	this._map.addDefaultLayer(this._options.layer).enable();
//  	this._map.addLayer(this._layer).enable();
//  	
//  	this._mapRedraw();
//  }
// 
//  /**
//   * Změna nastavení za běhu
//   * @param {object} options Viz poslední parametr konstruktoru
//   */
//  SMap.Control.Minimap.prototype.setOptions = function(options) {
//  	if (this._map) {
//  		var layer = this._map.getLayer(this._options.layer);
//  		layer.disable();
//  		this._map.removeLayer(layer);
//  		layer.$destructor();
//  	}
//  
//  	for (var p in options) { this._options[p] = options[p]; }
//  	this._dom.container.borderColor = this._options.color;
//  	
//  	if (this._map) { 
//  		this._map.addDefaultLayer(this._options.layer).enable();
//  		this._mapRedraw(); 
//  	}
//  }
// 
//  SMap.Control.Minimap.prototype._mapRedraw = function() {
//  //log("ccccredraw");
//  	var map = this.getMap();
//  	var zoom = map.getZoom();
//  	if (zoom == 19) { zoom = 18; }
//  	zoom += this._options.diff;
//  	this._map.setCenterZoom(map.getCenter(), zoom);
//  	
//  	this._layer.removeAll();
//  	var size = map.getSize();
//  	var w = Math.round(size.x/2);
//  	var h = Math.round(size.y/2);
//  	
//  	var lt = new SMap.Pixel(-w, -h).toCoords(map);
//  	var rt = new SMap.Pixel( w, -h).toCoords(map);
//  	var lb = new SMap.Pixel(-w,  h).toCoords(map);
//  	var rb = new SMap.Pixel( w,  h).toCoords(map);
//  	
//  	size = this._map.getSize();
//  	var w = Math.round(size.x/2);
//  	var h = Math.round(size.y/2);
//  
//  	var LT = new SMap.Pixel(-w, -h).toCoords(this._map);
//  	var RT = new SMap.Pixel( w, -h).toCoords(this._map);
//  	var LB = new SMap.Pixel(-w,  h).toCoords(this._map);
//  	var RB = new SMap.Pixel( w,  h).toCoords(this._map);
//  	
//  	var g = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, [lt, rt, rb, lb, lt, LT, LB, RB, RT, LT], {outlineOpacity:0, color:this._options.color, opacity:this._options.opacity});
//  	this._layer.addGeometry(g);
//  
//  	var g = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, [lt, rt, rb, lb], {outlineColor:this._options.color, outlineOpacity:0.8, color:"transparent"});
//  	this._layer.addGeometry(g);
//  }


//**************************************************************************************
function startMinimap()
{
  var prefix = SplitScreen ? "minimap_" : "minimap_small_";
  var width = Math.max(Math.min(m.getContainer().offsetWidth, settings[prefix + "width"] || 250), 50);
  var height = Math.max(Math.min(m.getContainer().offsetHeight, settings[prefix + "height"] || 250), 50);
  var map = settings[prefix + "map"] || "_same_as_map";
  var zoom = settings[prefix + "zoom"] || "-3";

  var minimap = new SMap.Control.Minimap(width, height, {diff: -3, color: "red", opacity: 0.3/*, layer: "ZM10_ID"*/, 
    borderWidth: "1px 0 0 1px", map: map, zoom: zoom});
    
  m.addControl(minimap, {right:"0px", bottom:"0px"});
  window.minimap = minimap;
}

function closeMinimap()
{
  if (!window.minimap)
    return;

  var mm = window.minimap;

  mm.getMap()._signals.makeEvent("minimap-end", mm);

  while(mm.dragHandlers.length)
    DragDrop.removeHandler(mm.dragHandlers.pop());
  
  DragDrop.disable();
  
  while(mm.eventUtilHandlers.length)
    EventUtil.removeHandler(mm.eventUtilHandlers.pop());      
  
  // remove context menu
  if(mm.contextMenuDiv)
  {
    contextMenu.hideAllContextMenus();
    contextMenu.removeContextMenu(mm.contextMenu);
    mm.contextMenuDiv.parentNode.removeChild(mm.contextMenuDiv);
    mm.contextMenuDiv = null;
  }
  
  mm.getMap().removeControl(mm);
  mm.$destructor();    
  
  window.minimap = null;
  //log("@@@@@@@");
}


window.minimapLoaded = true;
 
// 
