// documentation for Street View:
// https://developers.google.com/maps/documentation/javascript/streetview#StreetViewService
// https://developers.google.com/maps/documentation/javascript/reference

var StreetViewDiv = null;
var StreetViewJSLoaded = false;
var StreetViewMap = null;
var markerStreetViewLayer = null;
var StreetViewMarker = null;
var StreetViewMarkerDiv = null;
var svSizeButton;
var lastRotation = 0;
StreetViewInit();

function LoadStreetView(coords)
{
  // on first start, get value from global variable
  if (window.startStreetViewCoords)
  {
    coords = window.startStreetViewCoords;
    window.startStreetViewCoords = null;
  }

  var lat = coords.lat();
  var lon = coords.lon();

  if (!StreetViewJSLoaded)
  {
    StreetViewJSLoaded = true;

    StreetViewMap = new google.maps.StreetViewPanorama(StreetViewDiv,
      {visible: false, position: null, enableCloseButton: true});
    
    // handle click on close button  
    google.maps.event.addListener(StreetViewMap, 'closeclick', function(e)
    {
      StreetViewDiv.style.display = "none";
      remove_class(document.documentElement, "streetview_on");
      markerStreetViewLayer.disable();
      mapadiv.style.display = "";
      mapadiv.style.width = "";
      createDefaultMapControls();
      m.syncPort();
    });
    
    // add marker layer
    markerStreetViewLayer = new SMap.Layer.Marker();
    m.addLayer(markerStreetViewLayer);

    // add marker
    StreetViewMarkerDiv = JAK.mel("div", {className: "StreetViewMarker"});
    StreetViewMarker = new SMap.Marker(new SMap.Coords.fromWGS(lat, lon), null, {anchor: {left: 17,top: 19}, url: StreetViewMarkerDiv});
    StreetViewMarker.decorate(SMap.Marker.Feature.Draggable);
    markerStreetViewLayer.addMarker(StreetViewMarker);

    // marker drag --> change position
    m.getSignals().addListener(window, "marker-drag-stop", function(e)
    {
      if (e.target !== StreetViewMarker)
        return;

      StreetViewMap.setPosition(CoordsToGoogle(e.target.getCoords()));
    });
    
    // marker left-click --> change heading
    EventUtil.addHandler(StreetViewMarkerDiv, "click", function(e, target)
    {
      // gecko doesn't work correctly
      if (JAK.Browser.client == "gecko")
        return;
        
      var width = target.offsetWidth;
      var height = target.offsetHeight;
      var x = e.layerX || /* IE7-8 */ e.offsetX;
      var y = e.layerY || /* IE7-8 */ e.offsetY;

      x = x - width / 2;
      y = y - height / 2;
      
      var azimuth = Math.atan(x / y) * 180 / Math.PI;
      
      if (y > 0 && x > 0)
        azimuth = 180 - azimuth;
      else if (y > 0 && x < 0)
        azimuth = 180 + Math.abs(azimuth);
      else if (y < 0 && x > 0)
        azimuth = Math.abs(azimuth);
      else if (y < 0 && x < 0)
        azimuth = 360 - azimuth;       
      
      //log_object(e);
      
      StreetViewMap.setPov({heading: (azimuth) % 360, pitch: 0, zoom: 0});//setPov({heading: azimuth});
    });
    
    // change position - move marker
    google.maps.event.addListener(StreetViewMap, 'position_changed', function(e)
    {
      var coords = new SMap.Coords.fromGoogle(StreetViewMap.getPosition());
      StreetViewMarker.setCoords(coords);
      m.setCenter(coords);
    });
    
    // change orientation - rotate marker
    google.maps.event.addListener(StreetViewMap, 'pov_changed', function(e)
    {
      svRotateElement(StreetViewMarkerDiv, StreetViewMap.getPov().heading - 90);
    });
    
    // add all markers
    StreetViewAddPoints();
    
    // create button to change size of SV from 50% to 100% or vice versa
    svSizeButton = JAK.mel("div", {id: "sv_size_button", className: "button sidebar_button", innerHTML: settings.streetview_width});  
    StreetViewDiv.appendChild(svSizeButton);
    
    EventUtil.addHandler(svSizeButton, "click", function(e, target)
    {
      svResize(true);
    });
    
    // on window resize, adjust SV size
    EventUtil.addHandler(window, "resize", function()
    {
      if (!contains_class(document.documentElement, "streetview_on"))
        return;
    
      StreetViewDiv.style.height = mapadiv.offsetHeight - 2 /* 1px border */ + "px";
      svResize();
    });        
  }
  
  add_class(document.documentElement, "streetview_on");
  StreetViewDiv.style.display = "";
  //StreetViewDiv.style.height = mapadiv.offsetHeight - 2 /* 1px border */ + "px";
  StreetViewMap.setVisible(true);
  StreetViewMap.setPosition(new google.maps.LatLng(lat, lon));
  markerStreetViewLayer.enable();
  svRotateElement(StreetViewMarkerDiv, StreetViewMap.getPov().heading - 90);
  svResize();
}

// load after right click (not the first time, for that use LoadStreetView)
function reloadStreetView(coords)
{
  LoadStreetView(coords);
}

function svResize(advance)
{
  var sizes = {"25%": "50%", "50%": "75%", "75%": "100%", "100%": "25%"};
  var complement_size = {"25%": "75%", "50%": "50%", "75%": "25%", "100%": "0%"};
  
  var size = advance ? sizes[svSizeButton.innerHTML] : settings.streetview_width;
  var map_size = complement_size[size];
  
  // resize divs
  StreetViewDiv.style.width = size;
  mapadiv.style.width = map_size;
  mapadiv.style.display = (size == "100%") ? "none" : "";
  
  // set button to value 
  svSizeButton.innerHTML = size;
  settings.streetview_width = size;
  
  // synchronize google maps  
  google.maps.event.trigger(StreetViewMap, "resize");
  
  // synchronize mapy.cz (skip 100% to fix IE7-8 error)
  if (size !== "100%")
  {
    m.syncPort();
    createDefaultMapControls();
  }
}

function StreetViewInit()
{
  if (!window.StreetViewDiv)
  {
    StreetViewDiv = document.createElement("div");
    StreetViewDiv.id = "StreetViewDiv";
    StreetViewDiv.style.display = "none";
    loadJS("http://maps.googleapis.com/maps/api/js?sensor=false&callback=LoadStreetView");
  }

  mapadiv.parentNode.insertBefore(StreetViewDiv, mapadiv.nextSibling);
  StreetViewDiv.style.height = mapadiv.offsetHeight - 2 /* 1px border */ + "px";

  if (StreetViewMap)
    google.maps.event.trigger(StreetViewMap, "resize");
}

var StreetViewMarkers = [];

function StreetViewAddPoints()
{
  StreetViewDeletePoints();
  
  for (var i in wp)
  {
    var image = null;
    var shadow = null;
    
    // waymarks
    if (wp[i].WaymarkCategoryIcon)
    {
      image = new google.maps.MarkerImage
      (
        wp[i].WaymarkCategoryIcon,
        new google.maps.Size(20, 34),
        new google.maps.Point(0, 0),    //origin
        new google.maps.Point(-4, -4)   //anchor point
      );
      
      // set size of waymark icons
      image.scaledSize = new google.maps.Size(12, 12);
      
      // set background image (generic marker icon)
      shadow = new google.maps.MarkerImage
      (
        basePath + "Images/Markers/WaymarkGeneric.png",
        new google.maps.Size(20, 34),
        new google.maps.Point(0, 0),  //origin
        new google.maps.Point(0, 0)   //anchor point
      );
    }
    
    // everything else
    else
    {
      image = new google.maps.MarkerImage(basePath + "Images/Markers/" + wp[i].CacheType + ".png");    
    }
    
    // create marker  
    var marker = new google.maps.Marker
    ({
      position: new google.maps.LatLng(wp[i].Latitude, wp[i].Longitude),
      map: StreetViewMap,
      icon: image,
      title: wp[i].Name,
      shadow: shadow
    });
    
    StreetViewMarkers.push(marker);
  }
}

function StreetViewDeletePoints()
{
  while(StreetViewMarkers[0])
    StreetViewMarkers.pop().setMap(null);
}

function CoordsToGoogle(coords)
{
  var lat = lat || coords.lat();
  var lon = lon || coords.lon();
  return (new google.maps.LatLng(lat, lon));
}

function svRotateElement(elm, deg)
{
  lastRotation = deg;
  return(rotateElement(elm, deg));
}

function rotateElement(elm, deg)
{
  var degStr = "rotate(" + deg + "deg)";
  var properties = ["WebkitTransform", "MozTransform", "OTransform", "msTransform", "transform"];
  var p;
  
  while (p = properties.shift())
  {
    if (typeof elm.style[p] !== 'undefined')
      elm.style[p] = degStr;  
  }
  
  /* IE7-8 */
  var rad = deg * Math.PI * 2 / 360;
  costheta = Math.cos(rad);
  sintheta = Math.sin(rad);

  var a = parseFloat(costheta).toFixed(8);
  var c = parseFloat(-sintheta).toFixed(8);
  var b = parseFloat(sintheta).toFixed(8);
  var d = parseFloat(costheta).toFixed(8);

  if (typeof elm.style.msTransform === 'undefined' && typeof elm.style.transform === 'undefined')
    elm.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11=" + a + ", M12=" + c + ", M21=" + b + ", M22=" + d + ",sizingMethod='auto expand')";
}

// // <ul class="context_menu" id="contextMenuStreetView" style="display: none">
// //   <li id="cmsv_size_change">aaa</li>
// // </ul>
// 
// var svContextMenuStr = "";
// svContextMenuStr += "<ul class=\"context_menu\" id=\"contextMenuStreetView\" style=\"display: none\">";
// svContextMenuStr += "  <li id=\"cmsv_size_change\">aaa<\/li>";
// svContextMenuStr += "<\/ul>";
// 
// var svContextMenu = document.createElement("div");
// svContextMenu.innerHTML = svContextMenuStr;
// 
// contextMenu.addContextMenu(StreetViewDiv, svContextMenu.firstChild, 
// function (e, target, menu, options)
// {
// },
// 
// function (e, target, menu, options) 
// {
//   if (target.id == "cmsv_size_change")
//     log("a");
// });